// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.asmkey.gui;


import java.io.*;
import java.util.*;

import javax.swing.JOptionPane;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;
import java.awt.event.*;

import de.uka.ilkd.asmkey.logic.*;
import de.uka.ilkd.asmkey.parser.AstParser;
import de.uka.ilkd.asmkey.parser.TreeParser;
import de.uka.ilkd.asmkey.parser.ast.AstDeclaration;
import de.uka.ilkd.asmkey.parser.ast.AstProblem;
import de.uka.ilkd.asmkey.parser.ast.AstImportFile;
import de.uka.ilkd.asmkey.parser.env.*;
import de.uka.ilkd.asmkey.proof.ProofParser;
import de.uka.ilkd.asmkey.rule.*;
import de.uka.ilkd.key.gui.*;
import de.uka.ilkd.key.pp.*;
import de.uka.ilkd.key.java.Services;
import de.uka.ilkd.key.java.abstraction.PrimitiveType;
import de.uka.ilkd.key.java.abstraction.SLListOfType;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.ldt.*;
import de.uka.ilkd.key.logic.op.Op;
import de.uka.ilkd.key.parser.Location;
import de.uka.ilkd.key.parser.ParserException;
import de.uka.ilkd.key.proof.Proof;
import de.uka.ilkd.key.proof.TacletIndex;
import de.uka.ilkd.key.proof.BuiltInRuleIndex;
import de.uka.ilkd.asmkey.proof.decproc.AsmDecisionProcedureTranslationFactory;
import de.uka.ilkd.key.rule.*;

import jargs.gnu.*;


/** This is the main class for starting the standalone prover of
 * ASM-KeY. The class extends the default Main class of the core
 * system.
 *
 * @author Hubert Schmid
 * @author Stanislas Nanchen
 */

public final class Main extends de.uka.ilkd.key.gui.Main {

    protected Main(String title, ProofSettings settings) {
       super(title, settings);
    }

    public static de.uka.ilkd.key.gui.Main getInstance() {
        if (instance == null) instance = 
            new Main("KeY -- Prover", ProofSettings.THE_SETTINGS);
        if (!instance.isVisible()) instance.setVisible(true);
        return instance;
    }

    private static boolean ASMPROVER_MODE = false;
    private static String ASMPROVER_INCOMPLETE_PATH = null;
    private static String ASMPROVER_PROOFS_PATH = null;
    private static String ASMPROVER_NAME = null;

    public static void printUsage() {
	System.out.println("TO REWRITE : Possible parameters are (* = default): ");
	System.out.println("  no_debug        : disables debug mode (*)");
	System.out.println("  debug           : enables debug mode");
	System.out.println("  no_assertion    : disables assertions");
	System.out.println("  assertion       : enables assertions (*)");
	System.out.println("  <filename>      : loads a .key file");
    }

    public static void evaluateOptions(String[] args) {
	// Give back to KeY
	CmdLineParser parser = new CmdLineParser();
	CmdLineParser.Option debug = parser.addBooleanOption('d',"debug");
	CmdLineParser.Option assertion = parser.addBooleanOption('a',"assertion");
	CmdLineParser.Option asmprover = parser.addBooleanOption('p',"asmprover");

	try {
            parser.parse(args);
        }
        catch ( CmdLineParser.OptionException e ) {
            System.err.println(e.getMessage());
	    printUsage();
            System.exit(-1);
        }
	
	// Extract the values entered for the various options -- if the
        // options were not specified, the corresponding values will be
        // null.
        Boolean debugValue = (Boolean)parser.getOptionValue(debug);
        Boolean assertionValue = (Boolean)parser.getOptionValue(assertion);
        Boolean asmproverValue = (Boolean)parser.getOptionValue(asmprover);
 
        /*// For testing purposes, we just print out the option values
        System.out.println("debug: " + debugValue);
	System.out.println("assertion: " + assertionValue);
        System.out.println("asmprover: " + asmproverValue);
	*/

	// Extract the trailing command-line arguments ('a_number') in the
        // usage string above.
        String[] otherArgs = parser.getRemainingArgs();
	if (otherArgs.length == 1) {
	    fileNameOnStartUp = otherArgs[0];
	} else if (otherArgs.length == 0) {
	    fileNameOnStartUp = null;
	} else {
	    printUsage();
	    System.exit(-1);
	}

	if (assertionValue == null) {
	    de.uka.ilkd.key.util.Debug.ENABLE_ASSERTION = true;
	} else {
	    de.uka.ilkd.key.util.Debug.ENABLE_ASSERTION = assertionValue.booleanValue();
	}

	if (debugValue == null) {
	    de.uka.ilkd.key.util.Debug.ENABLE_DEBUG = false;
	} else {
	    de.uka.ilkd.key.util.Debug.ENABLE_DEBUG = debugValue.booleanValue();
	}

	if (asmproverValue == null) {
	    ASMPROVER_MODE = false;
	} else {
	    ASMPROVER_MODE = asmproverValue.booleanValue();
	}
	  
	if(ASMPROVER_MODE) {
	    if (fileNameOnStartUp == null) {
		printUsage();
		System.exit(-1);
	    }
	    generatePathStrings(fileNameOnStartUp);
	    System.out.println("Running in asmprover mode ...");
	} else {
	    System.out.println("Running in key mode ...");
	}

	if (de.uka.ilkd.key.util.Debug.ENABLE_DEBUG) {
	    System.out.println("Running in debug mode ...");
	} else {
	    System.out.println("Running in normal mode ...");
	}
	if (de.uka.ilkd.key.util.Debug.ENABLE_ASSERTION) {
	    System.out.println("Using assertions ...");	   
	} else {
	    System.out.println("Not using assertions ...");	   
	}

    }

    private static void generatePathStrings(String filename) {
	File fileTemp =  new File(fileNameOnStartUp);
	String groundPath, tempString;
	
	ASMPROVER_NAME = fileTemp.getName();
	
	fileTemp = fileTemp.getAbsoluteFile();
	fileTemp = fileTemp.getParentFile();
	fileTemp = fileTemp.getParentFile();
	groundPath = fileTemp.getAbsolutePath() + File.separator;
	tempString = File.separator + ASMPROVER_NAME + ".proof";

	ASMPROVER_PROOFS_PATH = groundPath + "proofs" + tempString; 
	ASMPROVER_INCOMPLETE_PATH = groundPath + "incomplete" + tempString;
    }

    public static void main(String[] args) {
	Main.evaluateOptions(args);
	de.uka.ilkd.key.gui.Main key = getInstance();
	key.loadCommandLineFile();
    }
/*
        evaluateOptions(args);
        Config.DEFAULT.setDefaultFonts();
        Main key = new Main("KeY -- Prover", ProofSettings.THE_SETTINGS);
        KeYMediator mediator = new KeYMediator(key);
        key.setMediator(mediator);
        key.initGoalList();
        key.initGUIProofTree();
        SwingUtilities.updateComponentTreeUI(key);
        key.show();
*/
//    }


    /** Return a collection of all dynamic function symbols contained
     * in the given collection. */
    private static Collection getDynamicFunctions(Collection c) {
        ArrayList list = new ArrayList();
        for (Iterator i = c.iterator(); i.hasNext();) {
            Object f = i.next();
            if (f instanceof DynamicFunction) {
                list.add(f);
            }
        }
        return list;
    }

    /** Load all taclet (.rules) files in the given directory and add
     * their declarations to the given environment. The files are read
     * in sorted order (by name). Only files with suffix '.rules' are
     * read. */
    private static void loadRulesDirectory(DeclarationEnvironment env, File path)
        throws ParserException
    {
        System.out.print("load rules from " + path + ":");
        File[] files = path.listFiles(new FileFilter() {
                public boolean accept(File file) {
                    return file.isFile() && file.getName().endsWith(".rules");
                }
            });
        if (files == null) {
            throw new RuntimeException("problem reading rules");
        }
        Arrays.sort(files, new Comparator() {
                public int compare(Object lhs, Object rhs) {
                    return ((File) lhs).getName().compareTo(((File) rhs).getName());
                }
            });
        for (int i = 0; i < files.length; ++i) {
            try {
                System.out.print(" " + files[i].getName());
                Reader in = new BufferedReader(new FileReader(files[i]));
                AstDeclaration[] decls = AstParser.parseDeclarations(in);
                TreeParser.parseDeclarations(env, decls);
                in.close();
            } catch (IOException ioe) {
                ioe.printStackTrace(System.err);
            }
        }
        System.out.println();
    }

    /** Load all taclets and add their declartions to the given
     * environment. The taclets are searched in the directory
     * specified by the system property 'asmkey.rules.path'. */
    private static void loadRules(DeclarationEnvironment env)
        throws ParserException
    {
        String path = System.getProperty("asmkey.rules.path");
        if (path == null) {
            path = System.getProperty("asmkey.base.path", System.getProperty("user.dir", "."))
                + "/rules";
        }
        loadRulesDirectory(env, new File(path));
    }

    /** Create an Integer LDT. */
    private static IntegerLDT createIntegerLDT(DefaultEnvironment env) {
        IntegerLDT intLdt = new IntegerLDT();
        Namespace ns = new Namespace();
        try {
            ns.add(env.getOperator("Z"));
            ns.add(env.getOperator("#"));
            ns.add(env.getOperator("neg"));
            for (char ch = '0'; ch <= '9'; ++ch) {
                ns.add(env.getOperator("" + ch));
            }
            intLdt.init(ns, env.getSort("int"),
                        SLListOfType.EMPTY_LIST.append(PrimitiveType.JAVA_INT));
        } catch (EnvironmentException ee) {
            ee.printStackTrace(System.err);
            System.exit(1);
        }
        return intLdt;
    }


    /* Overwrite method from super class. */
    protected void loadProblem(File file) {
        try {
            TacletIndex rules = new TacletIndex();
            /* meta ops */
            MetaJoin join = new MetaJoin();
            MetaEquiImp equiImp = new MetaEquiImp();
            MetaEquiS equiS = new MetaEquiS();
	    MetaBigOperator big = new MetaBigOperator();
            /* create environment with list of meta operators */
            Object[] metaOps = new Object[] {
                join,
                equiImp,
                equiS,
		big,
                new MetaOpEqualArgs(),
                new MetaNamedRule(),
		new MetaAccTArgs(),
		new MetaSCElimination(),
		new MetaLogicalCondition(),
                Op.META_ADD,
                Op.META_SUB,
                Op.META_MUL,
                Op.META_DIV,
                Op.META_MOD,
                Op.META_LESS,
                Op.META_GREATER,
                Op.META_LEQ,
                Op.META_GEQ,
                Op.META_EQ,
            };
            DefaultEnvironment env = new DefaultEnvironment(Arrays.asList(metaOps));
            /* load default rules */
            loadRules(env);
	    /* load importdecls declarations */
	    AstImportFile[] imfs = AstParser.parseImportDecls(new FileReader(file));
	    File package_file;
	    for (int i = 0; i<imfs.length; i++) {
		package_file = new File(imfs[i].getPath());
		TreeParser.parseDeclarations(env, AstParser.parsePackageDeclarations(new FileReader(package_file)));
	    }
	    /* load importlemmas lemmas*/
	    imfs = AstParser.parseImportLemmas(new FileReader(file));
	    for (int i = 0; i<imfs.length; i++) {
		package_file = new File(imfs[i].getPath());
		TreeParser.parseDeclarations(env, AstParser.parsePackageLemmas(new FileReader(package_file)));
	    }
            /* load problem */
            AstProblem problem = AstParser.parseProblem(new FileReader(file));
            TreeParser.parseDeclarations(env, problem.getDeclarations());
            /* set dynamic functions on meta operators */
            Collection dynamicFunctions = getDynamicFunctions(env.getOperators());
            join.setDynamicFunctions(dynamicFunctions);
            equiImp.setDynamicFunctions(dynamicFunctions);
            equiS.setDynamicFunctions(dynamicFunctions);
	    big.setDynamicFunctions(dynamicFunctions);
            /* parse the formula to be proven. */
            Term formula = TreeParser.parseTerm(env, problem.getFormula());

	    /* statical analysis */

	    StaticAnalysis analyse = new StaticAnalysis(env);

	    analyse.analyseEnv();
	    analyse.analyse(formula);
	    
            /* taclets */
            for (Iterator i = env.getTaclets().iterator(); i.hasNext();) {
                Taclet taclet = (Taclet) i.next();
                rules.add(taclet);
            }
	    BigOperatorRule bigRule = new BigOperatorRule();
	    bigRule.setDynamicFunctions(dynamicFunctions);
	    ListOfBuiltInRule lemmaRules = createLemmaRuleList(env.getLemmasMap());
	    ListOfBuiltInRule builtInRules =
                SLListOfBuiltInRule.EMPTY_LIST.prepend
		(new SimplifyIntegerRule
		 (new AsmDecisionProcedureTranslationFactory())).
		prepend(new StepRule()).
		prepend(lemmaRules).
		prepend(bigRule).
		prepend(new FunctionOperatorRule());
	    /* add all lemmas to the menubar. for the user to choose */
	    buildLemmaMenu(lemmaRules);

	    /* if we are in asmprover mode, put the new file menu */
	    if (ASMPROVER_MODE) {buildAsmProverFileMenu();}

            Proof p = new Proof("User defined problem", formula, rules, 
				new BuiltInRuleIndex(builtInRules));
            p.setNamespaces(env.getEnvAsNamespaceSet());
            p.setJavaServices(new Services());
            addProblem(new de.uka.ilkd.key.proof.ProofList(new Proof[]{p}, 
							   "ASMKeY PO"));

            ProcedurePool.put(mediator.getJavaInfo().getServices(), env.getProcedures());

            PresentationFeatures.initialize(mediator.func_ns(),
                                            mediator.getNotationInfo(),
                                            mediator.getSelectedProof());

            /* int integer LDT by hand. hack... is saved in settings... */
            IntegerLDT intLdt = createIntegerLDT(env);
	    BooleanLDT boolLdt = new BooleanLDT();
	    LDTSettings settings = new LDTSettings();
            mediator.getJavaInfo().getServices().getTypeConverter().init(intLdt,
									 boolLdt);
	    settings.setNaturalNumberLDT(intLdt);
	    settings.setBooleanLDT(boolLdt);
	    ProofSettings.THE_SETTINGS.setLDTSettings(settings);

            /* load the proof, if it exists. */
            if (problem.getProof() != null) {
                ProofLoader proofLoader = new ProofLoader(mediator);
                proofLoader.loadProof(ProofParser.parse(problem.getProof()));
            }
        } catch (IOException ioe) {
            JOptionPane.showMessageDialog(this,
                                          "Could not load Problem due "
                                          + "to an IO error. The file to "
                                          + "load does not exist or you do "
                                          + "not have the permission to"
                                          + " read it.\n"
                                          + ioe.getMessage(),
                                          "IO Error",
                                          JOptionPane.ERROR_MESSAGE);
	    if(ASMPROVER_MODE) {System.exit(-1);}
        } catch (ParserException pe) {
            String message = pe.getMessage();
            Location loc = pe.getLocation();
            if (loc != null) {
                message += " in ";
                if (loc.getFilename() != null) {
                    message += loc.getFilename() + ", ";
                }
                message += "line " + loc.getLine() + ", column " + loc.getColumn();
            }
            JOptionPane.showMessageDialog(this,
                                          "Could not load Problem.\n" + message,
                                          "Error",
                                          JOptionPane.ERROR_MESSAGE);
            pe.printStackTrace(System.err);
	    if(ASMPROVER_MODE) {System.exit(-1);}
        } catch (Exception e) {
            /* e.g. IllegalArgumentException in Term.checked(). */
            e.printStackTrace();
            JOptionPane.showMessageDialog(this,
                                          "Could not load problem/proof.\n" + "Cause: \n" + e.getMessage(),
                                          "Error",
                                          JOptionPane.ERROR_MESSAGE);
	    if(ASMPROVER_MODE) {System.exit(-1);}
        }
    }

    
    /** The lemma menu which permits to
     * invoke lemmas or to add new ones
     */
    private LemmaMenu lemmaMenu;

    private void buildLemmaMenu(ListOfBuiltInRule lemmaRules) {
	JMenuBar menubar = getJMenuBar();

	/* remove the old Lemma Menu */
	if(lemmaMenu != null) {
	    menubar.remove(lemmaMenu);
	}

	lemmaMenu = new LemmaMenu(mediator, lemmaRules);
	menubar.add(lemmaMenu, menubar.getMenuCount()-2);
	
    }

    private ListOfBuiltInRule createLemmaRuleList(Map lemmas) {
	ListOfBuiltInRule result = SLListOfBuiltInRule.EMPTY_LIST;
	String[] lemmasNames = (String[]) lemmas.keySet().toArray(new String[lemmas.size()]);
	Arrays.sort(lemmasNames);
	for (int i=0; i<lemmasNames.length; i++) {
	    result = result.prepend(new LemmaRule((AsmLemma)lemmas.get(lemmasNames[lemmasNames.length-i-1])));
	}

	return result;
    }


    private void buildAsmProverFileMenu() {
	JMenuBar menubar = getJMenuBar();

	menubar.remove(0);
	menubar.add(generateAsmProverFileMenu(), 0);
    }
    

    protected void saveProverModeProof() {
	String filename;
	if (mediator.getProof().closed()) {
	    filename = ASMPROVER_PROOFS_PATH;
	} else {
	    filename = ASMPROVER_INCOMPLETE_PATH;
	}

	System.out.println(filename);
	saveProof(new File(filename));
	if (mediator.getProof().closed()) {
	    File file = new File(ASMPROVER_INCOMPLETE_PATH);
	    file.delete();
	}
    }
   

    /** The generation of the asmprover file
     * menu, which is limited.
     */
    private JMenu generateAsmProverFileMenu() {
	JMenu fileMenu = new JMenu("File");
	JMenuItem save = new JMenuItem("Save");
	save.setAccelerator(KeyStroke.getKeyStroke
			    (KeyEvent.VK_S, ActionEvent.CTRL_MASK));
	save.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    saveProverModeProof();
		}});

	JMenuItem quit = new JMenuItem("Quit");
	quit.setAccelerator(KeyStroke.getKeyStroke
			    (KeyEvent.VK_Q, ActionEvent.CTRL_MASK));
	quit.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    saveProverModeProof();
		    exitMain();		    
		}});
	
	registerAtMenu(fileMenu, save);
	registerAtMenu(fileMenu, quit);

	return fileMenu;
    }

}
