// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.asmkey.logic;

import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.TermFactory;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.logic.op.Operator2;
import de.uka.ilkd.key.logic.op.SchemaVariable;
import de.uka.ilkd.asmkey.logic.CollectionSort;
import de.uka.ilkd.key.logic.sort.*;

/** This class represents the bigand and bigor operators
 * for expressing a property over all function, dynamic or
 * not and with access or update properties.
 *
 * @author Stanislas Nanchen 
 */

public abstract class BigOperator extends AsmOperator {

    public static final Operator2 BIGAND = 
	new BigOperator("bigand", Sort.FORMULA, new Sort[] {Sort.FORMULA}) {
	    public Term join(Term term1, Term term2) {
		return tf.createJunctorTerm(Op.AND, term1, term2);
	    }
	};

    public static final Operator2 BIGOR =
	new BigOperator("bigor", Sort.FORMULA, new Sort[] {Sort.FORMULA}) {
	    public Term join(Term term1, Term term2) {
		return tf.createJunctorTerm(Op.OR, term1, term2);
	    }
	};

    /** Short refernce to TermFactory. */
    private static final TermFactory tf = TermFactory.DEFAULT; 

    /*private Name id;
    private int type;
    private int property;
    private Name[] rules;*/

    public static Operator2 bigAnd(final Name id,
				   final int type_flag,
				   final int prop_flag,
				   final Name[] rules) {
	return new BigOperator("bigand", Sort.FORMULA, new Sort[] {Sort.FORMULA}) {
		
		public Term join(Term term1, Term term2) {
		    return tf.createJunctorTerm(Op.AND, term1, term2);
		}

		/** the name used for the NArityLogicVariable */
		public Name getVariableName() {
		    return new Name("%" + id.toString());
		}

	    };
    }

    public static Operator2 bigOr(final Name id,
				  final int type_flag,
				  final int prop_flag,
				  final Name[] rules) {
	return new BigOperator("bigor", Sort.FORMULA, new Sort[] {Sort.FORMULA}){
		
		public Term join(Term term1, Term term2) {
		    return tf.createJunctorTerm(Op.OR, term1, term2);
		}

		/** the name used for the NArityLogicVariable */
		public Name getVariableName() {
		    return new Name("%" + id.toString());
		}
		
	    };
    }

    private BigOperator(String name, Sort sort, Sort[] args) {
	super(name, sort, args);
    }

    /*private BigOperator(String name, Sort sort, Sort[] args,
		       Name id, int type_flag, int prop_flag, Name[] rules) {
	this(name, sort, args, null, id, type_flag, prop_flag, rules);
    }

    private BigOperator(String name, Sort sort, Sort[] args, boolean[] boundPositions,
		       Name id, int type_flag, int prop_flag, Name[] rules) {
	super(name, sort, args, boundPositions);
	this.id = id;
	this.type = type_flag;
	this.property = prop_flag;
	this.rules = rules;
	}*/

    /** Used for computing the result of the metaoperator {@link MetaBigOperator}*/
    public abstract Term join(Term term1, Term term2);

    /** Used for filtering the collection of functions */
    public Collection filter(Collection functions) {
	LinkedList list = new LinkedList();
	Iterator it = functions.iterator();
	while (it.hasNext()) {
	    Function f = (Function) it.next();
	    if (f instanceof DynamicFunction) {
		list.add(f);
	    }
	}
	return list;
    }

    /** the name used for the NArityLogicVariable */
    public Name getVariableName() {
	return new Name("%");
    }
}
