// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.asmkey.logic;

import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.TermFactory;
import de.uka.ilkd.key.logic.op.Op;
import de.uka.ilkd.key.logic.op.Operator;
import de.uka.ilkd.key.logic.op.Junctor;
import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.rule.inst.SVInstantiations;


/** Meta operator replace the short-circuit
 * operators by their logical conterpart.
 *
 * @author Hubert Schmid
 * @author Stanislas Nanchen
 */

public final class MetaLogicalCondition extends AsmMetaOperator {

    /** Short reference to AsmTermFactory. */
    private static final AsmTermFactory tf = AsmTermFactory.ASMDEFAULT;

    /** creates a new meta operator. Only one instance is required, so
     * this class may be a singleton. */
    public MetaLogicalCondition() {
        super("#LOGICAL", Sort.FORMULA,
              new Sort[] { Sort.FORMULA });
    }


    /** Expects a quantifier free pure formula (condition)
     */
    public Term calculate(Term term, SVInstantiations svInst) {
	
	return convert(term.sub(0));
    }


    /** Convert recursively
     */

    private Term convert(Term term) {
	Operator op;
	
	op = term.op();
	System.out.println("converting : " + term.toString() + " : " + op.toString());
	if (op instanceof AsmJunctor) {
	    Junctor newop;
	    if (op == AsmJunctor.ORELSE) {
		newop = Op.OR;
	    } else {
		newop = Op.AND;
	    }
	    return convert_subterms(term, newop);
	} else if (op instanceof Junctor) {
	    return convert_subterms(term, (Junctor) term.op());
	} else {
	    return term;
	}
    }

    private Term convert_subterms(Term term, Junctor op) {
	Term[] converted;

	converted = new Term[op.arity()];

	for (int i =0; i<op.arity(); i++) {
	    converted[i]=convert(term.sub(i));
	}

	return TermFactory.DEFAULT.createJunctorTerm(op, converted);
    }
}
