// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.asmkey.logic;

import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.asmkey.parser.env.DefaultEnvironment;
import de.uka.ilkd.asmkey.logic.*;
import java.util.Collections;
import java.util.List;
import java.util.Iterator;
import de.uka.ilkd.key.util.Debug;

/** This class is the entry point for the static analysis of
 * the asms: it performs:
 * - determination of : recursive rules, hierarchical rules
 *                    : hierachical part (for ndASM) (TODO).
 * - determination of : update and access info.
 *
 * @author Stanislas Nanchen
 */

public class StaticAnalysis {

    private DefaultEnvironment env;
    private HashMapFromAsmProcedureToAsmRule procs;

    public StaticAnalysis(DefaultEnvironment env) {
	this.env = env;
	procs = new HashMapFromAsmProcedureToAsmRule();
	prepareProcs();
    }

    private void prepareProcs() {
	List namedRules = env.getProcedures();
	Iterator it = namedRules.iterator();
	AsmProcedure namedRule;
	AsmRule proc;

	while(it.hasNext()) {
	    namedRule = (AsmProcedure) it.next();
	    proc = (AsmRule) AsmOperator.PROC.createTerm(null, new Term[] { namedRule.getBody() });
	    procs.put(namedRule, proc);
	}
    }

    public void analyseEnv() {

	try {
	    SCC recAnalysis = new SCC(env, procs);
	    recAnalysis.doAnalysis();
	    
	    UpdateDFInfo updAnalysis = new UpdateDFInfo(env, procs);
	    updAnalysis.doAnalysis();
	} catch (Exception e) {
	    // rethraw as parser mistake.
	    Debug.out("Exception thrown by class StaticAnalysis at doAnalysis()");
	}
    }

    public void analyse(Term term) {}
}
