// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.gui;

import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.*;

import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.java.abstraction.*;
import de.uka.ilkd.key.java.declaration.MethodDeclaration;
import de.uka.ilkd.key.jml.*;
import de.uka.ilkd.key.parser.jml.*;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.proof.init.*;


public class JMLSpecBrowser extends JDialog {

    /** the package hierarchy as a tree */
    private JTree packageHierarchy;

    private JList methodList; 
    private JList poList;

    private JCheckBox allInv;
    private JCheckBox invPost;

    /** true if the JMLSpecBrowser was invoked by Main and we already have
     * a proofenvironment for the javaModel we are dealing with.
     */
    private boolean envMode = true;

    private KeYMediator mediator;
    private Services services;

    private JMLProofOblInput poi = null;

    private String javaPath = "";

    private Object[] emptyArray = new Object[0];

    private TreeModel treeModel;

    //the selected class
    private KeYJavaType classType = null;

    public JMLSpecBrowser(KeYMediator mediator) {  
	super(new JFrame(), "JML Specification Browser", true);
       	this.services = mediator.getServices();
	this.mediator = mediator;
	this.javaPath = mediator.getProof().env().getJavaModel().getModelDir();
	buildTreeModel();
	layoutJMLSpecBrowser();
	pack();
	setLocation(70, 70);
	setVisible(true);
    }

    public JMLSpecBrowser(Services services, String javaPath) {  
	super(new JFrame(), "JML Specification Browser", true);
       	this.services = services;
	this.javaPath = javaPath;
	envMode = false;
	buildTreeModel();
	layoutJMLSpecBrowser();
	pack();
	setLocation(70, 70);
	setVisible(true);
    }

    /**
     * builds a TreeModel from the package structure of the Java model.
     */
    private void buildTreeModel(){
	JavaInfo ji = services.getJavaInfo();
	HashMap package2Node = new HashMap();
	DefaultMutableTreeNode root = new DefaultMutableTreeNode("");
	Implementation2SpecMap ism = services.getImplementation2SpecMap();
//	ism.checkPurity();
	Iterator it = ism.getAllClasses().iterator();
	while(it.hasNext()){
	    KeYJavaType kjt = (KeYJavaType) it.next();
	    String name = ji.getFullName(kjt);
	    if(name.lastIndexOf(".") != -1){
		DefaultMutableTreeNode parent = 
		    treeHelper(name.substring(0, name.lastIndexOf(".")),
			       package2Node, root);
		parent.add(new DefaultMutableTreeNode(kjt){
		    public String toString(){
			String name = ((KeYJavaType) userObject).getSort().
			    toString();
			return name.substring(name.lastIndexOf(".")+1); 
		    }
		});
	    }else{
		root.add(new DefaultMutableTreeNode(kjt){
			public String toString(){
			    return ((KeYJavaType) userObject).getSort().
				toString();
			}
		});
	    }
	}
	if(root.getChildCount() == 1){
	    treeModel = new DefaultTreeModel(root.getFirstChild());
	}else{
	    treeModel = new DefaultTreeModel(root);
	}
	packageHierarchy = new JTree(treeModel);
	packageHierarchy.addTreeSelectionListener(
	    new GUIPackageTreeSelectionListener());
    }

    private DefaultMutableTreeNode treeHelper(String pname, HashMap p2n, 
					      DefaultMutableTreeNode root){
	if(p2n.get(pname) != null){
	    return (DefaultMutableTreeNode) p2n.get(pname);
	}else{
	    final int dot = pname.lastIndexOf(".");
	    DefaultMutableTreeNode node = 
		new DefaultMutableTreeNode(pname){
		    public String toString(){
			return userObject.toString().substring(dot+1);
		    }
		};
	    p2n.put(pname, node);
	    if(dot != -1){
		DefaultMutableTreeNode parent = 
		    treeHelper(pname.substring(0, dot),
			       p2n, root);
		parent.add(node);
	    }else{
		root.add(node);
	    }
	    return node;
	}
    }

    /** layout */
    protected void layoutJMLSpecBrowser() {
	JScrollPane classListScroll = new
	    JScrollPane(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, 
			JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	classListScroll.getViewport().setView(packageHierarchy);
	classListScroll.setBorder(new TitledBorder("Classes"));

	methodList = new JList();
	methodList.addListSelectionListener(new ListSelectionListener() {
		public void valueChanged(ListSelectionEvent e) {
		    setPOList();				
		}});
	methodList.setCellRenderer(new DefaultListCellRenderer(){
		public Component getListCellRendererComponent(
		    JList list,
		    Object value,
		    int index,
		    boolean isSelected,
		    boolean cellHasFocus)
		    {
			if(value != null){
			    MethodDeclaration md = (MethodDeclaration) value;
			    setText((md.getTypeReference() == null ? "void" : 
					md.getTypeReference().getName())+
				    " "+md.getFullName());
			    if (isSelected) {
				setBackground(list.getSelectionBackground());
				setForeground(list.getSelectionForeground());
			    }
			    else {
				setBackground(list.getBackground());
				setForeground(list.getForeground());
			    }
			    setEnabled(list.isEnabled());
			    setFont(list.getFont());
			    setOpaque(true);
			}
			return this;
		    }
		    });
	JScrollPane methodListScroll = new
	    JScrollPane(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, 
			JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	methodListScroll.getViewport().setView(methodList);
	methodListScroll.setBorder(new TitledBorder("Methods"));

	poList = new JList();
	
	poList.setCellRenderer(new TextAreaRenderer());
	JScrollPane poListScroll = new
	    JScrollPane(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, 
			JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	poListScroll.getViewport().setView(poList);
	poListScroll.setBorder(new TitledBorder("Proof Obligations"));

	JPanel buttonPanel = new JPanel();

	JPanel optionPanel = new JPanel();
	optionPanel.setLayout(new BoxLayout(optionPanel, BoxLayout.Y_AXIS));
	buttonPanel.add(optionPanel);

	allInv = new JCheckBox("Use all applicable invariants");
	allInv.setToolTipText("<html><pre>"+
			      "If selected, the POs change to that effect,<br>"+
			      "that every invariant of every class is added\n"+
			      "to the precondition in the case of method\n"+
			      "specification POs, or to the pre- and\n"+
			      "postcondition in the case of invariant POs.\n "+
			      "  This changes the semantics of the invariant\n"+
			      "PO to that extend that it now expresses that\n"+
			      "the method preserves every invariant of every"+
			      "\nexisting object and type.\n"+
			      "  The effect on method spec POs is that they\n"+
			      "are then reflecting the fact that a method\n"+
			      "can assume all invariants of every existing\n"+
			      "object and type to hold in its prestate.\n"+
			      "Assuming all invariants to hold in the\n"+
			      "prestate of a certain method is often not\n"+
			      "necessary when proving its specification!"+
			      "</pre>");
	optionPanel.add(allInv);
	allInv.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    setPOList();
		}
	    });

	invPost = new JCheckBox("Add invariants to postcondition");
	invPost.setToolTipText("<html><pre>"+
			       "If selected the history constraints and\n"+
			       "invariants of the current class are added\n"+
			       "to the postcondition of the method speccase PO\n"+
			       "(this is the case if \"Use all applicable\n"+
			       "invariants\" is not selected otherwise also\n"+
			       "the invariants of every other class are added)"+
			       "</pre>");
	optionPanel.add(invPost);

	JButton lo = 
	    new JButton("Load Proof Obligation");
	lo.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    if(poList.getSelectedValue() == null){
			JOptionPane.showMessageDialog(
			    null, "Please select the PO you wish to load!",
			    "Which Proof Obligation?", 
			    JOptionPane.ERROR_MESSAGE);
		    }else{
			createProofObl();
			setVisible(false);
			dispose();
		    }
		}
	    });
	buttonPanel.add(lo);
	JButton cancel = new JButton("Cancel");
	cancel.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    setVisible(false);
		    dispose();
		}
	    });
  	buttonPanel.add(cancel);

	java.awt.Dimension paneDim = new java.awt.Dimension(220, 400);
	classListScroll.setPreferredSize(paneDim);
        classListScroll.setMinimumSize(paneDim);
	methodListScroll.setMinimumSize(paneDim);
	methodListScroll.setPreferredSize(paneDim);
	paneDim = new java.awt.Dimension(370, 400);
	poListScroll.setMinimumSize(paneDim);
	poListScroll.setPreferredSize(paneDim);

	JPanel listPanel=new JPanel();
	listPanel.setLayout(new BoxLayout(listPanel, BoxLayout.X_AXIS));
	listPanel.add(classListScroll);
	listPanel.add(methodListScroll);
	listPanel.add(poListScroll);

	getContentPane().setLayout(new BoxLayout(getContentPane(), 
						 BoxLayout.Y_AXIS));
	getContentPane().add(listPanel);
	getContentPane().add(buttonPanel);
    }

    /**
     * Sets a new proof obligation list (when the selected method has changed
     * for example) 
     */
    private void setPOList(){
	Method m = (MethodDeclaration) methodList.getSelectedValue();
	poList.setListData(emptyArray);
	if(m != null){
	    SetOfJMLMethodSpec specs = 
		services.getImplementation2SpecMap().getSpecsForMethod(
		    (MethodDeclaration) m);
	    JMLClassSpec cSpec = services.getImplementation2SpecMap().
		getSpecForClass(classType);
	    if(specs != null){
		specs = specs.union(services.getImplementation2SpecMap().
				    getInheritedMethodSpecs(
					(MethodDeclaration) m, classType));
	    }else{
		specs = services.getImplementation2SpecMap().
		    getInheritedMethodSpecs((MethodDeclaration) m, classType);
	    }
	    if(specs != SetAsListOfJMLMethodSpec.EMPTY_SET || cSpec != null){
		Vector specVector = new Vector();
		if(specs != null){
		    IteratorOfJMLMethodSpec it = specs.iterator();
		    while(it.hasNext()){
			JMLMethodSpec mSpec = it.next();
			specVector.add(mSpec);
			if(mSpec.replaceModelFieldsInAssignable() != null){
			    specVector.add(new AssignableCheckProofOblInput(
					       mSpec, javaPath));
			}
		    }
		}
		if(cSpec != null && (cSpec.containsInvOrConst() || 
				     allInv.isSelected())){
		    specVector.add(cSpec);
		}
		poList.setListData(specVector);
	    }
	}
    }

    /**
     * creates a JMLProofOblInput and starts the proofer.
     */
    private void createProofObl(){
	Object spec = poList.getSelectedValue();
	if(spec instanceof JMLMethodSpec){
	    if(invPost.isSelected()){
		poi = new JMLPostAndInvPO((JMLMethodSpec) spec, javaPath, 
				    allInv.isSelected());
	    }else{
		poi = new JMLPostPO((JMLMethodSpec) spec, javaPath, 
				    allInv.isSelected());
	    }
	}else if(spec instanceof JMLClassSpec){
	    poi = new JMLInvPO((JMLClassSpec) spec,
			       (MethodDeclaration) 
			       methodList.getSelectedValue(),
			       javaPath, allInv.isSelected());
	}else if(spec instanceof AssignableCheckProofOblInput){
	    poi = (AssignableCheckProofOblInput) spec;
	    ((AssignableCheckProofOblInput) poi).setAllInvariants(
		allInv.isSelected());
	}	
	if(envMode){
	    ProblemInitializer.startProver(poi, mediator.getProof().env());
	}
    }

    public JMLProofOblInput getPOI(){
	return poi;
    }

    class GUIPackageTreeSelectionListener implements TreeSelectionListener,
                                              java.io.Serializable {
	/**
	 * Sets a new methodlist if the selection in the package tree has
	 * changed and the selected value is a class.
	 */
	public void valueChanged(TreeSelectionEvent e) {
	    Object o = 
		((DefaultMutableTreeNode) e.getPath().getLastPathComponent()).
		getUserObject();
	    if(o instanceof KeYJavaType){
		classType = (KeYJavaType) o;
		ListOfProgramMethod l = 
		    services.getJavaInfo().getAllProgramMethods(
			classType);
		Vector mVector = new Vector();
		IteratorOfProgramMethod it = l.iterator();
		while(it.hasNext()){
		    MethodDeclaration md = it.next().getMethodDeclaration();
		    if(!md.getName().startsWith("<") ||
		       md.getName().equals("<clinit>")){
			mVector.add(md);
		    }
		}
		ListOfMethod ml = services.getJavaInfo().
		    getKeYProgModelInfo().getConstructors(classType);
		IteratorOfMethod mit = ml.iterator();
		while(mit.hasNext()){
		    mVector.add(mit.next());
		}
		methodList.setListData(mVector);
	    }
	}
    }

    class TextAreaRenderer extends JTextArea implements ListCellRenderer
    {

	private Color getColor(Object o){
	    if(o instanceof JMLNormalMethodSpec){
		return new Color(230, 255, 230);
	    }else if(o instanceof JMLExceptionalMethodSpec){
		return new Color(255, 210, 210);
	    }else if(o instanceof JMLMethodSpec){
		return Color.LIGHT_GRAY;
	    }else if(o instanceof AssignableCheckProofOblInput){
		return new Color(255, 255, 190);
	    }else{
		return new Color(230, 230, 255);
	    }	   
	}

        public TextAreaRenderer()
        {
	    setLineWrap(false);
	    setWrapStyleWord(false);
        }


        public Component getListCellRendererComponent(JList list, Object value,
            int index, boolean isSelected, boolean cellHasFocus)
        {
            setText(value.toString());
            setBackground(isSelected ? Color.BLACK : getColor(value));
            setForeground(isSelected ? Color.WHITE : null);
            return this;
        }
    }


}
