// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.gui;

import java.util.List;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import de.uka.ilkd.key.proof.mgt.*;
import de.uka.ilkd.key.proof.*;
import de.uka.ilkd.key.proof.init.*;

public class UsedMethodContractsList extends JFrame{

    private BasicTask invokedNode;
    private KeYMediator mediator;
    private MethodContractList mCL;


    public UsedMethodContractsList(BasicTask invokedNode, 
				   KeYMediator med) {
	super("Used Specifications");
	this.invokedNode=invokedNode;
	this.mediator=med;
	mediator.requestModalAccess(this);
	List allMC = invokedNode.getAllMethodContracts();		
	mCL = new MethodContractList(allMC, mediator);		
	JScrollPane scroll = new JScrollPane();
	scroll.setViewportView(mCL);
	scroll.setPreferredSize(new java.awt.Dimension(400,500));	
	getContentPane().setLayout(new BorderLayout());
	getContentPane().add(scroll, BorderLayout.CENTER);
	JButton cancel = new JButton("Cancel");
	cancel.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    mediator.freeModalAccess(this);
		    ((JFrame)((JButton)e.getSource())
		     .getTopLevelAncestor()).dispose();
		}});
	JButton jump = new JButton("Go To Proof");
	jump.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    mediator.freeModalAccess(this);
		    ((JFrame)((JButton)e.getSource())
		     .getTopLevelAncestor()).dispose();
		    openTaskForCurrentContract();
		}});
	JButton open = new JButton("Start New Proof");
	open.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    mediator.freeModalAccess(this);
		    ((JFrame)((JButton)e.getSource())
		     .getTopLevelAncestor()).dispose();			   
		    Runnable proofLoader = new Runnable() {
			    public void run() {
				openNewProofForCurrentContract();
			    }
			};
		    KeYMediator.invokeAndWait(proofLoader);
		}});
	

	JPanel p = new JPanel(new GridBagLayout());
	GridBagConstraints c = new GridBagConstraints();
		
	c.insets = new Insets(5,20,5,20);
	c.gridx = 0;
	p.add(jump, c);    
		
	c.gridx = 1;
	p.add(open, c);    

	c.gridx = 2;
	p.add(cancel, c);    
	p.setAlignmentY(JPanel.BOTTOM_ALIGNMENT);
		
	getContentPane().add(p, BorderLayout.SOUTH);
	getContentPane().add(new JLabel("Used specifications:"), 
				BorderLayout.NORTH);
	pack();
	getContentPane().add(scroll);
    } 




    private void openTaskForProof(de.uka.ilkd.key.proof.ProofList p) {
	mediator.setProof(p.get(0));
    }


    public void openTaskForCurrentContract() {
	de.uka.ilkd.key.proof.ProofList p = mCL.getProofForCurrentContract();
	if (p != null) {
	    openTaskForProof(p);
	}
    }

    public void openNewProofForCurrentContract() {
	Contract mC = mCL.getCurrentContract();
	if (mC==null) return;
	ProofOblInput poin = mC.getProofOblInput();
	if (poin==null) return; 
	ProblemInitializer.startProver(poin, 
				       invokedNode.proof().env());
    }

}
