// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.java.annotation;

import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.ArrayOfTerm;

public class LoopInvariantAnnotation extends TermContainerAnnotation{

    private final Term invariant;
    private final Term variant;
    private final ArrayOfTerm assignable;

    public LoopInvariantAnnotation(Term invariant, ArrayOfTerm assignable, 
				   Term variant){
	this.invariant = invariant;
	this.variant = variant;
	this.assignable = assignable;
    }    

    public LoopInvariantAnnotation(Term invariant, ArrayOfTerm assignable){
	this(invariant, assignable, null);
    } 

    public int getTermCount(){
	int count = assignable.size();
	if(invariant != null) count++;
	if(variant != null) count++;
	return count;
    }

    public ArrayOfTerm getArrayOfTerm(){
	Term[] ta = new Term[getTermCount()];
	int i = 0;
	if(invariant != null){
	    ta[i++] = invariant;
	}
	if(variant != null){
	    ta[i++] = variant;
	}
	for(int j = 0; i<ta.length; i++){
	    ta[i] = assignable.getTerm(j++);
	}
	return new ArrayOfTerm(ta);
    }

    public TermContainerAnnotation create(TermContainerAnnotation copy, 
					  ArrayOfTerm aot){
	LoopInvariantAnnotation tca = (LoopInvariantAnnotation) copy;
	Term inv = null;
	Term var = null;
	int i = 0;
	if(tca.invariant() != null){
	    inv = aot.getTerm(i++);
	}
	if(tca.variant() != null){
	    var = aot.getTerm(i++);
	}
	Term[] terms = new Term[aot.size()-i];
	for(int j = i; j-i < terms.length; j++){
	    terms[j-i] = aot.getTerm(j);
	}
	return new LoopInvariantAnnotation(inv, new ArrayOfTerm(terms), var);
    }

    public Term invariant(){
	return invariant;
    }

    public Term variant(){
	return variant;
    }

    public ArrayOfTerm assignable(){
	return assignable;
    }

    public String toString(){
	String result = "invariant: "+invariant+"\nvariant: "+variant+
	    "\nassignable: ";
	for(int i=0; i<assignable.size(); i++){
	    result += " "+assignable.getTerm(i);
	}
	return result;
    }

}
