// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
package de.uka.ilkd.key.java.recoderext;

import de.uka.ilkd.key.util.Debug;
import recoder.*;
import recoder.abstraction.*;
import recoder.java.*;
import recoder.java.declaration.*;
import recoder.java.declaration.modifier.*;
import recoder.java.expression.operator.*;
import recoder.java.expression.literal.*;
import recoder.java.reference.*;
import recoder.java.statement.*;
import recoder.kit.TwoPassTransformation;
import recoder.list.ClassTypeList;
import recoder.list.FieldList;

import recoder.list.CompilationUnitMutableList;
import recoder.list.FieldSpecificationList;
import recoder.list.MemberDeclarationMutableList;
import recoder.list.ModifierMutableList;
import recoder.list.ModifierArrayList;
import recoder.list.ParameterDeclarationMutableList;
import recoder.list.ParameterDeclarationArrayList;
import recoder.list.StatementArrayList;
import recoder.list.StatementMutableList;
import java.util.*;

/**
 */
public class JVMIsTransientMethodBuilder extends RecoderModelTransformer {

    public static final String 
	IMPLICIT_JVM_IS_TRANSIENT = "<jvmIsTransient>";


    private boolean alreadyTransformed = false;

    public JVMIsTransientMethodBuilder
	(CrossReferenceServiceConfiguration services, 
	 CompilationUnitMutableList units) {	
	super(services, units);
    }

    private ReferencePrefix createJavaLangPrefix() {
	return new PackageReference
	    (new PackageReference(new Identifier("java")),
	     new Identifier("lang"));
    }

    /**
     * creates the implicit  <code>&lt;jvmIsTransient&gt(Object obj);</code>
     * method 
     * @param type the TypeDeclaration 
     * @return the implicit method
     */
    public MethodDeclaration createMethod(ClassDeclaration type) {	
	ParameterDeclaration paramDecl = new ParameterDeclaration
	    (new TypeReference(createJavaLangPrefix(), 
			       new Identifier("Object")), 
	     new Identifier("obj"));

	StatementMutableList body = new StatementArrayList(2);
	
	body.add(new Return
		 (new FieldReference
		  (new VariableReference(new Identifier("obj")),
		   new ImplicitIdentifier
		   (ImplicitFieldAdder.IMPLICIT_TRANSIENT))));

	ModifierMutableList modifiers = new ModifierArrayList(2);
	modifiers.add(new Public());

	ParameterDeclarationMutableList params = 
	    new ParameterDeclarationArrayList(1);
	params.add(paramDecl);

	MethodDeclaration md = new MethodDeclaration
	    (modifiers, 
	     new TypeReference(new Identifier("int")), 
	     new ImplicitIdentifier(IMPLICIT_JVM_IS_TRANSIENT),
	     params, null, new StatementBlock(body));

	md.makeAllParentRolesValid();
	return md;
    }    


    /**
     * entry method for the constructor normalform builder
     * @param td the TypeDeclaration 
     */
    protected void makeExplicit(TypeDeclaration td) {
	if (alreadyTransformed) return;
	if ("JCSystem".equals(td.getName())) {
	    PackageReference pr = 
		td.getASTParent() instanceof CompilationUnit ? 
		((CompilationUnit)td.getASTParent()).
		getPackageSpecification().getPackageReference() : null;
	    if (pr != null && "framework".equals(pr.getName())) {
		PackageReference prefix = 
		    (PackageReference)pr.getReferencePrefix();
		if (prefix != null && "javacard".equals(prefix.getName())) {
		    attach(createMethod((ClassDeclaration)td), 
			   td, td.getMembers().size());
		    alreadyTransformed = true;
		}
	    }
	}
    }






}
