// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//


package de.uka.ilkd.key.java.reference;

import de.uka.ilkd.key.java.Services;
import de.uka.ilkd.key.java.PrettyPrinter;
import de.uka.ilkd.key.java.SourceElement;
import de.uka.ilkd.key.java.NonTerminalProgramElement;
import de.uka.ilkd.key.java.ProgramElement;
import de.uka.ilkd.key.java.visitor.Visitor;
import de.uka.ilkd.key.logic.ProgramElementName;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.rule.SchemaElement;
import de.uka.ilkd.key.rule.AbstractProgramElement;
import de.uka.ilkd.key.java.abstraction.KeYJavaType;


public class SchemaTypeReference extends TypeReferenceImp
    implements AbstractProgramElement, SchemaElement {

    public SchemaTypeReference(ProgramElementName name, 
			       int dimension,
			       ReferencePrefix prefix) {
	super(name, dimension, prefix);
    }

    public KeYJavaType getKeYJavaType() {
	return null;
    }

    public KeYJavaType getKeYJavaType(Services services) {
	return services.getJavaInfo().getKeYJavaType(getName());
    }

    public ProgramElement getConcreteProgramElement(Services services) {
	return new TypeRef(getKeYJavaType(services));
    }

    public boolean canMatch(Term t) {
	return false;
    }

    public boolean canMatch(ProgramElement t) {
	if (t instanceof TypeReference) {
	    return getName().equals(((TypeReference)t).getName()) &&
		((TypeReference)t).getDimensions() == getDimensions();
	} else {
	    return false;
	}
    }

    /** calls the corresponding method of a visitor in order to
     * perform some action/transformation on this element
     * @param v the Visitor
     */
    public void visit(Visitor v) {
	v.performActionOnAbstractProgramElement(this);
    }

    public void prettyPrint(PrettyPrinter p) throws java.io.IOException {
        p.printSchemaTypeReference(this);
    }

}


