// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.jml;

import java.util.*;
import javax.swing.JOptionPane;

import de.uka.ilkd.key.java.declaration.*;
import de.uka.ilkd.key.java.abstraction.*;
import de.uka.ilkd.key.java.annotation.*;
import de.uka.ilkd.key.java.statement.LoopStatement;
import de.uka.ilkd.key.java.Services;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.collection.SetOfString;
import de.uka.ilkd.key.collection.SetAsListOfString;

public class Implementation2SpecMap{
    
    //maps method declarations to a set of method specifications.
    private HashMap method2specs;
    //maps class or interface declarations to a class specification.
    private HashMap class2spec;
    private HashMap loop2spec;
    private HashMap method2JMLModifiers;
    private Services services;
    private Namespace progVars;
    //used for caching inherited methodspecs
    private HashMap inheritedSpecCache;

    public Implementation2SpecMap(Services services){
	method2specs = new HashMap();
	class2spec = new HashMap();
	method2JMLModifiers = new HashMap();
	this.services = services;
	inheritedSpecCache = new HashMap();
	loop2spec = new HashMap();
    }

    private Implementation2SpecMap(HashMap method2specs, HashMap class2spec,
				   HashMap method2JMLModifiers,
				   HashMap inheritedSpecCache, 
				   HashMap loop2spec,
				   Services services){
	this.method2specs = method2specs;
	this.class2spec = class2spec;
	this.method2JMLModifiers = method2JMLModifiers;
	this.inheritedSpecCache = inheritedSpecCache;
	this.loop2spec = loop2spec;
	this.services = services;
    }

    public void addMethodSpec(JMLMethodSpec ms){
	if(method2specs.get(ms.getMethodDeclaration()) == null){
	    method2specs.put(ms.getMethodDeclaration(), 
			     SetAsListOfJMLMethodSpec.EMPTY_SET.add(ms));
	}else{
	    method2specs.put(ms.getMethodDeclaration(),
			     ((SetOfJMLMethodSpec) method2specs.
			      get(ms.getMethodDeclaration())).add(ms));
	}
    }

    public void addAllMethodSpecs(Set set){
	Iterator it = set.iterator();
	while(it.hasNext()){
	    addMethodSpec((JMLMethodSpec) it.next());
	}
    }

    public void setProgVarNS(Namespace progVars){
	this.progVars = progVars;
    }

    public Namespace getProgVarNS(){
	return progVars;
    }

    public SetOfJMLMethodSpec getSpecsForMethod(MethodDeclaration md){
	return (SetOfJMLMethodSpec) method2specs.get(md);
    }

    /**
     * Removes the specifications for <code>md</code>.
     */
    public void clearSpecsForMethod(MethodDeclaration md){
	method2specs.remove(md);
    }

    public void addModifier(MethodDeclaration md, String modifier){
	if(method2JMLModifiers.get(md) == null){
	    method2JMLModifiers.put(md, SetAsListOfString.EMPTY_SET.add(
					modifier));
	}else{
	    method2JMLModifiers.put(
		md,((SetAsListOfString) method2JMLModifiers.get(md)).
		add(modifier));
	}
    }

    public void addLoopSpec(LoopInvariant l){
	loop2spec.put(l.getLoop(), l);
    }

    public void clearSpecsForLoop(LoopStatement ls){
	loop2spec.remove(ls);
    }

    public LoopInvariant getSpecForLoop(LoopStatement ls){
	return (LoopInvariant) loop2spec.get(ls);
    }

    /**
     * Creates LoopInvariantAnnotations from the parsed loop specifications and
     * adds them to the correcponding loop statements.
     */
    public void createLoopAnnotations(){
	Iterator it = loop2spec.keySet().iterator();
	while(it.hasNext()){
	    LoopStatement loop = (LoopStatement) it.next();
	    LoopInvariant li = (LoopInvariant) loop2spec.get(loop);
	    Term[] ar;
	    if("nothing".equals(li.getAssignableMode())){
		ar = new Term[1];
		ar[0] = TermFactory.DEFAULT.createFunctionTerm(Op.NOTHING);
	    }else if("everything".equals(li.getAssignableMode())){
		ar = new Term[1];
		ar[0] = TermFactory.DEFAULT.createFunctionTerm(Op.EVERYTHING);
	    }else{
		SetOfTerm as = li.getAssignable();
		ar = new Term[as.size()];
		IteratorOfTerm ait = as.iterator();
		int j=0;
		while(ait.hasNext()){
		    ar[j++] = ait.next(); 
		}
	    }
	    loop.addLoopInvariant(new LoopInvariantAnnotation(
				      li.getInvariant(), new ArrayOfTerm(ar), 
				      li.getVariant()));
	}
    }
 
    /**
     * adds the pure-modifier to every constructor and instance method declared
     * in <code>kjt</code>
     */
    public void setPure(KeYJavaType kjt, NamespaceSet nss){
	ListOfMethod l = services.getJavaInfo().getAllMethods(kjt);
	IteratorOfMethod it = l.iterator();
	MethodDeclaration md;
	while(it.hasNext()){
	    md = (MethodDeclaration) it.next();
	    if(!md.isStatic()){
                new JMLPuritySpec(md, TermFactory.DEFAULT, services, 
				  UsefulTools.buildParamNamespace(md), 
                                  new HashMap(), getSpecForClass(kjt), nss);
		addModifier(md, "pure");
	    }
	}
    }

    public SetOfString getModifiers(MethodDeclaration md){
	if(method2JMLModifiers.get(md) != null){
	    return (SetOfString) method2JMLModifiers.get(md);
	}else{
	    return SetAsListOfString.EMPTY_SET;
	}
    }

    /** checks whether the methods annotated with the pureModifier really
     * have no side effect. This is done by allowing only normal_behavior
     * specs with \nothing as assignable clause which is slightly different
     * to jml definition of purity which allows the possibility of throwing
     * exceptions.
     */
/*    public void checkPurity(){
	Iterator it = method2JMLModifiers.keySet().iterator();
	while(it.hasNext()){
	    MethodDeclaration md = (MethodDeclaration) it.next();
	    SetOfString set = (SetOfString) method2JMLModifiers.get(md);
	    if(set.contains("pure")){
		SetOfJMLMethodSpec specs = getSpecsForMethod(md);
		if(specs != null){
		    IteratorOfJMLMethodSpec spit = specs.iterator();
		    while(spit.hasNext()){
			JMLMethodSpec spec = spit.next();
			if(!(spec instanceof JMLNormalMethodSpec)){
			    JOptionPane.showConfirmDialog
				(null,
				 "One of your specification cases for the "+
				 "pure method\n"+md.getName()+
				 "\nis not a normal_behavior specification case", 
				 "Warning",
				 JOptionPane.DEFAULT_OPTION,
				 JOptionPane.WARNING_MESSAGE);
			}
			if(!spec.terminates()){
			    JOptionPane.showConfirmDialog
				(null,
				 "Your specification for the pure method\n"+
				 md.getName()+
				 "\nallows nontermination", 
				 "Warning",
				 JOptionPane.DEFAULT_OPTION,
				 JOptionPane.WARNING_MESSAGE);
			}			    
			if(!(md instanceof ConstructorDeclaration)){
			    if(!"nothing".equals(spec.getAssignableMode())){
				JOptionPane.showConfirmDialog
				    (null,
				     "Pure Methods may not"+
				     " assign to any fields. "+
				     "\nplease check your specifications for "+
				     md.getName(), 
				     "Inconsistent Specification",
				     JOptionPane.DEFAULT_OPTION,
				     JOptionPane.WARNING_MESSAGE);
			    }
			}else{
			    IteratorOfTerm itt = 
				spec.getAssignable().iterator();
			    while(itt.hasNext()){
				checkStatic(itt.next(), md);
			    }
			}
		    }
		}
	    }
	}
    }

    private void checkStatic(Term t, MethodDeclaration md){
	if(t.op() instanceof ProgramVariable &&
	   !((ProgramVariable) t.op()).isStatic()){
	    JOptionPane.showConfirmDialog
		(null,
		 "Pure Constructors may only"+
		 " assign to static fields. "+
		 "\nthat's why "+
		 md.getName()+" can't be declared pure",
		 "Inconsistent Specification", 
		 JOptionPane.DEFAULT_OPTION,
		 JOptionPane.WARNING_MESSAGE);
	}
	if(t.op() instanceof AttributeOp){
	    checkStatic(t.sub(0), md);
	}
	if(t.op() instanceof ArrayOp){
	    checkStatic(t.sub(0), md);
	    checkStatic(t.sub(1), md);
	}
    }
*/

    public void addClassSpec(JMLClassSpec cs){
	class2spec.put(services.getJavaInfo().
		       getKeYJavaType(cs.getClassDeclaration()), cs);
    }

    public JMLClassSpec getSpecForClass(KeYJavaType classType){
	return (JMLClassSpec) class2spec.get(classType);
    }

    public void removeMethodSpec(JMLMethodSpec ms){
	if(method2specs.get(ms.getMethodDeclaration()) != null){
	    method2specs.put(ms.getMethodDeclaration(),
			     ((SetOfJMLMethodSpec) method2specs.
			      get(ms.getMethodDeclaration())).remove(ms));
	}
    }

    public void removeClassSpec(JMLClassSpec cs){
	class2spec.remove(services.getJavaInfo().
			  getKeYJavaType(cs.getClassDeclaration()));
    }

    public Implementation2SpecMap copy(Services serv){
	Implementation2SpecMap ism = 
	    new Implementation2SpecMap((HashMap)method2specs.clone(), 
				       (HashMap)class2spec.clone(),
				       (HashMap)method2JMLModifiers.clone(),
				       (HashMap)inheritedSpecCache.clone(),
				       (HashMap)loop2spec.clone(),
				       serv);
	ism.setProgVarNS(getProgVarNS());
	return ism;
    }

    /**
     * returns all methoddeclarations in this map.
     */
    public Set getAllMethods(){
	return method2specs.keySet();
    }

    /**
     * returns all classtypes in this map.
     */
    public Set getAllClasses(){
	return class2spec.keySet();
    }

    public SetOfJMLMethodSpec getInheritedMethodSpecs(MethodDeclaration md,
						      KeYJavaType classType){
	InhKey key = new InhKey(md, classType);
	if(inheritedSpecCache.get(key) != null){
	    return (SetOfJMLMethodSpec) inheritedSpecCache.get(key);
	}
	TypeDeclaration cDec = 
	    services.getJavaInfo().getTypeDeclaration(
		classType.getSort().toString());
	SetOfJMLMethodSpec inh = SetAsListOfJMLMethodSpec.EMPTY_SET;
	if((cDec instanceof ClassDeclaration) && !md.isAbstract()){
	    ListOfKeYJavaType st = cDec.getSupertypes();
	    IteratorOfKeYJavaType it = st.iterator();
	    while(it.hasNext()){
		ListOfProgramMethod l = 
		    services.getJavaInfo().getAllProgramMethods(it.next());
		IteratorOfProgramMethod pmIt = l.iterator();
		while(pmIt.hasNext()){
		    MethodDeclaration m = pmIt.next().getMethodDeclaration();
		    SetOfJMLMethodSpec specs = getSpecsForMethod(m);
		    if(specs != null){
			IteratorOfJMLMethodSpec sIt = specs.iterator();
			while(sIt.hasNext()){
			    JMLMethodSpec spec = sIt.next();
			    if(!(spec instanceof JMLPuritySpec) && 
			       spec.isCloneableFor(md)){
				inh = inh.add(
				    spec.cloneFor(md, 
						  getSpecForClass(classType)));
			    }else if(!spec.isCloneableFor(md)){
				break;
			    }
			}
		    }
		}
	    }
	}
	inheritedSpecCache.put(key, inh);
	return inh;
    }

    private class InhKey{

	protected MethodDeclaration md;
	protected KeYJavaType ct;

	public InhKey(MethodDeclaration md, KeYJavaType ct){
	    this.md = md;
	    this.ct = ct;
	}

	public boolean equals(Object o){
	    if(!(o instanceof InhKey)) return false;
	    return ((InhKey) o).md == md && ((InhKey) o).ct == ct;
	}

	public int hashCode(){
	    return md.hashCode() + 31*ct.hashCode();
	}
    }
}
