// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.jml;

import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.logic.sort.ObjectSort;
import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.java.statement.*;
import de.uka.ilkd.key.java.abstraction.*;
import de.uka.ilkd.key.java.reference.*;
import de.uka.ilkd.key.java.expression.operator.*;
import de.uka.ilkd.key.java.declaration.*;
import de.uka.ilkd.key.java.declaration.modifier.*;
import de.uka.ilkd.key.util.ExtList;

import java.util.*;

/** wraps specifications that are valid for the entire class (e.g. invariants
 * or history constraints) or interface
 */
public class JMLClassSpec extends JMLSpec{

    private Term instInv, instCon, staticInv, staticCon;
    private HashMap term2old;
    private HashMap represents;
//    private HashMap suchthat;
    private ReferencePrefix staticPrefix;
    private ReferencePrefix instancePrefix;
    private Services services;
    /** the class or interfacedeclaration this specification refers to */
    private TypeDeclaration cd;
    // the KeYJavaType of cd
    private KeYJavaType type;
    private TermFactory tf;
    private Namespace modelVars;
    private Namespace modelMethods;
    private NamespaceSet nss;
    private static int param_count = 0;
    protected static Term trueTerm = null;
    protected static Term falseTerm = null;

    public JMLClassSpec(Services services, TypeDeclaration cd, 
			TermFactory tf, NamespaceSet nss){
	instInv=instCon=staticInv=staticCon=null;
	term2old = new HashMap();
	this.services = services;
	this.cd = cd;
	type = services.getJavaInfo().getKeYJavaType(cd);
	staticPrefix = new TypeRef(type);
	instancePrefix = new ProgramVariable(
	    new ProgramElementName("self_"+cd.getName()), type);
	nss.programVariables().
	    add((ProgramVariable) instancePrefix);
	this.tf = tf;
	modelVars = new Namespace();
	modelMethods = new Namespace();
	represents = new HashMap();
//	suchthat = new HashMap();
	this.nss = nss;
	services.getImplementation2SpecMap().addClassSpec(this);
	if(trueTerm == null){
	    trueTerm = tf.createJunctorTerm(Op.TRUE);
	}
	if(falseTerm == null){
	    falseTerm = tf.createJunctorTerm(Op.FALSE);
	}
    }

    public boolean containsInvOrConst(){
	return (staticCon != null ||
	    staticInv != null ||
	    instCon != null ||
	    instInv != null);
    }

    public Services getServices(){
	return services;
    }

    /** 
     * returns the class or interfacedeclaration this specification refers to. 
     */
    public TypeDeclaration getClassDeclaration(){
	return cd;
    }

    public ReferencePrefix getInstancePrefix(){
	return instancePrefix;
    }

    public ReferencePrefix getStaticPrefix(){
	return staticPrefix;
    }
    
    public void addInstanceInvariant(Term t){
	instInv = UsefulTools.makeConjunction(instInv,t);
    }

    public void addStaticInvariant(Term t){
	staticInv = UsefulTools.makeConjunction(staticInv, t);
    }

    public void addInstanceConstraint(Term t){
	instCon = UsefulTools.makeConjunction(instCon, t);
    }

    public void addStaticConstraint(Term t){
	staticCon = UsefulTools.makeConjunction(staticCon, t);
    }

    public void addModelVariable(ProgramVariable v){
	//	nss.programVariables().add(v);
	modelVars.add(v);
    }

    public void addModelMethod(ProgramMethod pm){
	modelMethods.add(pm);
    }

    /**
     * returns the model variables declared in this class
     */
    public Namespace getModelVars(){
	return modelVars;
    }

    /**
     * returns the model methods declared in this class
     */
    public Namespace getModelMethods(){
	return modelMethods;
    }

    public void addRepresents(Term var, Term rep){
	ProgramMethod pm = createProgramMethodForMV(var);
	modelMethods.add(pm);
	JMLMethodSpec spec = new JMLNormalMethodSpec(
	    pm.getMethodDeclaration(), tf, services,
	    new Namespace(), new HashMap(), this, nss);
	spec.setAssignableMode("nothing");
	if(rep.sort() == Sort.FORMULA){
	    Term eq = tf.createEqualityTerm(
		tf.createVariableTerm(spec.getResultVar()), 
		JMLMethodSpec.trueLitTerm);
	    spec.addPost(tf.createEqualityTerm(eq, rep));
	}else{
	    spec.addPost(tf.createEqualityTerm(tf.createVariableTerm(
						   spec.getResultVar()), rep));
	}
	ProgramVariable v;
	if(var.op() instanceof ProgramVariable){
	    v = (ProgramVariable) var.op();
	}else{
	    v = (ProgramVariable) ((AttributeOp) var.op()).attribute();
	}
	represents.put(v, getTermForModelMethod(pm));
    }

    public void addSuchThat(Term var, Term axiom){
	ProgramMethod pm = createProgramMethodForMV(var);
	ProgramVariable v;
	if(var.op() instanceof ProgramVariable){
	    v = (ProgramVariable) var.op();
	}else{
	    v = (ProgramVariable) ((AttributeOp) var.op()).attribute();
	}
	represents.put(v, getTermForModelMethod(pm));
	modelMethods.add(pm);
	JMLMethodSpec spec = new JMLNormalMethodSpec(
	    pm.getMethodDeclaration(), tf, services,
	    new Namespace(), new HashMap(), this, nss);
	spec.setAssignableMode("nothing");
	spec.addPost(tf.createUpdateTerm(
			 var, 
			 tf.createVariableTerm(spec.getResultVar()),
			 axiom));
	spec.addPre(getExTermForModelVar(var, axiom));
    }

    /** 
     * returns a mapping from modelmethods to their specifications.
     */
    public HashMap buildModelMethod2Specs(){
	HashMap modelMethod2Specs = new HashMap();
	IteratorOfNamed it = modelMethods.allElements().iterator();
	Implementation2SpecMap ism = services.getImplementation2SpecMap();
	while(it.hasNext()){
	    ProgramMethod pm = (ProgramMethod) it.next();
	    modelMethod2Specs.put(pm, ism.getSpecsForMethod(
					 pm.getMethodDeclaration()));
	}
	return modelMethod2Specs;
    }

    /**
     * returns a HashMap that contains the locally defined and inherited 
     * represents relations.
     */
    public HashMap getRepresents(){
	HashMap result = new HashMap();
	KeYJavaType kjt = services.getJavaInfo().getSuperclass(type);
	JMLClassSpec superSpec = 
	    services.getImplementation2SpecMap().getSpecForClass(kjt);
	if(superSpec != null && (cd instanceof ClassDeclaration)){
	    result.putAll(superSpec.getRepresents());
	}
	result.putAll(represents);
	return result;
    }

    public HashMap getTerm2Old(){
	return term2old;
    }

    public Term getLocalInstanceInvariants(){
	return instInv;
    }

    /**
     * returns locally declared and inherited instance invariants. 
     */
    public Term getInstanceInvariants(){
	Term result = UsefulTools.makeConjunction(null, instInv);
	Implementation2SpecMap ism = services.getImplementation2SpecMap();
	ListOfKeYJavaType l = cd.getSupertypes();
	IteratorOfKeYJavaType it = l.iterator(); 
	while(it.hasNext()){
	    JMLClassSpec cs = ism.getSpecForClass(it.next());
	    if(cs != null){
		if(cs.getInstanceInvariants() != null){
		    Term upd = tf.createUpdateTerm(
                         tf.createVariableTerm((ProgramVariable) 
					       cs.getInstancePrefix()),
			 tf.createVariableTerm((ProgramVariable) 
					       getInstancePrefix()),
			 cs.getInstanceInvariants());
		    result = UsefulTools.makeConjunction(result, upd);
		}
	    }
	}
	return result;
    }

    public Term getLocalStaticInvariants(){
	return staticInv;
    }

    public Term getStaticInvariants(){
	Term result = staticInv;
	Implementation2SpecMap ism = services.getImplementation2SpecMap();
	ListOfKeYJavaType l = cd.getSupertypes();
	IteratorOfKeYJavaType it = l.iterator(); 
	while(it.hasNext()){
	    JMLClassSpec cs = ism.getSpecForClass(it.next());
	    if(cs != null){
		result = UsefulTools.makeConjunction(
		    result, cs.getStaticInvariants());
	    }
	}
	return result;
    }

    public Term getLocalInstanceConstraints(){
	return instCon;
    }

    public Term getInstanceConstraints(){
	Term result = instCon;
	Implementation2SpecMap ism = services.getImplementation2SpecMap();
	ListOfKeYJavaType l = cd.getSupertypes();
	IteratorOfKeYJavaType it = l.iterator(); 
	while(it.hasNext()){
	    JMLClassSpec cs = ism.getSpecForClass(it.next());
	    if(cs != null){
		if(cs.getInstanceConstraints() != null){
		    Term upd = tf.createUpdateTerm(
			 tf.createVariableTerm((ProgramVariable) 
					       cs.getInstancePrefix()),
			 tf.createVariableTerm((ProgramVariable) 
					       getInstancePrefix()),
			 cs.getInstanceConstraints());
		    result = UsefulTools.makeConjunction(result, upd);
		}
	    }
	}
	return result;
    }

    public Term getLocalStaticConstraints(){
	return staticCon;
    }

    public Term getStaticConstraints(){
	Term result = staticCon;
	Implementation2SpecMap ism = services.getImplementation2SpecMap();
	ListOfKeYJavaType l = cd.getSupertypes();
	IteratorOfKeYJavaType it = l.iterator(); 
	while(it.hasNext()){
	    JMLClassSpec cs = ism.getSpecForClass(it.next());
	    if(cs != null){
		result = UsefulTools.makeConjunction(
		    result, cs.getStaticConstraints());
	    }
	}
	return result;
    }

    /** generates \exists x . p(x) where p is the relation for 
     * <code>modelvar</code> described by its represents-such_that clause.
     */
    public Term getExTermForModelVar(Term modelVar, Term axiom){
	Term result = null;
	LogicVariable lv = new LogicVariable(new Name(modelVar.toString()+
						 "_jml_lv"), modelVar.sort());
	result = tf.createUpdateTerm(modelVar, tf.createVariableTerm(lv), axiom);
	return tf.createQuantifierTerm(Op.EX, lv, result);
    }

    public ProgramVariable lookupModelField(Name name)
	throws AmbigiousModelElementException{
	if(modelVars.lookup(name) != null){
	    return (ProgramVariable) modelVars.lookup(name);
	}
	Implementation2SpecMap ism = services.getImplementation2SpecMap();
	ListOfKeYJavaType st = cd.getSupertypes();
	IteratorOfKeYJavaType it = st.iterator();
	ProgramVariable result = null;
	while(it.hasNext()){
	    KeYJavaType kjt = it.next();
	    if(ism.getSpecForClass(kjt) != null &&
	       ism.getSpecForClass(kjt).lookupModelField(name) != null){
		if(result == null){
		    result = (ProgramVariable) ism.getSpecForClass(kjt).
			lookupModelField(name);
		}else{
		    throw new AmbigiousModelElementException(result);
		}
	    }
	}
	return result;
    } 

    /**
     * Returns the Term ((self!=null & self.created=true) -> instanceInvariants
     * ) & staticInvariants, which is sometimes needed in POs.  
     */
    public Term getAllInvariants(){
	Term result = null;
	Term self = tf.createVariableTerm((ProgramVariable) instancePrefix);
	if(getInstanceInvariants() != null && 
	   !trueTerm.equals(getInstanceInvariants())){
	    Term eqn = 
		tf.createEqualityTerm(self, tf.createFunctionTerm(Op.NULL));
	    eqn = tf.createJunctorTerm(Op.NOT, new Term[]{eqn});
	    ProgramVariable created = services.getJavaInfo().getAttribute(
		"<created>", (ObjectSort) type.getSort());
	    Term eqc = tf.createAttributeTerm(created, self);
	    eqc = tf.createEqualityTerm(eqc, JMLMethodSpec.trueLitTerm);
	    result = tf.createJunctorTerm(Op.AND, new Term[]{eqn, eqc});
	    result = tf.createJunctorTerm(Op.IMP, new Term[]{
		result, getInstanceInvariants()});
	    if(getStaticInvariants() != null){
		ProgramVariable ci = services.getJavaInfo().getAttribute(
		    "<classInitialized>", type);
		Term eqi = tf.createEqualityTerm(
		    tf.createVariableTerm(ci),JMLMethodSpec.trueLitTerm);
		eqi = tf.createJunctorTerm(Op.IMP, new Term[]{
		    eqi, getStaticInvariants()});
		result = tf.createJunctorTerm(Op.AND, new Term[]{result, eqi});
	    }
	}else if(getStaticInvariants() != null &&
		 !trueTerm.equals(getStaticInvariants())){
	    result = getStaticInvariants();
	}
	if(result == null){
	    return null;
	}
	result = UsefulTools.addRepresents(result, services, this);
	LogicVariable selfLv = new LogicVariable(new Name("self_"+cd.getName()+
						     "_lv"),
						 self.sort());
	Term lvTerm = tf.createVariableTerm(selfLv);
	result = tf.createUpdateTerm(self, lvTerm, result);
	result = tf.createQuantifierTerm(Op.ALL,selfLv, result);
	return result;
    }

    public ProgramMethod lookupModelMethod(Name name)
	throws AmbigiousModelElementException{
	if(modelMethods.lookup(name) != null){
	    return (ProgramMethod) modelMethods.lookup(name);
	}
	Implementation2SpecMap ism = services.getImplementation2SpecMap();
	ListOfKeYJavaType st = cd.getSupertypes();
	IteratorOfKeYJavaType it = st.iterator();
	ProgramMethod result = null;
	while(it.hasNext()){
	    KeYJavaType kjt = it.next();
	    if(ism.getSpecForClass(kjt) != null &&
	       ism.getSpecForClass(kjt).lookupModelField(name) != null){
		if(result == null){
		    result = (ProgramMethod) ism.getSpecForClass(kjt).
			lookupModelMethod(name);
		}else{
		    throw new AmbigiousModelElementException(result);
		}
	    }
	}
	return result;
    } 

    protected Term getTermForModelMethod(ProgramMethod pm){
	Term[] sub;
	if(pm.isStatic()){
	    sub = new Term[0];
	}else{
	    sub = new Term[1];
	    sub[0] = tf.createVariableTerm((ProgramVariable) instancePrefix);
	}
	return tf.createFunctionTerm(pm, sub);
    }
    
    /**
     * creates a ProgramMethod and the Term for substituting the 
     * corresponding Model Variable.
     */
    private ProgramMethod createProgramMethodForMV(Term var){
	ExtList l = new ExtList();
	ProgramVariable mv;
	if(var.op() instanceof AttributeOp){
	    mv = (ProgramVariable) ((AttributeOp) var.op()).attribute();
	}else{
	    mv = (ProgramVariable) var.op();
	}
	l.add(mv.name());
	if(mv.isStatic()){
	    l.add(new Static());
	}
	l.add(new Model());
	l.add(new TypeRef(mv.getKeYJavaType()));
	MethodDeclaration md = new MethodDeclaration(l, false);
	ProgramMethod pm = new ProgramMethod(md, type, 
					     mv.getKeYJavaType(),
					     PositionInfo.UNDEFINED);
	services.getImplementation2SpecMap().addModifier(md, "pure");
	services.getImplementation2SpecMap().addModifier(md, "helper");
	if(nss.functions().lookup(pm.name()) == null){
	    nss.functions().add(pm);
	}
	return pm;
    }

    /**
     * returns a term of the form inv -> <{try{ m();}catch(Exception e)}>inv
     * If <code>allInv</code> is true, inv is a conjunction of all
     * (static and instance) invariants of every existing type or just
     * the invariants of this type specification otherwise.
     */
    public Term getPreservesInvariantBehavior(MethodDeclaration md,
					      boolean allInv){
	if(!services.getImplementation2SpecMap().
	   getModifiers(md).contains("helper") && (
	       !md.isStatic() &&(getInstanceInvariants()!=null ||
				 getInstanceConstraints()!=null) ||
	       getStaticInvariants() != null ||
	       getStaticConstraints() != null || allInv)){
	    Term excPre, excPost, result;
	    result = excPre = excPost = null;
	    KeYJavaType exc = 
		services.getJavaInfo().getTypeByClassName("Exception");
	    Branch[] catches = new Branch[1];
	    StatementBlock catchBody = new StatementBlock();
	    ProgramVariable e = new ProgramVariable(
		new ProgramElementName("_exc"+
				       (JMLMethodSpec.exceptionVarCount++)), 
		exc);
	    nss.programVariables().add(e);
	    ParameterDeclaration param = 
		new ParameterDeclaration(new Modifier[0], new TypeRef(exc),
					 new VariableSpecification(e), false);
	    catches[0] = new Catch(param, catchBody);
	    excPost = addClassSpec2Post(excPost, true, !allInv, md, this);
	    excPost = UsefulTools.addRepresents(excPost, services, this);
	    if(allInv){
		Term ai = UsefulTools.allInvariants(
		    services.getImplementation2SpecMap());
		excPre = UsefulTools.makeConjunction(ai, excPre);
		excPost = UsefulTools.makeConjunction(ai, excPost);
	    }else{
		excPre = addClassSpec2Pre(excPre, md, this);
	    }
	    StatementBlock tryBlock = 
		new StatementBlock(new Try(makeMBS(md), catches));
	    JavaBlock jb = JavaBlock.createJavaBlock(tryBlock);
	    excPost = tf.createBoxTerm(jb, excPost);
	    if(excPost == null){
		return trueTerm;
	    }
	    if(excPre == null){
		result = excPost;
	    }else{
		result=tf.createJunctorTerm(Op.IMP,new Term[]{excPre,excPost});
	    }
	    result = JMLMethodSpec.addOldQuantifiers(
		result, getTerm2Old(), false);
	    result = UsefulTools.addRepresents(result, services, this);
	    if(UsefulTools.purityCheck(result, 
				       services.getImplementation2SpecMap()) 
	       != null){
		throw new RuntimeException("Nonpure method "+
					   UsefulTools.purityCheck(result, 
					       services.
					       getImplementation2SpecMap())+
					   " used in the "+
					   "specification for class/interface "
					   + cd.getName());
	    }

	    //disjoins the methods preconditions
	    SetOfJMLMethodSpec specs = 
		services.getImplementation2SpecMap().getSpecsForMethod(md);
	    if(specs != null){
		specs = specs.union(services.getImplementation2SpecMap().
				    getInheritedMethodSpecs(md, type));
	    }
	    if(specs != null){
		Term invPre = falseTerm;
		IteratorOfJMLMethodSpec it = specs.iterator();
		while(it.hasNext()){
		    JMLMethodSpec ms = (JMLMethodSpec) it.next();
		    if(!(ms instanceof JMLPuritySpec)){
			invPre = tf.createJunctorTermAndSimplify(
			    Op.OR, ms.getCompletePre(false, false), invPre);
		    }
		}
		if(invPre != falseTerm){
		    result = tf.createJunctorTerm(Op.IMP, 
						  new Term[]{invPre, result});
		}
	    }
	    result = UsefulTools.quantifySelf(result, md, (ProgramVariable) 
					      getInstancePrefix());
	    return UsefulTools.quantifyProgramVariables(result, 
			    UsefulTools.getParameters(md).iterator(), Op.ALL);
	}else{
	    return trueTerm;
	}
    }

    /** Adds the applicable invariants of cSpec to the precondition of md 
     * iff md isn't declared with the modifier helper. md must be a member
     * method of cSpec. 
     */
    public Term addClassSpec2Pre(Term pre, MethodDeclaration md, 
				 JMLClassSpec cSpec){
	Term result = pre;
	if(md.getName().equals("<clinit>")){
	    ProgramVariable v = services.getJavaInfo().getAttribute(
		"<classInitialized>", type);
	    if(result == null){
		result = tf.createEqualityTerm(
		    tf.createVariableTerm(v),JMLMethodSpec.falseLitTerm);
	    }else{
		result = tf.createJunctorTerm(
		    Op.AND,new Term[]{result,tf.createEqualityTerm(
				      tf.createVariableTerm(v),
				      JMLMethodSpec.falseLitTerm)});
	    }
	    v = services.getJavaInfo().getAttribute("<classErroneous>", 
						    type);
	    result = tf.createJunctorTerm(
		Op.AND,new Term[]{result,tf.createEqualityTerm(
				      tf.createVariableTerm(v),
				      JMLMethodSpec.falseLitTerm)});
	    v = services.getJavaInfo().getAttribute(
		"<classInitializationInProgress>", 
		type);
	    result = tf.createJunctorTerm(
		Op.AND,new Term[]{result,tf.createEqualityTerm(
				      tf.createVariableTerm(v),
				      JMLMethodSpec.falseLitTerm)});
	    return result;
	}
	if(cSpec == null || md.getName().startsWith("<") || 
	   services.getImplementation2SpecMap().getModifiers(md).
	   contains("helper")){
	    return pre;
	}
	result = 
	    UsefulTools.makeConjunction(result, cSpec.getStaticInvariants());
	if(!md.isStatic() && !(md instanceof ConstructorDeclaration)){
	    result = UsefulTools.makeConjunction(
		result, cSpec.getInstanceInvariants());
	}
//	result = addAxioms2Pre(result, true, false);
	return result;
    }

    /** Adds invariants and history constraints to the postcondition iff md 
     * isn't declared with the modifier helper.
     */
    public Term addClassSpec2Post(Term post, boolean constraints,
				  boolean invariants,
				  MethodDeclaration md, 
				  JMLClassSpec cSpec){
	if(cSpec == null || 
	   services.getImplementation2SpecMap().getModifiers(md).
	   contains("helper")){
	    return post;
	}
	Term result = post;
	if((!md.getName().startsWith("<") || md.getName().equals("<clinit>"))&&
	   invariants){
	    result = UsefulTools.makeConjunction(
		result, cSpec.getStaticInvariants());
	}
	if(constraints && !md.getName().startsWith("<")){
	    result = UsefulTools.makeConjunction(
		result, cSpec.getStaticConstraints());
	}
	if(!md.isStatic() && !md.getName().startsWith("<")){
	    if(invariants){
		result = UsefulTools.makeConjunction(
		    result, cSpec.getInstanceInvariants());
	    }
	    if(constraints && !(md instanceof ConstructorDeclaration)){
		result = UsefulTools.makeConjunction(
		    result, cSpec.getInstanceConstraints());
	    }
	}
	return result;
    }

    private StatementBlock makeMBS(MethodDeclaration md){
	StatementBlock mBS;
	ArrayOfParameterDeclaration params = md.getParameters();
	Expression[] aop = new Expression[md.getParameters().size()]; 
	for(int i=0; i<params.size(); i++){
		aop[i] = (ProgramVariable) params.
		    getParameterDeclaration(i).
		    getVariableSpecification().getProgramVariable();
	}
	if(!(md instanceof Constructor)){
	    KeYJavaType returnType;
	    if(md.getTypeReference() == null){
		returnType = services.getJavaInfo().getKeYJavaType("void");
	    }else{
		returnType = md.getTypeReference().getKeYJavaType();
	    }
	    ProgramMethod pm = 
		new ProgramMethod(md, type, 
				  returnType, PositionInfo.UNDEFINED);
	    MethodBodyStatement mb = new MethodBodyStatement(
		pm,
		getInstancePrefix(),
		null,
		new ArrayOfExpression(aop));
	    mBS = new StatementBlock(mb);
	}else{
	    New n = new New(aop,
			    new TypeRef(type),
			    getStaticPrefix());
	    mBS = new StatementBlock(new CopyAssignment(
					 (ProgramVariable) getInstancePrefix(),
					 n
					 ));
	}
	return mBS;
    }

    public String toString(){
	return "Class specification for class "+cd.getName();
    }

}

