// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.jml;

import java.util.*;

import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.java.declaration.*;
import de.uka.ilkd.key.java.abstraction.PrimitiveType;
import de.uka.ilkd.key.java.abstraction.Constructor;
import de.uka.ilkd.key.java.expression.literal.BooleanLiteral;


public class JMLExceptionalMethodSpec extends JMLMethodSpec{

    public JMLExceptionalMethodSpec(MethodDeclaration md, 
				TermFactory tf, Services services, 
				Namespace params, HashMap term2old, 
				JMLClassSpec cSpec, NamespaceSet nss){
	super(md, tf, services, params, term2old, cSpec, nss);
    }

    protected JMLExceptionalMethodSpec(MethodDeclaration md){
	super(md);
    }

    /**
     * This is used for modelling specification inheritance for overwriten
     * methods.
     */
    public JMLMethodSpec cloneFor(MethodDeclaration md, JMLClassSpec cSpec){
	if(!isCloneableFor(md)) return null;
	JMLExceptionalMethodSpec cloned = new JMLExceptionalMethodSpec(md);
	return (JMLExceptionalMethodSpec) cloneForHelper(cloned, md, cSpec);
    }

    public void addPost(Term t){}

    public Term getPost(){
	return falseTerm;
    }

    public JMLMethodSpec copy(){
	JMLExceptionalMethodSpec copy = 
	    new JMLExceptionalMethodSpec(md, tf, services, params, 
					 term2old, cSpec, nss);
	return copyHelper(copy);
    }

    public String toString(){
	return inhFrom+"exceptional_behavior speccase for method "+
	    md.getName()+"\nin context "+cd.getName()+
	    "\nrequires: "+(getPre()==null ? "true" : getPre().toString());
    } 

    public Term getCompletePost(boolean withClassSpec, boolean allInv){
	if(lv2old == null){
	    getPi1();
	}
	Term result = 
	    createModalityTermForEnsures(
		falseTerm, false, withClassSpec, allInv).sub(0);
	result = addOldQuantifiers(result, lv2old, false);
	if(!(md instanceof Constructor)){
	    result = updatePrefix(result);
	}
	return result;
    }

}

