// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.jml;

import de.uka.ilkd.key.java.declaration.MethodDeclaration;
import de.uka.ilkd.key.java.declaration.TypeDeclaration;
import de.uka.ilkd.key.java.statement.LoopStatement;
import de.uka.ilkd.key.java.Services;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.op.Op;

public class LoopInvariant implements AssignableSpec{

    private MethodDeclaration md;
    private TypeDeclaration cd;
    private Services services;

    // contains an assignable keyword (like nothing, everithing, ...), 
    // if one occurd in the specification.
    private String assignableMode = null;
    protected SetOfTerm assignableLocations = SetAsListOfTerm.EMPTY_SET;

    private LoopStatement loop;

    private Term variant = null;
    private Term invariant = null;

    private TermFactory tf = TermFactory.DEFAULT;

    public LoopInvariant(MethodDeclaration md, TypeDeclaration cd,
			 Services services, LoopStatement loop){
	this.services = services;
	this.md = md;
	this.cd = cd;
	invariant = tf.createJunctorTerm(Op.TRUE);
	this.loop = loop;
    }

    public void addInvariant(Term t){
	invariant = tf.createJunctorTermAndSimplify(Op.AND, invariant, t);
    }

    public void setVariant(Term t){
	variant = t;
    }

    public Term getInvariant(){
	return invariant;
    }

    public Term getVariant(){
	return variant;
    }

    public LoopStatement getLoop(){
	return loop;
    }

    public void register(){
	System.out.println("invariant "+invariant+" variant "+variant+" assignable "+assignableLocations);
	services.getImplementation2SpecMap().addLoopSpec(this);
    }

    public void addAssignable(Term t){
	assignableLocations = assignableLocations.add(t);
    }

    public void addAssignable(SetOfTerm as){
	assignableLocations = assignableLocations.union(as);
    }

    public SetOfTerm getAssignable(){
	return assignableLocations;
    }

    public void setAssignableMode(String s){
	assignableMode = s;
    }

    public String getAssignableMode(){
	if(assignableLocations == SetAsListOfTerm.EMPTY_SET &&
	    assignableMode == null){
	    return "everything";
	}
	// an inconsistent case
	if(assignableLocations != SetAsListOfTerm.EMPTY_SET &&
	   "nothing".equals(assignableMode)){
	    return null;
	}
	return assignableMode;
    }

}
