// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

/**
 * Implements "inner renaming", i.e. renaming the newly introduced variable in 
 * case of a name clash.
 */
 
package de.uka.ilkd.key.logic;

import de.uka.ilkd.key.logic.op.ProgramVariable;
import de.uka.ilkd.key.proof.Goal;
import java.util.HashMap;


public class InnerVariableNamer extends VariableNamer {
 
    public ProgramVariable rename(ProgramVariable var,
                                  Goal goal,
                                  PosInOccurrence posOfFind) {
	String basename = getBaseName(var.getProgramElementName());
	HashMap map = new HashMap();

	//prepare renaming of inner var
	ProgramElementName newname = createName(basename,
                                           	getCounter(basename, goal),
                                            	getMethodStack(posOfFind));
	ProgramVariable newvar = new ProgramVariable(newname, 
						     var.getKeYJavaType());
	map.put(var, newvar);

	//execute renaming
	replaceInSequent(map, goal);
	replaceInTacletApps(map, goal);
	
	return newvar;
    }
}
