// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

/**
 * Implements "outer renaming", i.e. renaming the old variable in case the
 * name of a newly introduced variable clashes with it.
 */

package de.uka.ilkd.key.logic;

import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.proof.Goal;
import java.util.HashMap;


public class OuterVariableNamer extends VariableNamer {
    
    private ProgramVariable getConflictingVar(ProgramVariable var, 
                                              ProgramElementName name, 
					      Goal goal) {
        IteratorOfProgramVariable it = goal.getGlobalProgVars().iterator();
        while(it.hasNext()) {
	    ProgramVariable var2 = it.next();
            ProgramElementName name2 = var2.getProgramElementName();
            if(var2 != var && name2.equals(name)) {
            	return var2;
            }
        }

        return null;
    }

    
    public ProgramVariable rename(ProgramVariable var,
                                  Goal goal,
                                  PosInOccurrence posOfFind) {
	String basename	= getBaseName(var.getProgramElementName());
	HashMap map = new HashMap();

	//prepare renaming of inner var
	ProgramElementName newname = createName(basename,
			     			0,
			     			getMethodStack(posOfFind));
	ProgramVariable newvar = new ProgramVariable(newname,
						     var.getKeYJavaType());
	map.put(var, newvar);

	//prepare renaming of outer var, if existent
	ProgramVariable confVar = getConflictingVar(var, newname, goal);
	if(confVar != null) {
	    ProgramElementName confName = confVar.getProgramElementName();
	    ProgramElementName newConfName;
	    newConfName = createName(basename,
	    			     getCounter(basename, goal),
				     confName.getCreationInfo());
	    ProgramVariable newConfVar;
	    newConfVar = new ProgramVariable(newConfName,
	    				     confVar.getKeYJavaType());
	    map.put(confVar, newConfVar);
	}

	//execute renaming
	replaceInSequent(map, goal);
	replaceInGlobals(map, goal);
	replaceInTacletApps(map, goal);

	return newvar;
    }
}
