// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.logic;

import java.io.File;

import junit.framework.TestCase;
import de.uka.ilkd.key.proof.Proof;
import de.uka.ilkd.key.proof.ProofList;
import de.uka.ilkd.key.proof.init.KeYUserProblemFile;
import de.uka.ilkd.key.proof.init.ProblemInitializer;
import de.uka.ilkd.key.util.HelperClassForTests;

/** 
 * class tests the term factory
 */
public class TestUpdatetermNormalisation extends TestCase { 

    public static final String testRules = System.getProperty("key.home")+
        File.separator+"system"+File.separator+"proofExamples"+
        File.separator+"_testcase"+File.separator+"updateterm";

    private TermFactory tf = TermFactory.DEFAULT;

    private final HelperClassForTests helper = new HelperClassForTests();

    public ProofList parse(File file) {
        return helper.parse(file);
    }
   

    public TestUpdatetermNormalisation(String name) {
	super(name);
    }

    public void setUp() {     	
    }

    private Term extractProblemTerm(Proof p) {
        return helper.extractProblemTerm(p);
    }

    public void testLocalVariableSort() {
	ProofList proofList = parse
	    (new File(testRules+File.separator+"updatetermTest0.key"));
	Term t = extractProblemTerm(proofList.get(0));
	assertTrue("Expected " + t.sub(1) + " but is " + t.sub(0),
		   t.sub(0).equals(t.sub(1))); 
    }
    
    public void testLocalVariableAttributeSort() {
    	ProofList proofList = parse
	    (new File(testRules+File.separator+"updatetermTest1.key"));
	Term t = extractProblemTerm(proofList.get(0));
	assertTrue("Expected " + t.sub(1) + " but is " + t.sub(0),
		   t.sub(0).equals(t.sub(1)));
    }

    public void testLocalVariableAttributeArraySort() {
    	ProofList proofList = parse
	    (new File(testRules+File.separator+"updatetermTest2.key"));
	Term t = extractProblemTerm(proofList.get(0));
	assertTrue("Expected " + t.sub(1) + " but is " + t.sub(0),
		   t.sub(0).equals(t.sub(1)));
    }

    public void testLocalVariableAttributeArrayShadowSort() {
    	ProofList proofList = parse
	    (new File(testRules+File.separator+"updatetermTest3.key"));
	Term t = extractProblemTerm(proofList.get(0));
	assertTrue("Expected " + t.sub(1) + " but is " + t.sub(0),
		   t.sub(0).equals(t.sub(1)));
    }
    
}
