// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
//
//


package de.uka.ilkd.key.ocl;

import java.io.*;
import java.util.Vector;
import java.util.HashMap;
import java.util.Iterator;

import de.uka.ilkd.key.casetool.ReprModelClass;
import de.uka.ilkd.key.casetool.ReprModelMethod;


/** Export OCL specifications to a file */
public class OCLExport {

    private File file;
    private ReprModelClass cls[];
    private FileWriter out;

    private final static String SEP = "\n";

    public OCLExport(ReprModelClass cls, FileWriter out){
        this.cls = new ReprModelClass[1];
        this.cls[0] = cls;
        this.out = out;
    }

    public OCLExport(ReprModelClass[] cls, FileWriter out){
        this.cls = cls;
        this.out = out;
    }

    /** get OCL specs (invariant, pre-/posts for methods) for one class */
    private String getOCL(ReprModelClass c)
    {
        String result = "";

        String cName = c.getClassName();

        String inv = c.getMyInv();
        if (inv != null && !inv.equals("")) {
            result += "context " + cName + "\ninv: " + inv + "\n";
        }
        
        Vector ops = c.getOps();
        for (int i = 0; i < ops.size(); i++) {
            ReprModelMethod meth = (ReprModelMethod) (ops.elementAt(i));
            String pre = meth.getMyPreCond();
            String post = meth.getMyPostCond();

            if (pre != null && ! pre.equals("")) { 
                pre = "pre: " + pre + "\n"; 
            } else {
                pre = "";
            }
            if (post != null && ! post.equals("")) {
                post = "post: " + post + "\n";
            } else {
                pre = "";
            }
                        
            if (pre != "" || ! post.equals("")) {
                result += "\ncontext " + cName + "::" +
                            // BUG: getCallSignature does not qualify types
                            // with their respective package 
                            meth.getCallSignature() + "\n";
                result += pre + post;
            }
        }
        
        return result;
    }

    public void export() throws IOException
    {
        // type: key=String (package name), value = String (OCL text)
        // used since we need to group specifications by package in output file
        HashMap packages = new HashMap(); 
        
        // collect OCL specs from all classes in array cls:
        for (int i = 0; i < cls.length; i++) {
            String ocl = getOCL(cls[i]);
            if (ocl != null && ! ocl.equals("")) {
                String pack = cls[i].getContainingPackage();
                if (pack == null || pack.equals("")) { 
                    // put everything in a package by default
                    pack = "NOPACKAGE"; 
                } else {
                    // Java package notation differs from OCL
                    pack = pack.replaceAll("\\.","::");
                }

                String oldPackSpec = (String) packages.get(pack);
                if (oldPackSpec == null) {
                    packages.put(pack, ocl);
                } else {
                    packages.put(pack, oldPackSpec + "\n" + ocl);
                }
            }
        }

        // print all OCL specs in the hashmap packages:
        Iterator it = packages.keySet().iterator();
        while (it.hasNext()) {
            String pack = (String) it.next();
            String spec = (String) packages.get(pack);
            
            if (spec != null && ! spec.equals("")) {
                out.write("package " + pack + "\n" + spec + "\nendpackage\n\n");
            }
        }
    }
}

