// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
//
//

package de.uka.ilkd.key.ocl.gf;


import java.io.*;
import java.net.URL;
import de.uka.ilkd.key.util.KeYResourceManager;


/** @author Kristofer Johannisson */

/** handle the copying of GF OCL grammars from the resource directory
 * to a temporary directory (so that GF can access them) */
class TempGrammarFiles
{

    File absTemp = null;
    File oclTemp = null;
    File engTemp = null;

    /** copy GF OCL grammars from internal resource directory to
	a temporary directory, where GF can access them */
    public TempGrammarFiles() throws IOException
    {
	URL absURL = KeYResourceManager.getManager().
	    getResourceFile(TempGrammarFiles.class, "grammars/all.Abs.gf");
	URL oclURL = KeYResourceManager.getManager().
	    getResourceFile(TempGrammarFiles.class, "grammars/all.OCL.gf");
	URL engURL = KeYResourceManager.getManager().
	    getResourceFile(TempGrammarFiles.class, "grammars/all.Eng.gf");
	if (absURL == null || oclURL == null || engURL == null){
	    // System.err.println("TempGrammarFiles: could not find GF OCL grammars.");
	    throw new IOException("TempGrammarFiles: could not find GF OCL grammars.");
	}
	else {
	    absTemp = File.createTempFile("all.Abs", ".gf");
	    oclTemp = File.createTempFile("all.OCL", ".gf");
	    engTemp = File.createTempFile("all.Eng", ".gf");
	    //	    System.out.println("TempGrammarFiles: tempfiles created");
	    File abs = new File(absURL.getPath());
	    File ocl = new File(oclURL.getPath());
	    File eng = new File(engURL.getPath());
	    // System.out.println("TempGrammarFiles: resource files found");
	    simpleCopy(abs, absTemp);
	    simpleCopy(ocl, oclTemp);
	    simpleCopy(eng, engTemp);
	    // System.out.println("TempGrammarFiles: files copied");
	}
    }



    /** @return absolute paths to grammars which should be imported in
     * the abstract wrapper grammar.
     */
    public String[] getAbsPaths()
    {
	String[] path = new String[] {absTemp.getAbsolutePath()};
	return path;
    }

    /** @return absolute paths to grammars which should be imported in
     * the OCL wrapper grammar.
     */
    public String[] getOclPaths()
    {
	String[] path = new String[] {oclTemp.getAbsolutePath()};
	return path;
    }

    /** @return absolute paths to grammars which should be imported in
     * the English wrapper grammar.
     */
    public String[] getEngPaths()
    {
	String[] path = new String[] {engTemp.getAbsolutePath()};
	return path;
    }

    /** replace the contents of file to with the contents of file from */
    private void simpleCopy(File from, File to) throws IOException
    {
	BufferedReader fromR = new BufferedReader(new FileReader(from));
	PrintWriter toW = new PrintWriter(new BufferedWriter( new FileWriter(to)));
	
	String s = new String();
	while((s=fromR.readLine()) != null) {
	    // System.out.println("** "+s);
	    toW.println(s);
	}
	toW.close();
	fromR.close();
    }

}
