// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.ocl.ocldltranslator.control;

import tudresden.ocl.*;
import tudresden.ocl.parser.analysis.*;
import java.util.Set;
import java.util.HashMap;
import java.util.Map;
import java.util.Iterator;
import java.util.ListIterator;
import java.util.Vector;
import tudresden.ocl.parser.node.*;
import tudresden.ocl.check.types.*;

import de.uka.ilkd.key.ocl.ocldltranslator.*;
import de.uka.ilkd.key.ocl.ocldltranslator.heuristics.*;

/**
 * The basic translation control class as used in the KeY-System.
 * 
 * An instance of this class automatically uses the following
 * translation heuristics:
 *  - Collections as Sets Heuristic
 *  - If-then-else Optimization Heuristic
 *  - Boolean expressions as formulas Heuristic
 */

public class KeyTranslationControl extends DefaultTranslationControl {
  
    /**
     * Initializes a translation control instance for the KeY system.
     */
    public KeyTranslationControl(){
	super();
	addHeuristic(new CollectionsAsSetsHeuristic());
	addHeuristic(new IfThenElseHeuristic());
	addHeuristic(new BooleanExpressionsAsFormulasHeuristic());
	addHeuristic(new AttributesAndMethodsAsFunctionSymbolsHeuristicForKey());
    }
    

}

















