// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.ocl.ocldltranslator.detailedocl;

import de.uka.ilkd.key.ocl.ocldltranslator.*;
import java.util.*;
import tudresden.ocl.parser.node.Node;
import tudresden.ocl.*;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.op.Modality;
import de.uka.ilkd.key.logic.op.ProgramVariable;
import de.uka.ilkd.key.ProverFacade;
import de.uka.ilkd.key.casetool.UMLOCLClassifier;
import de.uka.ilkd.key.casetool.UMLOCLStructuralFeature;
import de.uka.ilkd.key.casetool.HashMapOfClassifier;


public class NullMod extends OCLMod{

    public static final NullMod SINGLETON=new NullMod();

    private String constraint="";
    private final static String intermednull="####";
    private static UMLOCLClassifier nullClass
	= new UMLOCLClassifier("Null", "Null");


    protected String nullAttr(String classId) {
	return XMIFactory.getAttribute("null", "'"+classId+"'");
    }

    protected String nullClass() {
	String classId=XMIFactory.getId();
	String nullAttr= XMIFactory.getAttribute("null", "'"+classId+"'");
	return  XMIFactory.getClassTag("Null",classId,nullAttr);
     
    }

    protected String nullGeneralization() {
	return "";
    }


    public boolean insertXMIAtEndElement(StringBuffer result, String elemName, String currentModElemId) {
 	if (elemName.equals(XMIFactory.CLASSIF_FEATURE)) {
 	    result=result.append(nullAttr(currentModElemId));
 	    return true;
 	}
	return false;
    }


    private void translateNull(KeyOclExpressionTranslator exprTransl, Node n, Node appliedTo) {
	KeyTranslationResult resultTranslation=new KeyTranslationResult(); //(KeyTranslationResult) getTranslationForNode(appliedTo);
	ProverFacade proverFacade = exprTransl.proverFacade();
	resultTranslation.setTerm(proverFacade.getNullTerm());
	exprTransl.setTranslationForNode(n, resultTranslation);
    }


    public boolean translateOCL2DL(KeyOclExpressionTranslator exprTransl, 
				Node n, Node appliedTo,
				String featureName,
				Vector parameterNodes) {
	if (featureName.equals("null") || featureName.endsWith(".null")) {
	    translateNull(exprTransl, n, appliedTo);	    
	    return true;
	}
	return false;
    }

    public JavaBlock getMethodCallJavaBlock(KeyOclConstraintTranslator 
					    consTransl, 
					    String methodReturnType, 
					    ProgramVariable[] argProgVars, 
					    String constrainedOperation, 
					    String constrainedType) {
	return null;
    }

    public TranslationResult getTranslationFor(DetailedConstrTransl 
					       consTransl,
					       OclTree oclTree, 
					       boolean useDiamondForm, 
					       boolean ignorePost,
					       boolean 
					       oclTreeIsSimpleExpression,
					       Modality modality){
	return null;
    }


    private String suffix(String s) {
	boolean stop=false;
	int brcount=0;
	int i;
	for (i=0; i<s.length() && !stop; i++) {
	    if (brcount==0) {
		if (s.charAt(i)==' ') stop=true;
		if (s.charAt(i)==')') stop=true;
	    } else {
		if (s.charAt(i)==')') brcount--;
	    }
	    if (s.charAt(i)=='(') brcount++;
	}
	return s.substring(0, i);
    }


    private String prefix(String s) {
	boolean stop=false;
	int brcount=0;
	int i;
	for (i=s.length()-1; i>=0 && !stop; i--) {
	    if (brcount==0) {
		if (s.charAt(i)==' ') stop=true;
		if (s.charAt(i)=='(') stop=true;
	    } else {
		if (s.charAt(i)=='(') brcount--;
	    }
	    if (s.charAt(i)==')') brcount++;
	}
	return stop ? s.substring(i+2) : s.substring(i+1);
    }

    private String replace(String s, String old, String ne) {
	String result=s;
	int last=result.lastIndexOf(old);
	while (last!=-1) {
	    result=result.substring(0,last)+ne+result.substring(last+old.length());
	    last=result.lastIndexOf(old);
	}
	return result;
    }


    public boolean transformOCL(String constraint) {
	int lastnull=constraint.lastIndexOf("null");
	String wholeModified=constraint;
	while (lastnull!=-1) {
	    String before=wholeModified.substring(0,lastnull).trim();
	    String after=wholeModified.substring(lastnull+4).trim();
	    String modify="";
	    if (after.startsWith("=")) {
		modify=suffix(after.substring(1).trim()).trim();
		//	    String back=after.substring(afterModify.length());
		
	    } else if (before.endsWith("=")) {
		modify=prefix(before.substring(0,before.length()-1).trim()).trim();
		//	    String front=before.substring(0, beforeModify.length()-1);
	    } else {
		System.err.println("null could not be modified!");
	    }
	    wholeModified=before+" "+modify+"."+intermednull+" "+after;	    
	    lastnull=wholeModified.lastIndexOf("null");
	}
	this.constraint=replace(wholeModified, intermednull, "null");
	return !(this.constraint.equals(constraint));
    }

    public String getOCLConstraint() {
	return constraint;
    }

    public void addFeatureToUMLOCLClassifier(UMLOCLClassifier cls) {
    	cls.addFeature
	    ("null", new UMLOCLStructuralFeature("null", nullClass));
    }

    public void modifyAllTypes(HashMapOfClassifier classifiers) {
    	Iterator it = classifiers.values().iterator();
	while (it.hasNext()) {
	    UMLOCLClassifier clf = (UMLOCLClassifier)it.next();
	    nullClass.addSupertype(clf.getName(), clf);	    
	}
	classifiers.putClassifier("Null", nullClass);
    }

}
