// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.ocl.ocldltranslator.transform;

import java.util.*;
import de.uka.ilkd.key.logic.*;

import de.uka.ilkd.key.ocl.ocldltranslator.*;
import de.uka.ilkd.key.ocl.ocldltranslator.heuristics.*;
import de.uka.ilkd.key.ocl.ocldltranslator.control.*;
import de.uka.ilkd.key.ocl.ocldltranslator.normalize.*;
import  de.uka.ilkd.key.ocl.ocldltranslator.transform.*;

/** 
 * Only for use in the generation of the translation of an OCL constaint !!!
 *
 * Interface for classes handling occurences of @pre-tagged
 * function symbols in the formulas that have been generated during the 
 * translation of of a pre/post-constraint.
 */ 
 
public interface KeyAtPreTransformer extends AtPreTransformer {
    
   /**
     * Sets up the transformer with the necessary informations.
     * @param nonrigidFunctionSymbols - the nonrigid function symbols in the given term.
     * @param prepostfSymbolMap - maps all names of function symbols onto the corresponding function symbol
     * @param atPreTaggedAttributeFunctions - maps function symbols for @pre-tagged attributes onto the 
     *                                        corresponding (non-tagged) attribute operator
     * @param constraitTranslator - the constraint translator which was used to construct the term to be transformed.
     */

    void initTransformer(Set nonrigidFunctionSymbols, 
			 Map prepostfSymbolMap, 
			 Map atPreTaggedAttributeFunctions, 
			 Map atPreTaggedMethodFunctions, 
			 KeyOclConstraintTranslator constraitTranslator);
    
    
    
    
}
