// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

/* -*-Antlr-*- */

header {
	package de.uka.ilkd.key.parser;

    import java.io.InputStream;
    import de.uka.ilkd.key.util.*;
}

/**
 * The common lexer for declarations, terms, formulae, Taclets, etc.
 */

class KeYLexer extends Lexer;
options {
	k=2;
    defaultErrorHandler = true;
}

tokens {
	
	//JAVA = "java";
        NEW = "new";
	BRANCH = "branch";
	NOTSIMPLE  = "notsimple";
        INCLUDE = "include";
        LDTS = "LDTs";
        NOSTANDARDRULES = "nostandardrules";
	TYPEOF = "typeof";
	OBJECT = "object";
        NUMBER = "number";
        INTLITERAL = "intliteral";
        LONGLITERAL = "longliteral";
	GENERIC = "generic";
        EXTENDS = "extends";
        ONEOF = "oneof";
	PROGRAM = "program";
	SVLIST   = "svlist";
	FORALL = "all";
	EXISTS = "ex";
	TRUE = "true";
	FALSE = "false";
	FIND = "find";
	IF = "if";
	VARCOND = "varcond";
    EVERYTHING = "everything";
    NOTHING = "nothing";
    ISQUERY = "isquery";
	NONINTERACTIVE = "noninteractive";
	RECURSIVE = "recursive";
        DISPLAYNAME = "displayname";
        HELPTEXT = "helptext";
	NOT = "not";
	FREE = "free";
	IN = "in";
        DEPENDING = "depending";
        ON = "on";
	CLOSE = "close";
	GOAL = "goal";
	REPLACEWITH = "replacewith";
	ADD = "add";
	ADDRULES = "addrules";
	ADDPROGVARS = "addprogvars";
	HEURISTICS = "heuristics";
    HASSORT = "hasSort";
	SORTS = "sorts";
        OPTIONS = "options";
        WITH = "with";
	SCHEMA = "schema";
	VARIABLES = "variables";
	LOCAL  = "local";
	RIGID = "rigid";
	PREDICATES = "predicates";
	FUNCTIONS = "functions";
	FORMULA = "formula";
    	MODAL = "modal";
    	OPERATOR = "operator";
	RULES = "rules";
        PROOF = "proof";
        RULE = "rule";
	BUILTIN = "builtin";
        TERM = "term";
        INST = "inst";
        IFSEQFORMULA = "ifseqformula";
        INTERACTIVE = "interactive";
        HEUR = "heur";
    MODIFIES = "modifies";
    CONTRACTS = "contracts";
        PROBLEM = "problem";
        JML_SPECIFICATIONS = "jml_specifications";
	IN_TYPE = "#inType";
	IS_OBJECT = "#isObject";
	META_IN_LONG ="#inLong";
	META_IN_INT ="#inInt";
	META_IN_CHAR ="#inChar";
	META_IN_SHORT ="#inShort";
	META_IN_BYTE ="#inByte";
	META_ADD = "#add";
	META_SUB = "#sub";
	META_MUL = "#mul";
	META_DIV = "#div";
	META_JDIV = "#jdiv";
	META_MOD = "#mod";
	META_JMOD = "#jmod";
	META_LESS = "#less";
	META_GREATER = "#greater";
	META_LEQ = "#leq";
	META_GEQ = "#geq";
	META_EQ = "#eq";
	META_INT_UNARY_MINUS ="#JavaIntUnaryMinus";
	META_INT_ADD ="#JavaIntAdd";
	META_INT_SUB ="#JavaIntSub";
	META_INT_MUL ="#JavaIntMul";
	META_INT_DIV ="#JavaIntDiv";
	META_INT_MOD ="#JavaIntMod";
	META_INT_AND ="#JavaIntAnd";
	META_INT_OR  ="#JavaIntOr";
	META_INT_XOR ="#JavaIntXor";
	META_INT_COMPLEMENT ="#JavaIntComplement";
	META_INT_SHIFTRIGHT ="#JavaIntShiftRight";
	META_INT_SHIFTLEFT ="#JavaIntShiftLeft";
	META_INT_UNSIGNEDSHIFTRIGHT ="#JavaIntUnsignedShiftRight";
	META_LONG_UNARY_MINUS ="#JavaLongUnaryMinus";
	META_LONG_ADD ="#JavaLongAdd";
	META_LONG_SUB ="#JavaLongSub";
	META_LONG_MUL ="#JavaLongMul";
	META_LONG_DIV ="#JavaLongDiv";
	META_LONG_MOD ="#JavaLongMod";
	META_LONG_AND ="#JavaLongAnd";
	META_LONG_OR  ="#JavaLongOr";
	META_LONG_XOR ="#JavaLongXor";
	META_LONG_COMPLEMENT ="#JavaLongComplement";
	META_LONG_SHIFTRIGHT ="#JavaLongShiftRight";
	META_LONG_SHIFTLEFT ="#JavaLongShiftLeft";
	META_LONG_UNSIGNEDSHIFTRIGHT ="#JavaLongUnsignedShiftRight";
        META_MODULO_BYTE = "#moduloByte";
        META_MODULO_SHORT = "#moduloShort";
        META_MODULO_INTEGER = "#moduloInteger";
        META_MODULO_LONG = "#moduloLong";	
        STATIC_ANALYSIS = "#staticanalysis";	
        WHILE_INV_RULE = "#whileInvRule";	
        RESOLVE_QUERY = "#ResolveQuery";	
        CONSTANT_VALUE = "#constantvalue";	
        META_LENGTH = "#lengthReference";
	    META_NEXT = "#nextReference";
        META_CREATED = "#created";
        META_TRANSIENT = "#transient";	
	META_SHADOW = "#shadowed";
	CONCAT    = "#concat";
	INTRODUCE = "#introduce";
        INTRODUCE_NEW_ANON_UPDATE = "#introNewAnonUpdate";
        WARY_ALL = "waryAll";	
        WARY_EX = "waryEx";	
	SAME = "same";
	COMPATIBLE = "compatible";
        SAME_UPDATE_LEVEL = "sameUpdateLevel";
	SMALLER =  "smaller";
    	THAN = "than";
    	QUOTES = "quotes";
    	THISREFERENCE = "ThisReference";
	KEYLOG = "keyLog";
	KEYLOGUSER = "keyUser";
	KEYLOGVERSION = "keyVersion";
	KEYLOGSETTINGS = "keySettings";
	KEYSETTINGS = "settings";
        OCL = "ocl";
        META_ALLSUBTYPES = "#allSubtypes";
}

{
    protected KeYExceptionHandler keh = new KeYRecoderExcHandler();

    public KeYLexer(InputStream in, KeYExceptionHandler keh){
        this(in);
        this.keh = keh; 
    }
    
    public KeYLexer(Reader in, KeYExceptionHandler keh) {
        this(new CharBuffer(in));
        this.keh = keh; 
    }

    public void reportError(RecognitionException ex){
        keh.reportException(ex);
    }
}

protected
VOCAB
   :       '\3'..'\377'
   ;



SEMI
options {
  paraphrase = "`;'";
} :	';'
    ;

SLASH
options {
  paraphrase = "`/'";
} :	'/'
    ;

//For OCL simplification (prefix of bound variables)
BACKSLASH
options {
  paraphrase = "`\'";
} :	'\\'
    ;
//

COLON
    options {
    paraphrase = "`:'";
} :    ':'
    ;

GUARD
    options {
    paraphrase = "`?='";
} :    "?="
    ;

CONDUP
    options {
    paraphrase = "`/->'";
} :    "/->"
    ;

ASSIGN
    options {
    paraphrase = "`:='";
} :    ":="
    ;

DOT
options {
  paraphrase = "`.'";
}
	:	'.'
	;

COMMA
options {
  paraphrase = "`,'";
}
	:	','
	;

LPAREN
options {
  paraphrase = "`('";
}
	:
	'(' 
	;

RPAREN
options {
    paraphrase = "`)'";
}
:	')'
    ;

LBRACE
options {
  paraphrase = "`{'";
}
	:	'{'
	;

RBRACE
options {
    paraphrase = "`}'";
}
:	'}'
    ;

LBRACKET
options {
    paraphrase = "`['";
}
:	'['
    ;

RBRACKET
options{
    paraphrase = "']'";
}
:	']'
	;

AT
options {
    paraphrase = "`at'";
}
:	'@'
	;

OR
options {
  paraphrase = "`|'";
}
	:	'|'
	;

AND
options {
  paraphrase = "`&'";
}
	:	'&'
	;

NOT
options {
  paraphrase = "`!'";
}
	:	'!'
	;

IMP
options {
  paraphrase = "`->'";
}
	:	"->"
	;

EQV
options {
  paraphrase = "`<->'";
}
	:	"<->"
	;

EQUALS
options {
  paraphrase = "`='";
}
	:	"="
	;

RBRACKETPRIMES
options {
  paraphrase = "] followed by a sequence of primes (at least one)";
}
	:	']' ('\'')+
	;

SEQARROW
options {
  paraphrase = "`==>'";
}
	:	"==>"
	;

COMPUTE_SPEC_OP
options { 
  paraphrase = "'^'";
}
	:	'^'
	;


ARITH_OP
options {
  paraphrase = "java operator";
}
	:	 "%" | "*" | "-"
	;

WS
options {
  paraphrase = "white space";
}
	:	(' '
	|	'\t'
	|	'\n'  { newline(); }
	|	'\r')
		{ $setType(Token.SKIP); }
	;


/*CARD_LITERAL
options {
    paraphrase = "a non-negative number";
}
    :
    (DIGIT)+
    ;
*/

STRING_LITERAL
options {
  paraphrase = "a string in double quotes";
}
    : '"'! ( ESC | ~('"'|'\\') )* '"'! ;



PRIMES_OR_CHARLITERAL
     :
     ('\'' '\'') => PRIMES {$setType(PRIMES);}
    |
     ('\'' '\\') => CHAR_LITERAL {$setType(CHAR_LITERAL);}
    |
     ('\'' ~('\'') '\'') => CHAR_LITERAL {$setType(CHAR_LITERAL);}
    |
     PRIMES {$setType(PRIMES);}
    ;


protected
PRIMES
options {
  paraphrase = "Sequence of primes (at least one)";
}
	:	('\'')+
	;

protected
CHAR_LITERAL 
options {
  paraphrase = "a char in single quotes";
}
    : '\'' 
                ((' '..'&') |
                 ('('..'[') |
                 (']'..'~') |
                 ('\\' ('\'' | '\\' | 'n' | 'r' | 't' | 'b' | 'f' | '"' | 'u' HEX ))
                )
      '\''
 ;

protected
ESC
    :	'\\'
    (	'n'         { $setText("\n"); }
	|	'r' { $setText("\r"); }
	|	't' { $setText("\t"); }
	|	'b' { $setText("\b"); }
	|	'f' { $setText("\f"); }
	|	'"' { $setText("\""); }
	|	'\'' { $setText("'"); }
	|	'\\' { $setText("\\"); }
	|	':' { $setText ("\\:"); }
	|	' ' { $setText ("\\ "); }
    )
    ;

DIAMOND
     : '<'! (WS)* BLOCK (WS)* '>'!
     ;

BOX
     : '['! (WS)* BLOCK (WS)* ']'!
     ;

THROUGHOUT
     : '['!'['! (WS)* BLOCK (WS)* ']'!']'!
     ;

MODALITYARG
    : '{'! BLOCK '}'!
    ;


protected
BLOCK
 :
	'{' (
		options {
			greedy = false; // exit upon '}'
		}
	:
		(
			options {
				generateAmbigWarnings = false; // shut off newline warning
			}
		:	'\r' '\n'	{newline();}
		|	'\r' 		{newline();}
		|	'\n'		{newline();}
		)
	|	BLOCK
	|	SL_COMMENT
	|	ML_COMMENT
	|	QUOTED_STRING_LITERAL
	|	.
	)* '}'
   ;


protected
QUOTED_STRING_LITERAL
options {
  paraphrase = "a string with double quotes";
}
    : '"' ('\\' . | ~('"'|'\\') )* '"' ;
    
SL_COMMENT
options {
  paraphrase = "a comment";
}

	:
	"//"
	(~'\n')* '\n'
	{ $setType(Token.SKIP); newline(); }
	;

ML_COMMENT
options {
  paraphrase = "a comment";
}
	:
	"/*"
	(
	        '\n' { newline(); }
	 |	'*' ~'/'
	 |	~'*'
	)*
	"*/"
	{ $setType(Token.SKIP);  }
	;

protected
DIGIT
options {
  paraphrase = "a digit";
}
	:	'0'..'9'
	;


protected 
HEX
options {
    paraphrase = "a hexadeciamal number";
}
:	('a'..'f'|'A'..'F'|DIGIT )
	('a'..'f'|'A'..'F'|DIGIT )
	('a'..'f'|'A'..'F'|DIGIT )
	('a'..'f'|'A'..'F'|DIGIT )
    ;

protected
LETTER
options {
    paraphrase = "a letter";
}
:	'a'..'z'|'A'..'Z' 
    ;


protected IDCHAR 
options {
	paraphrase = "an admissible character for identifiers";
}
	: LETTER | DIGIT | '_' | '+' | '~' 

    //For OCL simplification
    | '$'
    //
	;

protected PIDCHAR 
options {
	paraphrase = "an admissible character for identifiers protected with ``";
}
	: LETTER | DIGIT | '_' | '+' | '~' | '.'
	;

PIDENT
    : '`'! (PIDCHAR)+ '`'!
    ;

IDENT
options {
    testLiterals = true;
    paraphrase = "an identifer";
}

:       "#" (IDCHAR)* |  IDCHAR (IDCHAR | (AT "pre"))* | "<" (IDCHAR)+ ">"
	;



//IDENT
//options {
//    testLiterals = true;
//    paraphrase = "an identifer";
//}

//:   NONSVIDENT | SVIDENT
//    ;


// NONSVIDENT
// options {
//     testLiterals = true;
//     paraphrase = "an identifer";
// }

// :   IDCHAR (IDCHAR | (AT "pre"))*    
// 	;

// SVIDENT
// options {
//     testLiterals = true;
//     paraphrase = "an identifer";
// }

// :       ("#" IDCHAR (IDCHAR)*)
// 	;

/*
NUM_LITERAL
options {
    paraphrase = "a number";
}
    : 
        //"-"(DIGIT)+ | (DIGIT)+    ;
        (DIGIT)+    ;
*/

