// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

/* -*-antlr-*- */
header {

package de.uka.ilkd.key.parser;

import java.io.*;
import antlr.*;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.sort.*;
import de.uka.ilkd.key.logic.sort.oclsort.OclSort;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.rule.*;
import de.uka.ilkd.key.proof.RuleSource;
import de.uka.ilkd.key.proof.init.Includes;
import de.uka.ilkd.key.jml.*;
import de.uka.ilkd.key.pp.AbbrevMap;

import de.uka.ilkd.key.rule.metaconstruct.*;
import de.uka.ilkd.key.java.JavaReader;
import de.uka.ilkd.key.java.JavaInfo;
import de.uka.ilkd.key.java.Services;

import de.uka.ilkd.key.java.abstraction.KeYJavaType;
import de.uka.ilkd.key.java.abstraction.ListOfKeYJavaType;
import de.uka.ilkd.key.java.abstraction.SLListOfKeYJavaType;
import java.util.List;
import java.util.LinkedList;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.HashMap;
import de.uka.ilkd.key.java.visitor.DeclarationProgramVariableCollector;
import de.uka.ilkd.key.java.Recoder2KeY;
import de.uka.ilkd.key.java.Services;
import de.uka.ilkd.key.java.recoderext.KeYCrossReferenceServiceConfiguration;
import de.uka.ilkd.key.util.*;
import de.uka.ilkd.key.gui.*;
}

/** 
 * A parser for first-order terms and formulae.
 */  
class TermParser extends DeclParser;
options {
    importVocab=KeYLexer;
    k = 1;
    defaultErrorHandler=true;
}


{
    
    static final Sort[] AN_ARRAY_OF_SORTS = new Sort[0];
    static final Term[] AN_ARRAY_OF_TERMS = new Term[0];

    protected Choice defaultChoice = null;
    protected HashMap category2Default = new HashMap();
    protected boolean parse_includes = false;
    protected Includes includes = new Includes();
    protected boolean onlyWith=false;
    protected SetOfChoice activatedChoices = SetAsListOfChoice.EMPTY_SET;
    protected HashSet usedChoiceCategories = new HashSet();
    protected NamespaceSet nss;


    // these variables are set if a file is read in step by
    // step. This used when reading in LDTs because of cyclic
    // dependencies.
    private boolean skip_schemavariables;
    private boolean skip_functions;
    private boolean skip_predicates;
    private boolean skip_sorts;

    protected KeYExceptionHandler keh = null;

    private boolean skip_rulesets;

    private Services services;
    private AbbrevMap scm;

    private int lineOffset=0;
    private int colOffset=0;

    private TermFactory tf;
    private JavaReader javaReader;
    
    private boolean errors = false;

    public TermParser(TokenStream  lexer,                   
                      String       filename,
                      TermFactory  tf,
                      JavaReader   jr,
                      Services     services,
                      NamespaceSet nss,
                      AbbrevMap    scm) {
        this(lexer,2);
        setFilename(filename);
        this.javaReader = jr;
        this.nss = nss;
        this.tf  = tf;
        this.services = services;
        this.scm = scm;
        switchToNormalMode();
        this.keh = services.getExceptionHandler();
    }

    public void reportError(RecognitionException ex){
        keh.reportException(ex);
    }

    public void setKeYExceptionHandler(KeYExceptionHandler handler){
        keh = handler;
    }

    protected int getLine() {
        int line = -1;
        try {
            line = LT(0).getLine()+lineOffset;
        } catch (TokenStreamException e) {
            System.err.println("No further token in stream");
        }
        return line;
    }   

    public JavaInfo getJavaInfo() {
        return getServices().getJavaInfo();
    }

    public Services getServices() {
        return services;
    }

    private Namespace sorts() {
        return nss.sorts();
    }

    private Namespace variables() {
        return nss.variables();
    }

    private Namespace programVariables() {
        return nss.programVariables();
    }

    private Namespace functions() {
        return nss.functions();
    }

    private Namespace ruleSets() {
        return nss.ruleSets();
    }

    private Namespace choices(){
        return nss.choices();
    }

    private NamespaceSet namespaces() {
        return nss;
    }

    public Named lookup(Name n) {
        final Namespace[] lookups = {
            functions(), variables(), 
            programVariables()
        };
        for (int i = 0; i<lookups.length; i++) {
            if (lookups[i].lookup(n) != null) {
                return lookups[i].lookup(n);
            }
        }
        return null;    
    }

    // a pure term parser is never in schemamode
    // only grammers that extend this grammar may 
    // activate the schema mode

    public void switchToSchemaMode() {
    }

    private boolean inSchemaMode() {
        return false;
    }

    public void switchToNormalMode() {
    }


    public static Term toZNotation(String number, Namespace functions, TermFactory tf){
	String s = number;
        final boolean negative = (s.charAt(0) == '-');
	if (negative) {
	    s = number.substring(1, s.length());
	}

        Term result = tf.createFunctionTerm((Function) functions.lookup(new Name("#")));

        for(int i = 0; i<s.length(); i++){
            result = tf.createFunctionTerm((Function)functions.lookup
                 (new Name(s.substring(i,i+1))), result);
        }

       	if (negative) {
  	    result = tf.createFunctionTerm
		((Function) functions.lookup(new Name("neg")), result);
        }
	return tf.createFunctionTerm
            ((Function) functions.lookup(new Name("Z")), result); 
    }


    protected void addInclude(String filename, boolean relativePath, boolean ldt){
        InputStream input=null;
        RuleSource source=null;
        if(relativePath){
            int end = getFilename().lastIndexOf(File.separator);
            int start = 0;
            filename = filename.replace('/', File.separatorChar);
            filename = filename.replace('\\', File.separatorChar);
            if(getFilename().startsWith("file:")){
                start = 5;
            }
            File path=new File(getFilename().substring(start,end+1)+filename);
            try { 
                source = RuleSource.initRuleFile(path.toURL()); 
            } catch(java.net.MalformedURLException e){
                System.err.println("Exception due to malformed URL of file "+
                                   filename+"\n " +e);
            }
        }else{
            source = RuleSource.initRuleFile(filename+".key"); 
        }
        if(ldt){
            includes.putLDT(filename, source);
        }else{
            includes.put(filename, source);
        }
    }  

    protected int getColumn() {
        int     col = -1;
        try {
            col = LT(0).getColumn()+colOffset;
        } 
        catch (TokenStreamException e) {
            System.err.println("No further token in stream");
        }
        return col;
    }   

    protected LogicVariable bindVar(Token id,Sort s) {
        LogicVariable v=new LogicVariable(new Name(id.getText()), s);
        namespaces().setVariables(variables().extended(v));
        return v;
    }

    protected TermSymbol progVar(Token id, Sort s) throws NotDeclException{
        TermSymbol v = (TermSymbol)
            programVariables().lookup(new Name(id.getText()));

        if (v == null) {
            throw new NotDeclException("program variable", id,
                                       getFilename());
        }
        return v;
    }

    protected static boolean isNumber(String s){
        if (s==null){
           return false; 
        }
	
        for (int i=0; i<s.length(); i++){
           if ((s.charAt(i)<'0' || s.charAt(i)>'9') && 
		!(i==0 && s.charAt(i)=='-')) {
             return false;
           }
        }        
        return true;
    }

    protected KeYJavaType getTypeByClassName(String s){
        KeYJavaType kjt = null;
        try {
            kjt = getJavaInfo().getTypeByClassName(s);
        } catch(RuntimeException e){
            return null;
        } 
        return kjt;
    }

    protected boolean isPackage(String name){
        try {   
            return getJavaInfo().isPackage(name);
        } catch(RuntimeException e){        
            // may be thrown in cases of invalid java identifiers
            return false;
        } 
    }

    
    protected void unbindVars() {
        namespaces().setVariables(variables().parent());
    }

    protected void bindProgVars(HashSet progVars) {
        namespaces().setProgramVariables(new Namespace(programVariables()));
        Iterator it=progVars.iterator();
        while (it.hasNext()) {
            programVariables().add((IProgramVariable)it.next());
        }
    }

    protected void unbindProgVars() {
        namespaces().setProgramVariables(programVariables().parent());
    }

    protected static HashSet progVars(JavaBlock jb) {
        if (jb==JavaBlock.EMPTY_JAVABLOCK) {
            return new HashSet();
        }   
        DeclarationProgramVariableCollector pvc
            = new DeclarationProgramVariableCollector(jb.program());
        pvc.start();
        return pvc.result();
    }


    protected Term termForParsedVariable(ParsableVariable v) 
        throws antlr.SemanticException {
        if ( v instanceof LogicVariable ) {
            return tf.createVariableTerm((LogicVariable)v);
        } else if ( v instanceof ProgramVariable ) {
            return tf.createVariableTerm((ProgramVariable)v);
        } else if ( v instanceof Metavariable ) {
            return tf.createFunctionTerm((Metavariable)v);
        } else {
            throw new KeYSemanticException
		(v + " is an unknown kind of variable.", 
			getFilename(), getLine(), getColumn());
        }
    }

    public Term createAttributeTerm(Term prefix, 
                                    TermSymbol attribute,
                                    int shadowPrimes) {
        Term result = prefix;
        if (((ProgramVariable)attribute).isStatic()){
            result = tf.createVariableTerm((ProgramVariable)attribute);
        } else {
            if (shadowPrimes > 0) {
                Term shadownum = TermParser.toZNotation("" + shadowPrimes, functions(), tf);
                result = tf.createShadowAttributeTerm((ProgramVariable)attribute, 
                                                      result, shadownum);
            } else {
                result = tf.createAttributeTerm((ProgramVariable)attribute, result);
            }
        }
        return result;
    }



    public TermSymbol getAttribute(Sort prefixSort, 
                                   String attributeName) 
        throws antlr.SemanticException{
        final JavaInfo javaInfo = getJavaInfo();
        TermSymbol result = null;

        if (attributeName.indexOf(':') != -1) {     
            result = javaInfo.getAttribute
                (attributeName, 
                 javaInfo.getKeYJavaType
                 (attributeName.substring(0, 
                                          attributeName.indexOf(':'))));
        } else {
            result = javaInfo.getAttribute(attributeName,(ObjectSort) prefixSort);
        }

        if ( result == null && !("length".equals(attributeName)) ) {
            throw new NotDeclException
                ("Attribute ", attributeName, getFilename(), 
                 getLine(), getColumn());
        }

        return result;
    }

    public de.uka.ilkd.key.logic.op.Location[] extractLocations(List locNames)
        throws antlr.SemanticException {
	de.uka.ilkd.key.logic.op.Location[] vars = 
	    new de.uka.ilkd.key.logic.op.Location[locNames.size()];
	for (int i = 0; i<vars.length; i++) {
	    final String locName = (String)locNames.get(i);
	    Object o = programVariables().lookup(new Name(locName));
	    if (o != null) {
		vars[i] = (de.uka.ilkd.key.logic.op.Location) o;
	    } else {
		vars[i] = (de.uka.ilkd.key.logic.op.Location) getAttribute(null, locName);
	    }
	}
	return vars;
    }

    
    protected JavaBlock getJavaBlock(Token t) throws antlr.SemanticException {
        String s=t.getText();
        try {
            return javaReader.readBlockWithProgramVariables(programVariables(), s);
        } catch (de.uka.ilkd.key.java.PosConvertException e) {
            lineOffset=e.getLine()-1;
            colOffset=e.getColumn()+1;
            throw new JavaParserException(e.getMessage(), t, 
                getFilename(), lineOffset, colOffset);
            
        } catch (de.uka.ilkd.key.java.ConvertException e) { 
            if (e.parseException()!=null) {               
                lineOffset=e.parseException().currentToken.next.beginLine;               
                colOffset=e.parseException().currentToken.next.beginColumn;
                e.parseException().currentToken.next.beginLine=getLine()-1;
                e.parseException().currentToken.next.beginColumn=getColumn();
                throw new JavaParserException(e.parseException().getMessage(), t, 
                    getFilename(), -1, -1);  // row/columns already in text
            }       
            if (e.proofJavaException()!=null
            &&  e.proofJavaException().currentToken != null
            &&  e.proofJavaException().currentToken.next != null) {       
                lineOffset = 
                e.proofJavaException().currentToken.next.beginLine-1;
                colOffset=e.proofJavaException().currentToken.next.beginColumn;
                e.proofJavaException().currentToken.next.beginLine=getLine();
                e.proofJavaException().currentToken.next.beginColumn =
                getColumn();
                 throw  new JavaParserException(e.proofJavaException().
                    getMessage(), t, getFilename(), lineOffset, colOffset); 
                            
            }   
            throw new JavaParserException(e.getMessage(), t, getFilename());
        } 
    }

    
    public void schema_var_decl(String name,Sort s,
                                boolean makeVariableSV,
                                boolean makeDependingSV,
                                boolean rigid,
                                boolean listModifier)
        {
            if (!skip_schemavariables) {
                SchemaVariable v;
                if ( s == Sort.FORMULA ) {
                    v = SchemaVariableFactory.createFormulaSV
                        (new Name(name), listModifier, rigid);
                } else if ( s instanceof ProgramSVSort ) {
                    v = SchemaVariableFactory.createProgramSV
                        (new ProgramElementName(name),(ProgramSVSort) s, listModifier);
                } else {
                    if ( makeVariableSV ) {
                        v=SchemaVariableFactory.createVariableSV
                            (new Name(name), s, listModifier);
                    } else if ( makeDependingSV ) {
                        v = SchemaVariableFactory.createDependingSV
                            (new Name(name), s, listModifier);
                    } else { v = SchemaVariableFactory.createTermSV
                            (new Name(name), s, listModifier, rigid);
                    }
                }
                if (inSchemaMode()) {
                    variables().add(v);
                }
            }

        }


    protected Operator lookupVarfuncId(String varfunc_name, 
                    PairOfTermArrayAndBoundVarsArray args) throws NotDeclException {
        Term[] argTerms = null;
        if (args != null) {
            argTerms = args.getTerms();
        }
        return lookupVarfuncId(varfunc_name, argTerms);
    }

    /** looks up a function, (program) variable or static query of the 
     * given name varfunc_id and the argument terms args in the namespaces 
     * and java info. 
     * @param args is null iff no argument list is given, for instance `f', 
     * and is an array of size zero, if an empty argument list was given,
     * for instance `f()'.
     */
    protected Operator lookupVarfuncId(String varfunc_name, Term[] args) 
        throws NotDeclException{
        // case 1: variable
        Operator v = (TermSymbol) variables().lookup(new Name(varfunc_name));
        if (v != null && args==null) {
            return v;
        }
        // case 2: function
        v = (TermSymbol) functions().lookup(new Name(varfunc_name));
        if (v != null) { // we allow both args==null (e.g. `c')
                         // and args.length=0 (e.g. 'c())' here 
            return v;
        }
        // case 3: static query
        if (varfunc_name.lastIndexOf("::")>0 && args!=null) {
            String classname = varfunc_name.substring
                (0, varfunc_name.lastIndexOf("::"));
            String name = varfunc_name.substring
                (varfunc_name.lastIndexOf("::")+2);
            try {
                v = getServices().getJavaInfo().getProgramMethodTerm
                        (null, name, args, classname).op();
            } catch(TermCreationException e){
            } 
            if (v != null) {
                return v;
            }
            KeYJavaType kjt = 
            getServices().getJavaInfo().getKeYJavaTypeByClassName(classname);
            JMLClassSpec cs = 
                getServices().getImplementation2SpecMap().getSpecForClass(kjt);
            if(cs != null){
                try {
                    v = cs.lookupModelMethod(new Name(varfunc_name));
                }catch(AmbigiousModelElementException e){
                    e.printStackTrace();
                }
                if (v != null) {
                    return v;
                }
            }
        }
        // case 4: program variable
        v = (ProgramVariable) programVariables().lookup
            (new ProgramElementName(varfunc_name));
        if (v != null && args==null) {
            return v;
        }
        if (args==null) {
            throw new NotDeclException
                ("(program) variable or constant", varfunc_name,
                 getFilename(), getLine(), getColumn());
        } else {
            throw new NotDeclException
                ("function or static query", varfunc_name,
                 getFilename(), getLine(), getColumn());
        }
    }

}

/**
 * In the special but important case of Taclets, we don't yet know
 * whether we are going to have a term or a formula, and it is hard
 * to decide a priori what we are looking at.  So the `term' 
 * non-terminal will recognize a term or a formula.  The `formula'
 * reads a formula/term and throws an error if it wasn't a formula.
 * This gives a rather late error message. */

top {Term t;} : t=formula;

formula returns [Term a = null] 
    :
        a = term 
        {
            if (a != null && a.sort() != Sort.FORMULA ) {
                throw new KeYSemanticException
		  ("Just Parsed a Term where a Formula was expected",
		   getFilename(), getLine(), getColumn());
            }
        }
    ;


term returns [Term a = null] 
    :
        a=term20 
    ;

modifier_list returns [Term l = tf.createFunctionTerm(Op.NOTHING);]
{
    Term t;
}:
        (
            t=modifies_element 
            {
                l = tf.createFunctionTerm(Op.LIST, t, l);
            }
            (
                COMMA t=modifies_element 
                {
                    l = tf.createFunctionTerm(Op.LIST, t, l);
                }
            )*
        )?
;

modifies_element returns [Term t = null]:
        t=term70
    |
        EVERYTHING {t = tf.createFunctionTerm(Op.EVERYTHING);}
    |
        NOTHING {t = tf.createFunctionTerm(Op.NOTHING);}
    ;

term20 returns [Term a = null] 
{
    Term a1;
}
    :   a=term30 
        (EQV a1=term20 
            { a = tf.createJunctorTerm(Op.EQV, new Term[]{a, a1});} )?
; exception
        catch [TermCreationException ex] {
              keh.reportException
		(new KeYSemanticException
			(ex.getMessage(), getFilename(), getLine(), getColumn()));
        }



term30 returns [Term a = null] 
{
    Term a1;
}
    :   a=term40 
        (IMP a1=term30 
            { a = tf.createJunctorTerm(Op.IMP, new Term[]{a, a1});} )?
; exception
        catch [TermCreationException ex] {
              keh.reportException
		(new KeYSemanticException
			(ex.getMessage(), getFilename(), getLine(), getColumn()));
        }



term40 returns [Term a = null] 
{
    Term a1;
}
    :   a=term50 
        (OR a1=term40 
            { a = tf.createJunctorTerm(Op.OR, new Term[]{a, a1});} )?
; exception
        catch [TermCreationException ex] {
              keh.reportException
		(new KeYSemanticException
			(ex.getMessage(), getFilename(), getLine(), getColumn()));
        }



term50 returns [Term a = null] 
{
    Term a1;
}
    :   a=term60 
        (AND a1=term50
            { a = tf.createJunctorTerm(Op.AND, new Term[]{a, a1});} )?
; exception
        catch [TermCreationException ex] {
              keh.reportException
		(new KeYSemanticException
			(ex.getMessage(), getFilename(), getLine(), getColumn()));
        }




term60 returns [Term a = null] 
{
    Term a1,a2;
    Sort s;
}
    :  
        NOT a1  = term60 { a = tf.createJunctorTerm(Op.NOT,new Term[]{a1}); }
    |   COMPUTE_SPEC_OP a1 = term60 { a = tf.createJunctorTerm(Op.COMPUTE_SPEC_OP,new Term[]{a1}); }
    |   a = bindingterm 
    |   a = dlTermOrTerm70
; exception
        catch [TermCreationException ex] {
              keh.reportException
		(new KeYSemanticException
			(ex.getMessage(), getFilename(), getLine(), getColumn()));
        }


/**
 * this is a hook that can be redefined by inheriting grammars to
 * redefine the sorts of things that can appear inside a diamond or
 * box.  In particular, in Taclets, schema variables and pre/suffixes
 * might occur. 
 * And to get the parser deterministic it passes through a term70
 */
dlTermOrTerm70 returns [Term a = null] 
{}
    :
        a = normal_dl_term
    |  (IDENT MODALITYARG)=> a = named_dl_term
    |  a = term70
    ; 


term70 returns [Term a = null] 
{
    Term a1;
}
    :
        a = accessterm
        // a term like {o:=u}x=y is parsed as {o:=u}(x=y)
        ((EQUALS)=>EQUALS a1 = logicTermReEntry
            { a = tf.createEqualityTerm(a, a1);})?
 ; exception
        catch [TermCreationException ex] {
              keh.reportException
		(new KeYSemanticException
			(ex.getMessage(), getFilename(), getLine(), getColumn()));
        }


/**
 * helps to better distinguish if formulas are allowed or only term are
 * accepted
 */
logicTermReEntry returns [Term result = null]
{}
    :
        (
            result = accessterm |
            result = update_or_substitution
        ) 
        {
            if (result.sort() == Sort.FORMULA) {
                throw  new KeYSemanticException
                    ("Only terms may stand here.", getLine(), getColumn(), 
                     getFilename());
            }
        }          
    ;

classReference returns [String classReference = ""]
{}:
        
        id:IDENT {
            classReference = id.getText(); 
            while (isPackage(classReference)) {
                match(DOT);
                classReference += "." + LT(1).getText();
                match(IDENT);
            }      
            
            if (getJavaInfo().getTypeByClassName(classReference) == null) {
                throw new NotDeclException
                    ("class " + classReference + " is unknown.", 
                     classReference, getFilename(), getLine(), 
                     getColumn());
            }
        }  
    ;

static_attribute_suffix[String className] returns [Term result = null]
{
    TermSymbol v = null;
    int shadowPrimes = 0;    
    String attributeName = "";
}    
    :   
        DOT attributeName = attrid
        {   v = getAttribute(getTypeByClassName(className).
                             getSort(), attributeName); }   

        ((ps:PRIMES 
                { shadowPrimes = ps.getText().length();})?
            {
                result = createAttributeTerm(null, v, shadowPrimes);
            }
        )
 ; exception
        catch [TermCreationException ex] {
              keh.reportException
		(new KeYSemanticException
			(ex.getMessage(), getFilename(), getLine(), getColumn()));
        }


attribute_or_query_suffix[Term prefix] returns [Term result = null]
{
    TermSymbol v = null;
    int shadowPrimes = 0;
    result = prefix;
    String attributeName = "";    
}    
    :   
        DOT 
        ((id LPAREN)=>( result = query[prefix])
        | 
            attributeName = attrid 
            {   v = getAttribute(prefix.sort(), attributeName); }   
            ((ps:PRIMES 
                    { shadowPrimes = ps.getText().length();})?

                {
                    result = createAttributeTerm(prefix, v, shadowPrimes);
                }        
            ))
 ; exception
        catch [TermCreationException ex] {
              keh.reportException
		(new KeYSemanticException
			(ex.getMessage(), getFilename(), getLine(), getColumn()));
        }


query [Term prefix] returns [Term result = null] 
{
    String classRef = "";
    Term[] args = null;
    PairOfTermArrayAndBoundVarsArray argsWithBoundVars = null; 
}
    :

    classRef = classReference COLON COLON mid:IDENT argsWithBoundVars = argument_list
    { 
       if (argsWithBoundVars != null) {
         args = argsWithBoundVars.getTerms();
       }
       result = getServices().getJavaInfo().getProgramMethodTerm
            (prefix, mid.getText(), args, classRef);
    }        
 ; exception
        catch [TermCreationException ex] {
              keh.reportException
		(new KeYSemanticException
			(ex.getMessage(), getFilename(), getLine(),getColumn()));

        }



accessterm returns [Term result = null] 
{
    KeYJavaType kjt = null;
    Sort arraySort = null;
    Term indexTerm = null;
    int shadowPrimes = 0;
    String className = null;
}
    :
        (LPAREN logicTermReEntry (COMMA | GUARD)) => result = conditional
    |
        {"-".equals(LT(1).getText()) && 
	!TermParser.isNumber(LT(2).getText())}? neg:ARITH_OP result = logicTermReEntry
        {
            if (result.sort() != Sort.FORMULA) {
                result = tf.createFunctionTerm
                ((Function) functions().lookup(new Name("~m")), result);
            } else {
                throw new KeYSemanticException
                ("Formula cannot be prefixed with: ", neg, getFilename());
            }
        } 
    | ({LA(2) == DOT && ( isPackage ( LT(1).getText() ) || 
                ( kjt = getTypeByClassName(LT(1).getText()))!=null )}?

        className = classReference
        result = static_attribute_suffix[className]
      | 	
        result = term80)
        (LBRACKET indexTerm = logicTermReEntry
            ( RBRACKET
            | rb:RBRACKETPRIMES {
                    shadowPrimes = rb.getText().length()-1;                 
              }
            )
            {
                if (arraySort != null && 
                    arraySort != ProgramSVSort.VARIABLE) {
                    if (!(arraySort instanceof ArraySort)){
                        throw new NotDeclException
                            ("array "+result+" declared with a different"+
                             " dimension than required here", ""+result, 
                             getFilename(), getLine(), getColumn());
                        } else {
                            arraySort = ((ArraySort) arraySort).elementSort();
                        } 
                    } else if (arraySort == ProgramSVSort.VARIABLE && !inSchemaMode()) {
                        throw new KeYSemanticException
                        ("Schemavariable are not allowed in this context.", 
			  getFilename(), getLine(), getColumn());
                }
                if (shadowPrimes>0) {
                    result = tf.createShadowArrayTerm
                        (ShadowArrayOp.getShadowArrayOp(result.sort()), result, indexTerm, 
                         TermParser.toZNotation("" + shadowPrimes, functions(), tf));
                } else {
                    result = tf.createArrayTerm(ArrayOp.getArrayOp(result.sort()), result, indexTerm);
                }   
            }
        | result = attribute_or_query_suffix[result]
        )*
 ; exception
        catch [TermCreationException ex] {
              keh.reportException
		(new KeYSemanticException
			(ex.getMessage(), getFilename(), getLine(), getColumn()));
        }


accesstermlist returns [HashSet accessTerms = new HashSet()] {Term t = null;}:
     (t=accessterm {accessTerms.add(t);} ( COMMA t=accessterm {accessTerms.add(t);})* )? ;


term80 returns [Term a = null]
    :
        a = funcpredvarterm
    |   a = specialTerm
    |   LPAREN a = term RPAREN 
    |   "true"  { a = tf.createJunctorTerm(Op.TRUE); }
    |   "false" { a = tf.createJunctorTerm(Op.FALSE); }

    //Used for OCL Simplification.
    |   literal:STRING_LITERAL
        {
            String s = "\"" + literal.getText() + "\""; 
            Name name = new Name(s);
            TermSymbol stringLit = (TermSymbol)functions().lookup(name);
            if (stringLit == null) {
                stringLit = new Function(name, 
                        OclSort.STRING, new Sort[0]);
                functions().add((Function)stringLit);
            }
            a = tf.createFunctionTerm(stringLit);
        }
   ; exception
        catch [TermCreationException ex] {
              keh.reportException
		(new KeYSemanticException
			(ex.getMessage(), getFilename(), getLine(), getColumn()));
        }


abbreviation returns [Term a=null]
{ 
    String sc = null;
}
    :
        (   sc = id
            {
                a =  scm.getTerm(sc);
                if(a==null){
                    throw new NotDeclException
                        ("abbreviation", sc, 
                         getFilename(), getLine(), getColumn());
                }                                
            }
        )
    ;

conditional returns [Term result = null] 
{ 
    Term comparator = null;
}
    :
        LPAREN comparator = logicTermReEntry 
        ( result=normalConditional[comparator] | result=conjunctionConditional[comparator])
    ;

normalConditional[Term cond1] returns [Term result = null] 
{ 
    Term cond2 = null;
    Term c_then = null;
    Term c_else = null;
}
    :
        GUARD cond2 = logicTermReEntry RPAREN
        (
            DOT result = shortCond[cond1, cond2]
        | 
            c_then = logicTermReEntry COLON c_else = logicTermReEntry {
                result = tf.createIfElseTerm(cond1, cond2, c_then, c_else);
            }

        ) 
 ; exception
        catch [TermCreationException ex] {
              keh.reportException
		(new KeYSemanticException
			(ex.getMessage(), getFilename(), getLine(), getColumn()));
        }


// shortCond is overwritten in taclets.g
shortCond[Term cond1, Term cond2] returns [Term result = null]
{
    ParsableVariable v=null;
    Term c_then = null;
}:
        DOT v = varId CONDUP c_then = logicTermReEntry {
            result = tf.createIfElseTerm((ProgramVariable)v, cond1, 
                                         cond2, c_then);
        }
 ; exception
        catch [TermCreationException ex] {
              keh.reportException
		(new KeYSemanticException
			(ex.getMessage(), getFilename(), getLine(), getColumn()));
        }



conjunctionConditional[Term cond1_1] returns [Term a = null]
{ 
    Term cond1_2 = null;
    Term cond2_1 = null;
    Term cond2_2 = null;
    Term c_then = null;
    Term c_else = null;
} :
        COMMA cond1_2=logicTermReEntry COLON c_else=logicTermReEntry 
        GUARD cond2_1 = logicTermReEntry 
        COMMA cond2_2=logicTermReEntry COLON c_then=logicTermReEntry 
        RPAREN
        {
            a = tf.createConjCondTerm(new Term[]{cond1_1, cond1_2, c_else, 
                                                 cond2_1, cond2_2, c_then});
        }        
  ; exception
        catch [TermCreationException ex] {
              keh.reportException
		(new KeYSemanticException
			(ex.getMessage(), getFilename(), getLine(), getColumn()));
        }



/**
 * this is a hook that can be redefined by inheriting grammars to
 * introduce other sorts of binding behaviour for quantifiers or
 * substitution terms.  In particular, the Taclet parser will hook in
 * here to allow quantification of schema variables.
 */
bindingterm returns [Term a = null]
    :
        a = local_bindingterm             
    ;


//Is overridden in taclets.g for permitting bound vars to be schema variables.
//"TermWithBoundVars" is used here instead of "Term" to allow for
//syntax needed for OCL simplification.
argument returns [TermWithBoundVars p = null]
{
    Sort s = null;
    List listVars = new LinkedList();
    LogicVariable v = null;
    LogicVariable[] arrayVars = null;
    ArrayOfQuantifiableVariable vars = null;
    Term a1;
}
    :

        (
            BACKSLASH v0:IDENT COLON s=sortId
            {
                v = bindVar(v0, s); 
                listVars.add(v);
            }
        )* 
        SLASH a1=term
        {
            arrayVars = (LogicVariable[])listVars.toArray(new LogicVariable[listVars.size()]);
            vars = new ArrayOfQuantifiableVariable(arrayVars);
            p = new TermWithBoundVars(vars, a1);
            unbindVars();
        }
    | a1=term
        {
            p = new TermWithBoundVars(vars, a1);
        }
    ;


/** here is the standard behaviour, that declares a new variable
  * with local scope in a quantifier */

local_bindingterm returns [Term a = null]
{
    Operator op = null;
    LogicVariable v = null;
    Term a1;
    Sort s;
}
    :
        (
            "all" { op = Op.ALL; }
        |   "ex"  { op = Op.EX;  }
        )
        v0:IDENT COLON s=sortId DOT {v = bindVar(v0,s);} a1 = term60
        {
            a = tf.createQuantifierTerm((Quantifier)op, v,a1);
            unbindVars();
        }
    | a = update_or_substitution
 ; exception
        catch [TermCreationException ex] {
              keh.reportException
		(new KeYSemanticException
			(ex.getMessage(), getFilename(), getLine(), getColumn()));
        }



update_or_substitution returns [Term result = null]
{
} :

        ( LBRACE IDENT COLON ) => result = substitutionterm |
        result = updateterm
    ; 


substitutionterm returns [Term result = null]
{ Sort s;
    LogicVariable v = null;
    Term a1, a2;
    boolean waryAll = false;
    boolean waryEx = false;
} : 

        LBRACE v1:IDENT COLON s=sortId a1=logicTermReEntry
        ( "waryAll" { waryAll = true; } |
            "waryEx"  { waryEx  = true; } )?
        RBRACE 
        {v = bindVar(v1, s);} 
        a2 = term60 { 
            if ( waryAll )
                result = tf.createSubstitutionTerm ( Op.WARY_SUBST_ALL,
                                                     v, a1, a2 );
            else if ( waryEx )
                result = tf.createSubstitutionTerm ( Op.WARY_SUBST_EX,
                                                     v, a1, a2 );
            else
                result = tf.createSubstitutionTerm ( Op.SUBST,
                                                     v, a1, a2 );
            unbindVars();
        }
 ; exception
        catch [TermCreationException ex] {
              keh.reportException
		(new KeYSemanticException
			(ex.getMessage(), getFilename(), getLine(), getColumn()));
        }




updateterm returns [Term result = null] 
{ 
    SingleUpdateData sud = null;
    List locations = new LinkedList();  
    List values    = new LinkedList();  
    Term a2 = null;
} :
        LBRACE ((ARITH_OP ASSIGN)=> 
          (starLoc:ARITH_OP ASSIGN starValue:ARITH_OP 
            {"*".equals(starLoc.getText()) && 
             "*".equals(starValue.getText())}?
             id:IDENT RBRACE a2 = term60 {
                return tf.createAnonymousUpdateTerm
                   (new Name("*:=*"+id.getText()), a2);
             }                  
          )
          | 
          (sud = singleupdate {
            locations.add(sud.a0); 
            values.add(sud.a1); 
          } ( COMMA sud = singleupdate {
                locations.add(sud.a0); 
                values.add(sud.a1); 
          })*) RBRACE a2 = term60
        {   
            result = tf.createUpdateTerm
		((Term[])locations.toArray(new Term[locations.size()]),
		 (Term[])values.toArray(new Term[values.size()]),
		  a2);
        })
   ; exception
        catch [TermCreationException ex] {
              keh.reportException
		(new KeYSemanticException
			(ex.getMessage(), getFilename(), getLine(), getColumn()));
        }



singleupdate returns[SingleUpdateData sud=null]
{
    Term a0 = null;
    Term a1 = null;
    ArraySort as;
    TermSymbol pv;
    LhsUpdateData lud = null;
    boolean shadow_l = false;
    boolean shadow_r = false;
}  :
        (a0 = lhsSingle ) ASSIGN (a1 = rhsSingle) {
            sud = new SingleUpdateData();
            sud.a0 = a0;
            sud.a1 = a1;
        }
    ;

rhsSingle returns[Term result = null]
{} :
        result = logicTermReEntry
    ;

lhsSingle returns[Term result = null]
{
    Term a1 = null;
} :
        result = logicTermReEntry
        { 
            if (!(result.sort() instanceof ProgramSVSort) &&
                result.op() != Op.META_SHADOW &&
                !(result.op() instanceof ProgramVariable || 
                  result.op() instanceof AccessOp)) {
                throw new KeYSemanticException
                ("Term of sort " + result.sort() + 
                 " cannot stand for a location.",
                 getLine(), getColumn(), getFilename());
            }
        }
    ;



/** here is the standard behaviour, only accepts a program. */

normal_dl_term returns [Term a = null]
{   JavaBlock jb=null;
    Term a1;
}
    :
        d1:DIAMOND 
        {
            jb=getJavaBlock(d1);
            HashSet progVars=progVars(jb);
            bindProgVars(progVars);
        }
        a1=term60
        {
            a = tf.createDiamondTerm(jb, a1);
            unbindProgVars();
        }
    |
        d2:BOX
        {
            jb=getJavaBlock(d2);
            HashSet progVars=progVars(jb);
            bindProgVars(progVars);
        }
        a1=term60   
        {
            a = tf.createBoxTerm(jb, a1);
            unbindProgVars();
        }
    |
        d3:THROUGHOUT
        {
            jb=getJavaBlock(d3);
            HashSet progVars=progVars(jb);
            bindProgVars(progVars);
        }
        a1=term60
        {
            a = tf.createProgramTerm(Op.TOUT,jb, a1);
            unbindProgVars();
        }
     ; exception
        catch [TermCreationException ex] {
              keh.reportException
		(new KeYSemanticException
			(ex.getMessage(), getFilename(), getLine(), getColumn()));
        }


named_dl_term returns [Term a = null]
{
    PairOfTermArrayAndBoundVarsArray argsWithBoundVars = null;
    Term[] terms = null;
    Term term = null;
    JavaBlock jb = null;
} :
        id:IDENT
        bl:MODALITYARG ( 
            {
                jb = getJavaBlock(bl);               
                bindProgVars(progVars(jb));
            }
            argsWithBoundVars = argument_list
            {   
                unbindProgVars();
                if (argsWithBoundVars != null) {
                    terms = argsWithBoundVars.getTerms();
                }
                Operator op = null;
                                if (id.getText().charAt(0)=='#') {
                    if (!inSchemaMode()) {
                        throw new KeYSemanticException
                            ("No schema elements allowed outside taclet declarations.",
                             id, getFilename());
                    }
                    op = (SchemaVariable)variables().lookup(new Name(id.getText()));
                } else {
                    op = Op.getModality(id.getText());
                }
                if(op == null) {
                    throw new KeYSemanticException("Unknown modal operator.", id, getFilename());
                }
                if(op.arity() != terms.length) {
                    throw new KeYSemanticException
                        ("The arity of " + op.name() +
                         " is "+op.arity() + ", but you provided " + terms.length + " arguments",
                         getLine(), getColumn(), getFilename());
                }
                a = tf.createProgramTerm(op, jb, terms);
            }
        )
    ; exception
        catch [TermCreationException ex] {
              keh.reportException
		(new KeYSemanticException
			(ex.getMessage(), getFilename(), getLine(), getColumn()));
        }

//"PairOfTermArrayAndBoundVarsArray" is used here instead of "Term" to allow for
//syntax needed for OCL simplification.
argument_list returns [PairOfTermArrayAndBoundVarsArray ts = null]
{
    List args = new LinkedList();
    TermWithBoundVars p1, p2;
}
    :
        LPAREN 
        (p1 = argument { args.add(p1);  }

            (COMMA p2 = argument { args.add(p2); } )* )?

        RPAREN
        {
            ts = new PairOfTermArrayAndBoundVarsArray(args);
        }

    ;

funcpredvarterm returns [Term a = null]
{
    Term[] arguments = null;
    ArrayList attributeList = null;
    PairOfTermArrayAndBoundVarsArray argsWithBoundVars = null;
    String varfuncid;
    String attribute; 
    String neg = "";
}
    :
        ch:CHAR_LITERAL {
            String s = ch.getText();
            int intVal = 0;
            if (s.length()==3) {
                intVal = (int)s.charAt(1);
            } else {
                try {
                    intVal = Integer.parseInt(s.substring(3,s.length()-1),16);
                } catch (NumberFormatException ex) {
                    throw new KeYSemanticException
                        ("Expected '"+s+"' to be a character", ch, getFilename());
                }       
            }
            a = tf.createFunctionTerm((Function) functions().lookup(new Name("C")), 
                                      TermParser.toZNotation(""+intVal, functions(), tf).sub(0));
        }
    | { ("-".equals(LT(1).getText()) && TermParser.isNumber(LT(2).getText())) 
	|| 
        (TermParser.isNumber(LT(1).getText()) && !"(".equals(LT(2).getText()))}? 
	({"-".equals(LT(1).getText())}? ARITH_OP {neg = "-";})?
         number:IDENT     
        { a = TermParser.toZNotation(neg+number.getText(), functions(), tf);}    
    | AT a = abbreviation
    | varfuncid = id
        ((LBRACKET ( (attrid SEMI )+ | SEMI ) RBRACKET)=> (LBRACKET
            {attributeList = new ArrayList();} (
                (attribute = attrid SEMI { attributeList.add(attribute); } )+
                |
                 SEMI) {
		  varfuncid = DeclParser.createDependencyName(varfuncid, attributeList);
		  
                }
                RBRACKET))?
	((LPAREN)=>argsWithBoundVars = argument_list)? 
        //argsWithBoundVars==null indicates no argument list
        //argsWithBoundVars.size()==0 indicates open-close-parens ()
        {   
            Operator op = lookupVarfuncId(varfuncid, argsWithBoundVars);
            if (op instanceof ParsableVariable) {
                a = termForParsedVariable((ParsableVariable)op);
            } else  if (op instanceof TermSymbol) {

                if (argsWithBoundVars==null) {
                    argsWithBoundVars = new PairOfTermArrayAndBoundVarsArray(new LinkedList());
                }

                a = tf.createFunctionWithBoundVarsTerm((TermSymbol)op, argsWithBoundVars);

            }
        }
; exception
        catch [TermCreationException ex] {
              keh.reportException
		(new KeYSemanticException
			(ex.getMessage(), getFilename(), getLine(), getColumn()));
        }



attrid returns [String attr = "";]
{} : 
        ({ isPackage(LT(1).getText()) || (LA(2) == COLON && LA(3) == COLON) }? 
            attr = classReference COLON COLON {attr += "::";} )?
        attrID:IDENT {attr += attrID.getText();}
    ;



// entry point overwritten in taclets.g
specialTerm returns [Term result = null] 
{
    Operator vf = null;
}:
        vf = expr_op 
        {   
            result = tf.createFunctionTerm((Function)vf, AN_ARRAY_OF_TERMS);
        }                 
 ; exception
        catch [TermCreationException ex] {
              keh.reportException
		(new KeYSemanticException
			(ex.getMessage(), getFilename(), getLine(), getColumn()));
        }


expr_op returns [Operator v = null] 
{
    ParsableVariable left_var0 = null;
    ParsableVariable right_var0 = null;
    ParsableVariable unary_var0 = null;
}   
    :

        IN_TYPE LPAREN ((left_var0=varId ((id:ARITH_OP | id1:SLASH | id2:IDENT) right_var0 = varId)?) |
            ( "-" unary_var0 = varId) )  RPAREN
        {  
            if (id==null) id=id1;   
            if (unary_var0!=null) {
                // can only be unary minus
                SortedSchemaVariable unary_var = (SortedSchemaVariable) unary_var0;
                Sort type = de.uka.ilkd.key.logic.sort.Sort.FORMULA;
                return new InTypeOperator
                    (type, new de.uka.ilkd.key.java.expression.operator.Negative(unary_var));
            }
            else {
                SortedSchemaVariable left_var = (SortedSchemaVariable) left_var0;
                Sort type = de.uka.ilkd.key.logic.sort.Sort.FORMULA;
                if (right_var0 != null) {
                    SortedSchemaVariable right_var = (SortedSchemaVariable) right_var0;
                    String op_str = (id == null ? id2.getText() : id.getText());
                    if ("+".equals(op_str)) {
                        return new InTypeOperator
                            (type, new de.uka.ilkd.key.java.expression.operator.Plus(left_var, right_var));
                    } else if ("-".equals(op_str)) {
                        return new InTypeOperator
                            (type, new de.uka.ilkd.key.java.expression.operator.Minus(left_var, right_var));
                    } else if ("*".equals(op_str)) {
                        return new InTypeOperator
                            (type, new de.uka.ilkd.key.java.expression.operator.Times(left_var, right_var));
                    } else if ("/".equals(op_str)) {
                        return new InTypeOperator
                            (type, new de.uka.ilkd.key.java.expression.operator.Divide(left_var, right_var));
                    } else if ("%".equals(op_str)) {
                        return new InTypeOperator
                            (type, new de.uka.ilkd.key.java.expression.operator.Modulo(left_var, right_var));
                    } 
                }
                else {
                    return new InTypeOperator(type, left_var);
                }
            }
        }
    | IS_OBJECT  LPAREN (left_var0=varId )  RPAREN
        {
            SortedSchemaVariable var = (SortedSchemaVariable) left_var0;
            return new IsObject(de.uka.ilkd.key.logic.sort.Sort.FORMULA, var);
        }
    ;

varId returns [ParsableVariable v = null]
    :
        id:IDENT 
        {   
            v = (ParsableVariable) variables().lookup(new Name(id.getText()));
            if (v == null) {
                throw new NotDeclException("variable", id, 
                                           getFilename());
            }
        } 
  ;
