// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.proof;

import java.util.ArrayList;
import java.util.List;

import de.uka.ilkd.key.gui.GUIEvent;
import de.uka.ilkd.key.gui.ProofSettings;
import de.uka.ilkd.key.gui.SettingsListener;
import de.uka.ilkd.key.gui.StrategySettings;
import de.uka.ilkd.key.logic.Name;

/**
 * This class stores the configuration for automatic mode.
 */
public class AutoModeConfiguration {
    /**
     * name of a selectable strategy
     * @see StrategyFactory.getStrategyNamespace
     */
    private Name strategyName;

    /** 
     * maximum number of rule applications allowed in automatic mode 
     */
    private int maxAutomaticSteps;
    
    /**
     * List with AutoModeConfigurationListener objects
     */
    private List listenerList = new ArrayList(10);
    
    /**
     * Create a new AutoModeConfiguration.
     */
    public AutoModeConfiguration () {
        updateSettings();
    }

    private void updateSettings()
    {
        final StrategySettings strategySettings =
            ProofSettings.THE_SETTINGS.getStrategySettings();
        
        maxAutomaticSteps = strategySettings.getMaxSteps();
        strategyName = strategySettings.getStrategy();
    }
    
    public Name getStrategy () {
        return strategyName;
    }
    
    public void setStrategy ( Name p_strategyName ) {
        if ( p_strategyName != strategyName ) {
            strategyName = p_strategyName;
            fireStrategyChanged ();
        }
    }
    
    public int getMaxAutomaticSteps () {
        return maxAutomaticSteps;
    }
    
    public void setMaxAutomaticSteps ( int steps ) {
        if ( steps != maxAutomaticSteps ) {
            maxAutomaticSteps = steps;
            fireMaxAutomaticStepsChanged ();
        }
    }

    /** 
     * Fires the event that the strategy has changed. 
     */
    protected void fireStrategyChanged () {
        for (int i = 0; i < listenerList.size(); i++) {
            ((AutoModeConfigurationListener)listenerList.get(i))
                .strategyChanged(this);
        }
    }

    /** 
     * Fires the event that the maximum number of rule 
     * applications allowed in automatic mode has changed. 
     */
    protected void fireMaxAutomaticStepsChanged () {
        for (int i = 0; i < listenerList.size(); i++) {
            ((AutoModeConfigurationListener)listenerList.get(i))
                .strategyChanged(this);
        }
    }

    /**
     * adds a listener 
     * @param listener the AutoModeConfigurationListener to be added
     */
    public synchronized void addListener 
        (AutoModeConfigurationListener listener) {
        synchronized(listenerList) {
            listenerList.add(listener);
        }
    }

    /**
     * removes a listener 
     * @param listener the AutoModeConfigurationListener to be removed
     */
    public synchronized void removeListener
        (AutoModeConfigurationListener listener) {
        synchronized(listenerList) {
            listenerList.remove(listener);
        }
    }
}
