// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//This file is part of KeY - Integrated Deductive Software Design
//Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                    Universitaet Koblenz-Landau, Germany
//                    Chalmers University of Technology, Sweden
//
//The KeY system is protected by the GNU General Public License. 
//See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.proof;

import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.java.statement.*;
import de.uka.ilkd.key.java.annotation.*;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.logic.sort.*;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.pp.*;
import de.uka.ilkd.key.rule.*;


public class LoopInvariantProposal extends InstantiationProposal {

    /**
     * Returns a proposal for the instantiation of <code>var</code> 
     * iff <code>app</code> is a TacletApp for a loop invariant taclet
     * and <code>var</code> is the SchemaVariable representing the invariant 
     * and the loop on which the taclet matches contains a loop invariant
     * annotation. Otherwise null is returned.
     */
    public static String getProposal(TacletApp app, SchemaVariable var){
	Term t = tryToInstantiate(app, var);
	if(t != null){
	    LogicPrinter lp = 
		new LogicPrinter(new ProgramPrinter(null), 
				 NotationInfo.createInstance());
	    try{
		lp.printTerm(t);
		return lp.toString();
	    }catch(java.io.IOException e){
		return null;
	    }
	}
	return null;
    }

    /**
     * Returns an instantiation of <code>var</code> 
     * iff <code>app</code> is a TacletApp for a loop invariant taclet
     * and <code>var</code> is the SchemaVariable representing the invariant 
     * and the loop on which the taclet matches contains a loop invariant
     * annotation. Otherwise null is returned.
     */
    public static Term tryToInstantiate(TacletApp app, SchemaVariable var){
	if((app instanceof PosTacletApp) && 
	   app.taclet().name().toString().indexOf("_inv") != -1){
	    PosInOccurrence pos = ((PosTacletApp) app).posInOccurrence();
	    if(var.name().toString().equals("inv")){
		Term t = getLoopInvariant(pos.subTerm());
		return t;
	    }else if(var.name().toString().equals("#modifies")){
		Term t = getLoopAssignables(pos.subTerm());
		return t;
	    }
	}
	return null;
    }

    private static Term getLoopInvariant(Term t){
	LoopStatement l = getFirstLoopStatement(t);
	Annotation[] a = l.getAnnotations();
	for(int i = 0; i<a.length; i++){
	    if(a[i] instanceof LoopInvariantAnnotation){
		return ((LoopInvariantAnnotation) a[i]).invariant();
	    }
	}
	return null;
    }

    private static Term getLoopAssignables(Term t){
	LoopStatement l = getFirstLoopStatement(t);
	Annotation[] a = l.getAnnotations();
	Term result = TermFactory.DEFAULT.createFunctionTerm(Op.EVERYTHING);
	for(int i = 0; i<a.length; i++){
	    if(a[i] instanceof LoopInvariantAnnotation){
		ArrayOfTerm assignables = 
		    ((LoopInvariantAnnotation) a[i]).assignable();
		if(assignables.size()>0){
		    result = assignables.getTerm(0);
		}
		for(int j=1; j<assignables.size(); j++){
		    result = TermFactory.DEFAULT.createFunctionTerm(
			Op.LIST, assignables.getTerm(j), result);
		}
		if(assignables.size() == 1 && 
		   assignables.getTerm(0).sort() != Sort.TERMLIST){
		    result = TermFactory.DEFAULT.createFunctionTerm(
			      Op.LIST, result, TermFactory.
			      DEFAULT.createFunctionTerm(Op.NOTHING));
		}
		return result;
	    }
	}
	return null;
    }

    private static LoopStatement getFirstLoopStatement(Term t){
	while(t.op() instanceof UpdateOperator){
	    t = t.sub(t.arity()-1);
	}
	JavaBlock jb = t.javaBlock();
	ProgramElement p = jb.program();
	return getLoopHelp(p);
    }

    private static LoopStatement getLoopHelp(ProgramElement pe){
	if(pe instanceof LoopStatement){
	    return (LoopStatement) pe;
	}
	if(pe instanceof StatementContainer){
	    return getLoopHelp(((StatementContainer) pe).getStatementAt(0));
	}
	//shouldn't happen.
	return null;
    }
}
