// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.proof;

import de.uka.ilkd.key.java.abstraction.*;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.op.ProgramVariable;
import de.uka.ilkd.key.proof.init.*;
import de.uka.ilkd.key.parser.*;
import de.uka.ilkd.key.java.JavaInfo;
import de.uka.ilkd.key.java.TypeConverter;
import de.uka.ilkd.key.casetool.*;
import java.io.*;
import java.util.*;
import javax.swing.JOptionPane;
import de.uka.ilkd.key.util.KeYExceptionHandler;

/** parses the given modifies clause
 */
public class ModifiesParserHelper {
    private ReprModelMethod methRepr;
    private InitConfig initConfig;
    private Set modifiesElements;
    
    private JavaInfo javaInfo;
    private String methodName;

    private ProgramVariable self;
    private ProgramVariable pv;
    private Vector pvVector = new Vector();

    private Vector parameterNames = new Vector();
    private Vector parameterTypes = new Vector();
    
    private Namespace ns = new Namespace();

    public ModifiesParserHelper(ReprModelMethod methRepr, 
				InitConfig initConfig){
	this.methRepr = methRepr;
	this.initConfig = initConfig;
    }

    public void parseModifiesClauseToHashSet(){
	javaInfo = initConfig.getServices().getJavaInfo();
		
	// methodName, parameterNames, parameterTypes are filled in here
	parseSignatureString();
	
	// also creates pvVector, a vector of all the parameters as
	// program variables
	fillNamespace();
	
	// puts the parsed modifies clause into the set modifiesElements
	parseModifiesString();
    }

    public void parseModifiesClauseToHashSet(Namespace ns){
	this.ns = ns;
	javaInfo = initConfig.getServices().getJavaInfo();
		
	// methodName, parameterNames, parameterTypes are filled in here
	parseSignatureString();
	
	// also creates pvVector, a vector of all the parameters as
	// program variables
	//fillNamespace();
	
	// puts the parsed modifies clause into the set modifiesElements
	parseModifiesString();
    }

    private void parseSignatureString(){
	String signatureString = methRepr.getCallSignature(false);
	//System.out.println(signatureString);
	
	// first extract the name of the method
	int index = signatureString.indexOf("(");
        methodName = signatureString.substring(0, index).trim();
	signatureString = signatureString.substring(index+1).trim();
	
	// then extract the names and types of parameters
	index = signatureString.indexOf(")");
	String type = null;
	while (index > 0){
	    index = signatureString.indexOf(":");
	    parameterNames.add(signatureString.substring(0, index).trim());
	    signatureString = signatureString.substring(index+1).trim();
	    
	    index = signatureString.indexOf(";");
	    // index = -1 if there is no ";" in the string because we are
	    // parsing the last element
	    if (index < 0) {
		index = signatureString.indexOf(")");
		type = signatureString.substring(0, index).trim();
		// convert to KeYJavaType and add to the vector
		parameterTypes.add(javaInfo.getKeYJavaType(type));
		
		// the ")" is at the very end of the string so don't add
		// 1 to the index in this case otherwise there is a string
		// out of bounds error
		// by this the index calculated below will be 0
		signatureString = signatureString.substring(index+0).trim();
	    }
	    else {
		type = signatureString.substring(0, index).trim();
		// convert to KeYJavaType and add to the vector
		parameterTypes.add(javaInfo.getKeYJavaType(type));
		signatureString = signatureString.substring(index+1).trim();
	    }
	    index = signatureString.indexOf(")");
	    
	}
    }
    
    private void fillNamespace(){
	// Add "self" to the Namespace. 
	self = new ProgramVariable(new ProgramElementName("self"),
				   javaInfo.getKeYJavaType
				   (methRepr.getContainingClass().getFullClassName()));
	ns.add(self);


	// add the parameter variables to the namespace und keep them in a
	// vector for later reuse in a methodreference
	for (int i = 0; i < parameterTypes.size(); i++) {
	    pv = new ProgramVariable
		(new ProgramElementName
		 ((String) parameterNames.elementAt(i)),
		 (KeYJavaType) parameterTypes.elementAt(i));
	    pvVector.add(pv);
	    ns.add(pv);
	}
	
    }

    private void parseModifiesString() {
	String modifiesString = methRepr.getMyModifClause();
	if (methRepr.isQuery()) {
	    // query and empty or non-existing modifies string;
	    // make an empty modifies string
	    if (modifiesString == null || "".equals(modifiesString)){
		modifiesString = "";
	    }
	    // query but non-empty modifies string; not allowed to happen!
	    else {
		JOptionPane.showMessageDialog
		    (null, "Inconsistent Modifies Set",
		     "Non-empty modifies set but the method is a query!", 
		     JOptionPane.ERROR_MESSAGE);
		throw new IllegalStateException
		    ("Non-empty modifies set but the method is a query!");
	    }
	}
	// otherwise no changes need to be done
	
	if (modifiesString == null) {
	    modifiesElements = null;
	}
	else if ("".equals(modifiesString)) {
	    modifiesElements = new HashSet();
	}
	else {
	    //useTermParser(modifiesString); <-- use for general expressions in
	    // the modifies clause - needs some more work
	    useModifiesParser(modifiesString); // <-- alternative method in case
	    //special constructs need to be parsed 
	}
    }

    private void useTermParser(String modifiesString) {
	NamespaceSet nss = initConfig.namespaces();
	nss.setProgramVariables(ns);	    
	TermParser parser = new TermParser(new KeYLexer(new StringReader(modifiesString)),"", 
					   TermFactory.DEFAULT, null, initConfig.getServices(), nss, null);
	Set accessTerms = null;
	try {
	    accessTerms = parser.accesstermlist();
	} catch( antlr.RecognitionException e0) {
	    System.out.println("RecognitionException occurred");
	    e0.printStackTrace();
	} catch(antlr.TokenStreamException e1) {
	    System.out.println("TokenStreamException occurred");
	    e1.printStackTrace();
	}
	
	Iterator it = accessTerms.iterator();
	TypeConverter tc = initConfig.getServices().getTypeConverter();
	modifiesElements = new HashSet();
	while (it.hasNext()) {
	    modifiesElements.add(tc.convertToProgramElement((Term)it.next()));
	}
	System.out.println(modifiesElements);
    }

    private void useModifiesParser(String modifiesString) {
	ModifiesParser parser = new ModifiesParser(new KeYLexer
						   (new StringReader
						    (modifiesString)),
 						       ns, javaInfo);
	try {
	    parser.top();
	} catch( antlr.RecognitionException e0) {
	    System.out.println("RecognitionException occurred");
	    e0.printStackTrace();
	} catch(antlr.TokenStreamException e1) {
	    System.out.println("TokenStreamException occurred");
	    e1.printStackTrace();
	}
	
	modifiesElements = parser.modifiesElements();
    }

    public Set getModifiesElementsHashSet(){
	return modifiesElements;
    }

    public String getMethodName() {
	return methodName;
    }
    
    public ProgramVariable getPVSelf(){
	return self;
    }
    
    public Vector getPVVector(){
	return pvVector;
    }
    
    public Vector getParameterTypesVector(){
	return parameterTypes;
    }

}
