// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.proof;

import java.io.*;
import java.net.URL;
import de.uka.ilkd.key.util.KeYResourceManager;

public class RuleSource {

    private URL url=null;

    private File f=null;

    private long numberOfChars;

    private RuleSource(File f) {
	this.f=f;
    }

    public static RuleSource initRuleFile(String filename) {
        URL u = KeYResourceManager.getManager().
            getResourceFile(RuleSource.class, "rules/"+filename);
        if (u == null) {
            // a more specific exception type woul probably be better
            throw new RuntimeException("Could not find rule file "+filename);
        }
	return new RuleSource(u);
    }

    public static RuleSource initRuleFile(URL url) {
	return new RuleSource(url);
    }
    
    private RuleSource(URL url) {
	this.url = url;
	if (f!=null) {
	    numberOfChars=f.length();
	}
	if (url.getProtocol().equals("file"))
	    numberOfChars = (new File(url.getFile())).length();
	else {
	    try {
		InputStream input = url.openStream();
		numberOfChars=0;
		int i=input.read();
		while (i!=-1){
		    numberOfChars++;
		    i=input.read();
		}
	    } catch (IOException ioex){
		System.err.println("IOException in class RuleSource");
	    }
	}
    }
   
    
    public int getNumberOfChars(){
	return (int)numberOfChars;
    }

    public static RuleSource initRuleFile(File file) {
	return new RuleSource(file);
    }
   
    public File file() {
	if (f==null) {
	    return new File(url.getFile());
	} else {
	    return f;
	}
    }

   public String toString() {
       if (url!=null) {
	   return url.toString();
       } else {
	   return f.toString();
       }
   }

    public InputStream getNewStream() {
	try {
	    if (f==null) {
		return url.openStream();
	    } else {
		return new FileInputStream(f);
	    }
	}
	catch (IOException ioe) {
	    System.err.println("*******************************************");
	    System.err.println("IO-Error occured while opening rule stream.");
	    System.err.println("URL: "+url);
	    System.err.println(ioe);
	    System.err.println("*******************************************");
	    ioe.printStackTrace();
	    throw new RuntimeException("Error while parsing LDTs");
	}
    }
	

    public boolean isDirectory() {
	return file().isDirectory();
    }

}
