// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//This file is part of KeY - Integrated Deductive Software Design
//Copyright (C) 2001-2004 Universitaet Karlsruhe, Germany
//                    Universitaet Koblenz-Landau, Germany
//                    Chalmers University of Technology, Sweden
//
//The KeY system is protected by the GNU General Public License. 
//See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.proof;

import de.uka.ilkd.key.logic.op.SchemaVariable;
import de.uka.ilkd.key.logic.op.SortedSchemaVariable;
import de.uka.ilkd.key.logic.sort.ProgramSVSort;
import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.rule.TacletApp;


public class VariableNameProposal extends InstantiationProposal {
	
	/* 
	 * @see de.uka.ilkd.key.proof.InstantiationProposal#getProposal(de.uka.ilkd.key.rule.TacletApp, de.uka.ilkd.key.logic.op.SchemaVariable)
	 */
	public static String getProposal(TacletApp app, SchemaVariable var) {
		if ( var.isDependingSV() )
		    return VariableNameProposal.getNameProposalForDependingVariable(app, var);
		else
		    return VariableNameProposal.getNameProposalForSchemaVariable(var);
	}

	/**
	 * Generates a proposal for the instantiation of the given
	 * schema variable.
	 * Currently this proposal only serves for the case of names (for labels or 
	 * variables) and generates a unique new name.
	 * @param var the schema var for which a unique name should be proposed as 
	 * its instantiation.
	 * @return a new and unquie name that can be used as an instantiation for
	 *         the given schema variable.
	 */
	public static String getNameProposalForSchemaVariable(SchemaVariable var){
	String result = null;
	
	Sort svSort = ((SortedSchemaVariable)var).sort();
	if (svSort == ProgramSVSort.LABEL) {
	    result = TacletInstantiationsTableModel.LABEL_NAME_PREFIX + 
		TacletInstantiationsTableModel.globalLabelNameCnt++;
	} else if (svSort == ProgramSVSort.VARIABLE) {
	    result = TacletInstantiationsTableModel.VARIABLE_NAME_PREFIX + 
		TacletInstantiationsTableModel.globalVariableNameCnt++;
	} else if (var.isVariableSV()) {
	    // HACK
	    result = TacletInstantiationsTableModel.VARIABLE_NAME_PREFIX + 
		TacletInstantiationsTableModel.globalVariableNameCnt++;
	}
	
	return result;
	}

	/**
	 * Generates a proposal for the instantiation of the given term
	 * schema variable, which is declared as depending (i.e. skolem
	 * function).
	 * @param var the schema var for which a unique name should be proposed as 
	 * its instantiation.
	 * @return a new and unquie name that can be used as an instantiation 
	 * for the given schema variable.
	 */
    public static String getNameProposalForDependingVariable(
	TacletApp p_app, SchemaVariable p_var){
	String res;
	SchemaVariable v = p_app.taclet ().getNameCorrespondent ( p_var );
	if (v != null && p_app.instantiations ().isInstantiated ( v ) ){
	    res = "" + p_app.instantiations ().getInstantiation ( v );
	}else{
	    // ... otherwise use the name of the dependingSV
	    res = "" + p_var.name ();	
	    res = getNameProposalForDependingVariable(res);
	}
	return res;
    }

    public static String getNameProposalForDependingVariable(String name){
	name += TacletInstantiationsTableModel.
	    DEPENDING_VARIABLE_NAME_POSTFIX;
	
	synchronized ( TacletInstantiationsTableModel.
		       globalDependingVariableNameCnt ) {
	    Integer cnt = (Integer)TacletInstantiationsTableModel.
		globalDependingVariableNameCnt.get (name);
	    
	    if ( cnt == null )
		cnt = new Integer ( 0 );
	    else
		cnt = new Integer ( cnt.intValue () + 1 );
	    
	    TacletInstantiationsTableModel.globalDependingVariableNameCnt.
		put(name, cnt);
	    
	    name += cnt;
	}
	return name;
    }
}
