// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.proof.init;

import de.uka.ilkd.key.jml.*;
import de.uka.ilkd.key.casetool.ReprModelMethod;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.proof.mgt.Contract;
import de.uka.ilkd.key.proof.*;
import de.uka.ilkd.key.java.declaration.*;

import java.util.*;

/** Represents a proof obligation for the assignable clause of a certain 
 * (jml) method specification case.
 */
public class AssignableCheckProofOblInput extends ModifiesCheckProofOblInput 
    implements JMLProofOblInput{

    private JMLMethodSpec spec;
    private String javaPath;

    private boolean allInv = false;

    public AssignableCheckProofOblInput(JMLMethodSpec spec, String javaPath){
	super("Assignable PO for: "+spec, null, null);
	this.javaPath = javaPath;
	this.spec = spec;
    }

    public void setAllInvariants(boolean allInv){
	this.allInv = allInv;
    }

    private void createModifiesElements(){
	IteratorOfTerm it = spec.replaceModelFieldsInAssignable().iterator();
	modifiesElements = new HashSet();
	while(it.hasNext()){
	    modifiesElements.add(initConfig.getServices().getTypeConverter().
				 convertToProgramElement((Term) it.next()));
	}
    }

    public void read(ModStrategy mod){
	createModifiesElements();
	MethodDeclaration md = spec.getMethodDeclaration();
	initConfig.progVarNS().add(initConfig.getServices().
				   getImplementation2SpecMap().getProgVarNS());
	initConfig.progVarNS().add(UsefulTools.buildParamNamespace(md));
	if(spec.getMethodDeclaration().isStatic()){
	    self = null;
	}else{
	    self = (ProgramVariable) spec.getPrefix();
	}
	javaInfo = initConfig.getServices().getJavaInfo();
	createModifiesElements();
	pvVector = new Vector(md.getParameterDeclarationCount());
	ArrayOfParameterDeclaration ap = md.getParameters();
	for(int i=0; i<ap.size(); i++){
	    pvVector.add(ap.getParameterDeclaration(i).
			 getVariableSpecification().getProgramVariable());
	}
	m = spec.getProgramMethod();

	// puts all occuring classes into the set occuringClasses
	collectOccuringClasses(initConfig.getServices().getJavaInfo().
			       getKeYJavaType(spec.getClassDeclaration()));
	
	this.proofObl = TermFactory.DEFAULT.createJunctorTermAndSimplify(
	    Op.IMP, spec.getCompletePre(true, allInv), createProofStart());
    }

    public boolean checkPurity(){
	return proofObl != null ? 
	    (UsefulTools.purityCheck(proofObl, initConfig.getServices().
				    getImplementation2SpecMap()) == null):
	    true;
    }

    /**
     * The Java Model is already initialized;
     */
    public String readJava(){
	return null;
    }

    /** returns the path to the Java model.
     */
    public String getJavaPath(){
	return javaPath;
    }

    public Object getObjectOfContract(){
	return spec.getMethodDeclaration();
    }

    public boolean fitsToContract(Contract ct) {
	return false; //%%
    }

    public void startProtocol() {
	// do nothing
    }

    public String createProgramVarsSection(){
	IteratorOfNamed it = initConfig.progVarNS().allElements().iterator();
	String result = "program variables {\n";
	while(it.hasNext()){
	    ProgramVariable pv = (ProgramVariable) it.next();
	    result += pv.sort()+" "+pv.name()+";\n";
	}
	result += "}\n";
	return result;
    }

    public String createJavaSection(){
	return "java {\""+getJavaPath()+"\"}\n";
    }

    public String createProblemHeader(){
	return createJavaSection()+"\n"+createProgramVarsSection()+"\n";
    }

    public ProofList getPO(){
	return new ProofList(
	    new Proof(name(),
		      (Term) proofObl,
		      createProblemHeader(),
		      initConfig.createTacletIndex(), 
		      initConfig.createBuiltInRuleIndex(),
		      initConfig.getActivatedChoices()), name());
    }

    public String toString(){
	return name();
    }

    public String name(){
	return "Assignable PO ("+(allInv ? "all invariants" : 
				  "only invariants from "+
				  ((JMLMethodSpec) spec).getClassDeclaration().
				  getName())+
	    ") for: "+spec;
    }

}
