// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.proof.init;

import de.uka.ilkd.key.parser.jml.*;
import de.uka.ilkd.key.java.declaration.MethodDeclaration;
import de.uka.ilkd.key.jml.*;
import de.uka.ilkd.key.proof.mgt.*;
import de.uka.ilkd.key.collection.*;
import de.uka.ilkd.key.proof.*;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.TermFactory;
import de.uka.ilkd.key.logic.op.Op;

/** the proofobligation for invariants and historyconstraints*/
public class JMLInvPO extends JMLProofOblInputImpl{

    private Term po = null;

    public JMLInvPO(JMLClassSpec spec, MethodDeclaration md,
		    String javaPath, boolean allInv){
	super(spec, javaPath, allInv);
	this.md = md;
    }

    public ProofList getPO(){
	return new ProofList(
	    new Proof(name(),
		      (Term) po,
		      createProblemHeader(),
		      initConfig.createTacletIndex(), 
		      initConfig.createBuiltInRuleIndex(),
		      initConfig.getActivatedChoices()), name());
    }

    public void readProblem(ModStrategy mod) throws ProofInputException{
	initConfig.progVarNS().add(initConfig.getServices().
				   getImplementation2SpecMap().getProgVarNS());
	initConfig.progVarNS().add(UsefulTools.buildParamNamespace(md));
	po = ((JMLClassSpec) spec).getPreservesInvariantBehavior(md, allInv);
    }

    public boolean fitsToContract(Contract ct){
	return false;
    }

    public String name(){
	return "Invariant "+(allInv ? "(including all invariants)" : 
			     "(only invariants for "+
			     ((JMLClassSpec) spec).
			     getClassDeclaration().getName()+")")+
	    " and History Constraint PO for "+
	    md.getName(); 
    }

    public boolean checkPurity(){
	return po != null ? 
	    (UsefulTools.purityCheck(po, initConfig.getServices().
				    getImplementation2SpecMap()) == null):
	    true;
    }

}
