// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.proof.init;

import de.uka.ilkd.key.java.Recoder2KeY;
import de.uka.ilkd.key.java.ProgramElement;
import de.uka.ilkd.key.java.Expression;
import de.uka.ilkd.key.java.ArrayOfExpression;
import de.uka.ilkd.key.java.TypeConverter;
import de.uka.ilkd.key.java.abstraction.*;
import de.uka.ilkd.key.java.reference.FieldReference;
import de.uka.ilkd.key.java.reference.TypeReference; 
import de.uka.ilkd.key.java.reference.TypeRef; 
import de.uka.ilkd.key.java.StatementBlock;
import de.uka.ilkd.key.java.expression.literal.BooleanLiteral;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.JavaBlock;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.logic.op.Function;
import de.uka.ilkd.key.logic.op.ProgramVariable;
import de.uka.ilkd.key.logic.sort.ObjectSort;
import de.uka.ilkd.key.proof.*;
import de.uka.ilkd.key.util.Debug;
import de.uka.ilkd.key.util.KeYExceptionHandler;
import de.uka.ilkd.key.java.JavaInfo;
import de.uka.ilkd.key.casetool.*;
import de.uka.ilkd.key.java.visitor.ProgramTypeCollector;
import de.uka.ilkd.key.java.declaration.ClassDeclaration;
import java.util.*;
import de.uka.ilkd.key.proof.mgt.Contract;
import de.uka.ilkd.key.logic.SetOfChoice;
import de.uka.ilkd.key.logic.SetAsListOfChoice;
import de.uka.ilkd.key.java.statement.MethodBodyStatement;
import de.uka.ilkd.key.java.reference.ArrayReference;
import de.uka.ilkd.key.java.declaration.ArrayDeclaration;
import de.uka.ilkd.key.java.expression.operator.LessOrEquals;
import de.uka.ilkd.key.java.expression.operator.LessThan;
import de.uka.ilkd.key.java.expression.operator.New;
import de.uka.ilkd.key.java.expression.literal.IntLiteral;


/** represents a proof obligation originating from a given Modifies Clause
 * as input for the prover.
 */
public class ModifiesCheckProofOblInput implements ProofOblInput {

    private ReprModelMethod methRepr;
    protected String name;
    private String xmifile;

    protected InitConfig initConfig;
    
    protected Term proofObl;

    protected Set modifiesElements;
    protected Set occuringClasses;
    protected Set currentClassAttributes;

    protected KeYExceptionHandler exceptionHandler;

    protected JavaInfo javaInfo;
    protected ProgramMethod m;
    protected String methodName;

    protected ProgramVariable self;
    //private ProgramVariable pv;
    protected Vector pvVector;

    //private Vector parameterNames = new Vector();
    protected Vector parameterTypes;
    
//    private Namespace ns = new Namespace();

    /** creates a new representation of a proof obligation from a Modifies 
     * Clause with the given name, originating from a representation of a model
     * method and having the given String as text of the Modifies Clause.
     * The given XMI file name can be used to read the underlying model.
     */
    public ModifiesCheckProofOblInput(String name, ReprModelMethod aMethRepr, 
				      String xmifile) {
	this.name=name+(aMethRepr != null ? " "+aMethRepr.getName() : "");
	this.methRepr=aMethRepr;
	this.xmifile=xmifile;
    }

    /** reads the underlying Java model and modifies the initial configuration
     * of this object. If the latter is not yet defined an exception is
     * thrown.
     */
    public String readJava(){
	if (initConfig==null) {
	    throw new IllegalStateException("InitConfig not set.");
	}
	Recoder2KeY r2k = new Recoder2KeY(initConfig.getServices(), 
					  initConfig.namespaces());
	String[] cus = methRepr.getContainingClass().getClassesInPackage();
	r2k.readCompilationUnitsAsFiles(cus);
	Debug.out("Initialize JavaInfo");
	return methRepr.getContainingClass().getRootDirectory();
    }

    public String getJavaPath() {
        return methRepr.getContainingClass().getRootDirectory();
    }


    /** returns false, that is the input never asks the user which
     * environment he prefers
     */
    public boolean askUserForEnvironment() {
	return false;
    }
    
    public void setExceptionHandler(KeYExceptionHandler keh){
	exceptionHandler = keh;
    }

    /** generates the proof obligation
     * 
     */
    public void read(ModStrategy mod) {
	if (initConfig==null) {
	    throw new IllegalStateException("InitConfig not set.");
	}
        
	javaInfo = initConfig.getServices().getJavaInfo();
	
	getAllFromModParsHelper();
	
	// extracts the signature as a list
	ListOfType signatureList = createSignatureList();

	final KeYJavaType enclosingClassType = 
	    javaInfo.getKeYJavaType(methRepr.getContainingClass().getFullClassName());
	// finds the current program method
	m = javaInfo.getProgramMethod
	    (enclosingClassType, methodName, signatureList, enclosingClassType);
	//last argument added. hope it is right. /AR
	
	// puts all occuring classes into the set occuringClasses
	collectOccuringClasses(enclosingClassType);
	
	this.proofObl = createProof();
    }

    public void readProblem(ModStrategy mod) {
	read(mod);
    }

    private void getAllFromModParsHelper(){
	ModifiesParserHelper modParserH = 
	    new ModifiesParserHelper(methRepr, initConfig);
	
	// starts the internal computation of the helper class
	modParserH.parseModifiesClauseToHashSet();
	
	modifiesElements = modParserH.getModifiesElementsHashSet();
	methodName = modParserH.getMethodName();
	self = modParserH.getPVSelf();
	pvVector = modParserH.getPVVector();
	parameterTypes = modParserH.getParameterTypesVector();
    }

    private ListOfType createSignatureList(){
	ListOfType signatureList = SLListOfType.EMPTY_LIST;
	int index = parameterTypes.size()-1;
	while (index >= 0){
	    signatureList = signatureList.prepend((Type) parameterTypes.
	    				  elementAt(index));
	    index = index - 1;
	}
	return signatureList;
    }

    protected void collectOccuringClasses(KeYJavaType type) {
	ProgramTypeCollector mCollector = 
	    new ProgramTypeCollector(m, self, type, initConfig.getServices(),
				     new HashSet());
	mCollector.start();
	occuringClasses = mCollector.getResult();
	
	if (type!= null) {
	    occuringClasses.add(type);
	}

	// get the transitive closure of the occuring classes in the
	// sense that for each class the type of all its' attributes
	// is in the resulting Set

	Set occuringClassesToAdd = (HashSet) ((HashSet) occuringClasses).clone();
	KeYJavaType current = null;
	while (!(occuringClassesToAdd.isEmpty())){
	    current = (KeYJavaType) occuringClassesToAdd.iterator().next();
	    
	    Type currClassType = current.getJavaType();
	    KeYJavaType currentBase = current;
	    while (currClassType instanceof ArrayDeclaration) {
		currentBase = ((ArrayDeclaration) currClassType).
		    getBaseType().getKeYJavaType();
		currClassType = currentBase.getJavaType();
	    }
	    if (!(currClassType instanceof PrimitiveType || 
		  currClassType instanceof NullType)){
	    	// get all attributes of the given class
		ListOfField allAttributes = 
		    javaInfo.getKeYProgModelInfo().getAllFields(currentBase);

		IteratorOfField iterateAttributes = allAttributes.iterator();
		KeYJavaType addToOccCla = null;
		
		while (iterateAttributes.hasNext()){
		    ProgramVariable att = (ProgramVariable) iterateAttributes.next()
			.getProgramVariable();
		    if (!att.isImplicit()){
			addToOccCla = (KeYJavaType) att.getKeYJavaType();
			if (!(occuringClasses.contains(addToOccCla))){
			    occuringClasses.add(addToOccCla);
			    occuringClassesToAdd.add(addToOccCla);
			}
		    }
		}
	    }
	    occuringClassesToAdd.remove(current);
	}
    }
    
    
    protected Term createProof() {
	Term formula;
	if(self != null){
	    formula = TermFactory.DEFAULT.createJunctorTermAndSimplify
		(Op.IMP, 
		 TermFactory.DEFAULT.createJunctorTerm
		 (Op.NOT,
		  TermFactory.DEFAULT.createJunctorTerm
		  (Op.EQUALS,
		   TermFactory.DEFAULT.createVariableTerm(self),
		   TermFactory.DEFAULT.createFunctionTerm(Op.NULL))), 
		 createProofStart());
	}else{
	    formula = createProofStart();
	}
	return formula;
    }
    
    protected Term createProofStart() {
	Iterator iterateClasses = occuringClasses.iterator();
	Term accumulateClassFormulas = 
	    TermFactory.DEFAULT.createJunctorTerm(Op.TRUE);
	KeYJavaType currClass;
	while (iterateClasses.hasNext()){
	    currClass = (KeYJavaType) iterateClasses.next();
	    
	    accumulateClassFormulas = 
		TermFactory.DEFAULT.createJunctorTermAndSimplify
		(Op.AND,
		 createClassFormula(currClass),
		 accumulateClassFormulas);
	}
	return accumulateClassFormulas;
    }
    
    private Term createClassFormula(KeYJavaType currClass){
	LogicVariable oprime  = new LogicVariable
	    (new Name("o"), 
	     currClass.getSort());
	
	// generate formula starting with \forall o': currClass
	Term classFormula = TermFactory.DEFAULT.createQuantifierTerm
	    (Op.ALL,
	     oprime ,
	     createClassSubFormula(currClass, oprime));


	return classFormula;
    }
    
    private Term createClassSubFormula(KeYJavaType currClass, 
				       LogicVariable oprime){
	Type currClassType = currClass.getJavaType();
	// stop primitive types from getting checked for attributes, there are
	// none anyway
	if (currClassType instanceof PrimitiveType)
	    {return TermFactory.DEFAULT.createJunctorTerm(Op.TRUE);}

	Term classSubFormula;
	
	ListOfField allAttributes;
	// get all attributes of the given class
	if (currClassType instanceof ClassDeclaration) {
	    allAttributes = javaInfo.getKeYProgModelInfo().getAllFields
		(javaInfo.getKeYJavaType(currClassType));
	    IteratorOfField iterateAttributes = allAttributes.iterator();
	    
	    // initial value is TRUE which is the value of an empty conjunction
	    // as this formula is a conjunction this is the correct value in 
	    // case there are no elements
	    Term accumulateAttributeFormulas = 
		TermFactory.DEFAULT.createJunctorTerm(Op.TRUE);
	    
	    ProgramVariable currAttribute = null;
	    
	    while (iterateAttributes.hasNext()){
		currAttribute = (ProgramVariable) 
		    iterateAttributes.next().getProgramVariable();

		accumulateAttributeFormulas = 
		    TermFactory.DEFAULT.createJunctorTermAndSimplify
		    (Op.AND, 
		     createAttributeFormula(currAttribute, currClass, oprime),
		     accumulateAttributeFormulas);
	    }

	    classSubFormula = accumulateAttributeFormulas;
	    
	}else {
	    // maybe check whether it is an ArrayDeclaration with test below:
	    // check it; may be artificial nulltype
	    if (currClassType instanceof ArrayDeclaration) {
		classSubFormula = createArrayFormula(currClass, oprime);
	    } else {
		// is this correct?
		classSubFormula = TermFactory.DEFAULT.createJunctorTerm(Op.TRUE);
	    }
	}

	TermFactory tf = TermFactory.DEFAULT;
	Term nullTerm = tf.createFunctionTerm(Op.NULL);
	Term ov = tf.createVariableTerm(oprime);
	Term nn = tf.createJunctorTerm(
	    Op.NOT, tf.createEqualityTerm(ov, nullTerm));
	ProgramVariable created = 
	    initConfig.getServices().getJavaInfo().getAttribute(
		"<created>", (ObjectSort) oprime.sort());
	Term eqc = tf.createAttributeTerm(created, ov);
	eqc = tf.createEqualityTerm(
	    eqc, initConfig.getServices().getTypeConverter().
	    convertToLogicElement(BooleanLiteral.TRUE));
	classSubFormula = tf.createJunctorTermAndSimplify(
	    Op.IMP, tf.createJunctorTerm(Op.AND, nn, eqc), classSubFormula);
	return classSubFormula;
    }
    
    
    private Term createAttributeFormula(ProgramVariable currAttribute,
					KeYJavaType currClass,
					LogicVariable oprime) {
	if (!currAttribute.isImplicit()){

	    // create conjunction of o' != o111, o' != o112, ... for non-static
	    // create "false" in case of a static attribute which is part of
	    // the modifies clause and "true" if it not part of the m.c.
	    
	    // some initial value
	    Term ante = TermFactory.DEFAULT.createJunctorTerm(Op.TRUE);
	    
	    Iterator modifiesIterator = modifiesElements.iterator();
	    ProgramElement modElm;
	    ProgramElement modProgVar;
	    ProgramElement modProgAttrib;
	    TypeConverter typeConv = 
		initConfig.getServices().getTypeConverter();
	    while (modifiesIterator.hasNext()) {
		modElm = (ProgramElement) modifiesIterator.next();
		
		//check whether it is appropriate to add this to the ante
		modProgVar = modElm;		

		if (! (modProgVar instanceof ProgramVariable) & 
		    ! (modProgVar instanceof ArrayReference)){
		    modProgAttrib = 
			(ProgramVariable) ((FieldReference) modProgVar)
			.getChildAt(((FieldReference) modProgVar)
				    .getChildCount()-1);
		    modProgVar = 
			((FieldReference) modProgVar).getReferencePrefix();
		    
		    KeYJavaType classType = null;
		    // static attribute
		    if (modProgVar instanceof TypeReference) {
			// Check if the attributes match
			if (((ProgramVariable) modProgAttrib)==currAttribute){
			    // build formula
			    ante = TermFactory.DEFAULT.
				createJunctorTerm(Op.FALSE);
			}
		    }else {
			classType = ((Expression) modProgVar).
			    getKeYJavaType(initConfig
					   .getServices(), null);
			
			// Check if the Class Types match
			if (classType == currClass) {
			    // Check if the attributes match
			    if (((ProgramVariable) modProgAttrib) == currAttribute){
				// build formula
				ante = TermFactory.DEFAULT.
				    createJunctorTermAndSimplify
				    (Op.AND,
				     TermFactory.DEFAULT.
				     createJunctorTerm
				     (Op.NOT,
				      TermFactory.DEFAULT.createEqualityTerm
				      (Op.EQUALS,
				       TermFactory.DEFAULT.
				       createVariableTerm(oprime),
				       (Term) typeConv.convertToLogicElement
				       ((ProgramElement) modProgVar))),
				     ante);
			    }
			}
		    }
		}
		else {
		    if (modProgVar instanceof ProgramVariable) {
		    // The modifies element has been a single ProgramVariable,
		    // meaning no dereferencing has happened. In this case there
		    // is nothing that can be done because this shouldn't happen
		    // Only attributes can be changed and thus only those are 
		    // relevant in the modifies clause.
		    // THROW EXCEPTION HERE !!!
		    System.out.println("Error - Element with no dereferencing "+
				       "in the modifies clause: " + modProgVar);
		    }
		    if (modProgVar instanceof ArrayReference) {
			// nothing to do here as an array element is not
			// relevant for a normal attribute
		    }
		}	
	    }
	    
	    // create part after the implication arrow
	    LogicVariable x  = new LogicVariable
		(new Name("x"), 
		 currAttribute.getKeYJavaType().getSort());
	    
	    // Create the MethodReference belonging to the method whose modifies
	    // clause is being checked here.
	    Expression[] p = new Expression[pvVector.size()];
	    for (int i = 0; i < pvVector.size(); i = i+1) {
		p[i] = (ProgramVariable) pvVector.elementAt(i);
	    }
	   
	    StatementBlock mBlock = createMethodCall(p);
	    
	    Term post = TermFactory.DEFAULT.createQuantifierTerm
		(Op.ALL, x, 
		 TermFactory.DEFAULT.createJunctorTermAndSimplify
		 (Op.IMP,
		  TermFactory.DEFAULT.createEqualityTerm
		  (Op.EQUALS,
		   TermFactory.DEFAULT.createVariableTerm(x),
		   TermFactory.DEFAULT.createAttributeTerm
		   (currAttribute, 
		    TermFactory.DEFAULT.createVariableTerm(oprime))), 
		  TermFactory.DEFAULT.createBoxTerm
		  (JavaBlock.createJavaBlock(mBlock), 
		   TermFactory.DEFAULT.createEqualityTerm
		   (Op.EQUALS, 
		    TermFactory.DEFAULT.createVariableTerm(x), 
		    TermFactory.DEFAULT.createAttributeTerm
		    (currAttribute, 
		     TermFactory.DEFAULT.createVariableTerm(oprime))))));
	    
	    return TermFactory.DEFAULT.
		createJunctorTermAndSimplify(Op.IMP, ante, post);
	}
	else {
	    return TermFactory.DEFAULT.createJunctorTerm(Op.TRUE);
	}
    }
    
    
    private Term createArrayFormula(KeYJavaType currClass,
				    LogicVariable oprime) {
	// create conjunction of o' != o111, o' != o112, ... 
	// all this with a "check" before it for the right array index
	
	TypeConverter typeConv = 
	    initConfig.getServices().getTypeConverter();
	
	LogicVariable lambda  = new LogicVariable
	    (new Name("lambda"), 
	     typeConv.getIntegerLDT().targetSort()); 

	Type currAttribute = ((ArrayDeclaration) (currClass.getJavaType()))
	    .getBaseType().getKeYJavaType();
	
	// initial value true, as it is the value of an empty conjunction
	// and our terms are connected conjunctively
	Term ante = 
	    TermFactory.DEFAULT.createJunctorTerm(Op.TRUE);
	
	Iterator modifiesIterator = modifiesElements.iterator();
	ProgramElement modElm;
	ProgramElement modProgVar;
	ProgramElement modProgAttrib;
	while (modifiesIterator.hasNext()) {
	    modElm = (ProgramElement) modifiesIterator.next();
	    
	    modProgVar = modElm;
	    
	    //check whether it is appropriate to add this to the ante
	    if (modProgVar instanceof ArrayReference){
		if ( typeConv.getKeYJavaType((Expression) ((ArrayReference) modProgVar).getReferencePrefix())
		     == 
		     currClass) {
		    // index into the array
		    modProgAttrib = 
		    (Expression) ((ArrayReference) modProgVar)
		    .getChildAt(((ArrayReference) modProgVar)
				.getChildCount()-1); 
		    
		    modProgVar = (Expression) ((ArrayReference) modProgVar)
			.getChildAt(0);
		    
		    // modProgAttrib(=z_j) = lambda 
		    Term preCond = TermFactory.DEFAULT.createEqualityTerm
			(Op.EQUALS,
			 (Term) typeConv.convertToLogicElement
			 ((ProgramElement) modProgAttrib), 
			 TermFactory.DEFAULT.createVariableTerm(lambda));
		    
		    // o' != o_ij
		    Term postCond = TermFactory.DEFAULT.
			 createJunctorTerm
			 (Op.NOT,
			  TermFactory.DEFAULT.createEqualityTerm
			  (Op.EQUALS,
			   TermFactory.DEFAULT.
			   createVariableTerm(oprime),
			   (Term) typeConv.convertToLogicElement
			   ((ProgramElement) modProgVar)));
		    
		    Term cond = TermFactory.DEFAULT.createJunctorTermAndSimplify
			(Op.IMP,
			 preCond,
			 postCond);
		    
		    // build parts together into the ante formula
		    ante = TermFactory.DEFAULT.
			createJunctorTermAndSimplify
			(Op.AND,
			 cond,
			 ante); 
		}
	    }
	    else {
		// The modifies element has not been an ArrayReference,
		// that means it has nothing to do with the current class/array
		// as that is an array and we can thus ignore this element of
		// the modifies clause
	    }	
	}
	
	// create part after the outer implication arrow
	LogicVariable x  = new LogicVariable
	    (new Name("x"), 
	     ((ArrayDeclaration) currClass.getJavaType()).getBaseType().getKeYJavaType().getSort()
	     );
	
	// Create the MethodReference belonging to the method whose modifies
	// clause is being checked here.
	Expression[] p = new Expression[pvVector.size()];
	for (int i = 0; i < pvVector.size(); i = i+1) {
	    p[i] = (ProgramVariable) pvVector.elementAt(i);
	}
	
	StatementBlock mBlock = createMethodCall(p);
		
	Term equality = TermFactory.DEFAULT.createEqualityTerm
          (Op.EQUALS,
           TermFactory.DEFAULT.createVariableTerm(x),
           TermFactory.DEFAULT.createArrayTerm 
           (ArrayOp.getArrayOp(oprime.sort()), 
                 TermFactory.DEFAULT.createVariableTerm(oprime),
           TermFactory.DEFAULT.createVariableTerm(lambda))
          );
    
	Term post = TermFactory.DEFAULT.createQuantifierTerm
	    (Op.ALL, x, 
	     TermFactory.DEFAULT.createJunctorTermAndSimplify
	     (Op.IMP, equality,  
	      TermFactory.DEFAULT.createBoxTerm
	      (JavaBlock.createJavaBlock(mBlock), equality)));

	
	Term arrayFormula = TermFactory.DEFAULT.
	    createJunctorTermAndSimplify(Op.IMP, ante, post);

	
	// helpers for the next block below
	Function leq =typeConv.getIntegerLDT().getFunctionFor
		(new LessOrEquals(null, null));
	Function lt
	    = typeConv.getIntegerLDT().getFunctionFor(new LessThan(null, null));
	Term zero = 
	    typeConv.getIntegerLDT().translateLiteral(new IntLiteral(0));
	Term validArrayTerm0 = TermFactory.DEFAULT.createFunctionTerm
	    (leq, zero, TermFactory.DEFAULT.createVariableTerm(lambda));
	ProgramVariable lengthPV = (ProgramVariable) 
	    ((ArrayDeclaration) currClass.getJavaType()).length()
	    .getFieldSpecifications().getFieldSpecification(0)
	    .getProgramVariable();
	Term lengthTerm = TermFactory.DEFAULT.createAttributeTerm
	    (lengthPV, TermFactory.DEFAULT.createVariableTerm(oprime));
	Term validArrayTerm1 = TermFactory.DEFAULT.createFunctionTerm
	    (lt, TermFactory.DEFAULT.createVariableTerm(lambda), lengthTerm);

	// creates the 0<= lambda /\ lambda < o'.length() -> rest
	arrayFormula = TermFactory.DEFAULT.createJunctorTermAndSimplify
	    (Op.IMP,
	     TermFactory.DEFAULT.createJunctorTermAndSimplify
	     (Op.AND,
	      validArrayTerm0,
	      validArrayTerm1), 
	     arrayFormula);
	
	// generate formula starting with \forall \lambda: int
	arrayFormula = TermFactory.DEFAULT.createQuantifierTerm
	    (Op.ALL,
	     lambda,
	     arrayFormula);
	return arrayFormula;
    }

    private StatementBlock createMethodCall(Expression[] p){
	if(m.isConstructor()){
	    TypeRef prefix = new TypeRef(self.getKeYJavaType());
	    New n = new New(p,
			    prefix,
			    prefix);
	    return new StatementBlock(n);
	}else{
	    MethodBodyStatement mBS = new MethodBodyStatement
		(m,
		 self,
		 null,
		 new ArrayOfExpression(p));
	    return new StatementBlock(mBS);
	}
    }
    
    
    public ProofList getPO() {
        if (proofObl == null) throw 
           new IllegalStateException("No proofObl term");
        String s = createProofHeader(methRepr.getContainingClass()
				     .getRootDirectory());
	return new ProofList(
	    new Proof(name(), proofObl, s, 
		      initConfig.createTacletIndex(),
		      initConfig.createBuiltInRuleIndex()),
	    name());
    }

    /** creates declarations necessary to save/load proof in textual form */
    String createProofHeader(String javaPath) {
        String s;
        s = "java {\""+javaPath+"\"}\n\n";
        s += "sorts {\n"; // create declaration header for the proof
        IteratorOfNamed it;
        it = initConfig.sortNS().getProtocolled();
        while (it.hasNext()) {
	    String n=it.next().toString();
	    int i;
	    if ((i=n.indexOf("."))<0 || 
		initConfig.sortNS().lookup(new Name(n.substring(0,i)))==null) {
		//the line above is for inner classes.
		//KeY does not want to handle them anyway...
		s = s+"object "+n+";\n";
	    }
	}

        s+="}\n\nprogram variables {\n";
        it = initConfig.progVarNS().getProtocolled();
        while (it.hasNext()) 
           s = s+((ProgramVariable)(it.next())).proofToString();

        s+="}\n\nfunctions {\n";
        it = initConfig.funcNS().getProtocolled();
        while (it.hasNext()) s = s+((Function)(it.next())).proofToString();

        s+="}\n\n";
        return s;
    }

    /** sets the initial configuration that is used to read the OCL
     * input and that is used to be modified during reading.
     */
    public void setConfig(InitConfig conf) {
	this.initConfig=conf;
        initConfig.sortNS().startProtocol();
        initConfig.funcNS().startProtocol();
        initConfig.progVarNS().startProtocol();
    }

    /** returns the name of the modifies check proof obligation input.
     */
    public String name() {
	return name;
    }

    public void readActivatedChoices() throws ProofInputException {
	initConfig.setActivatedChoices(SetAsListOfChoice.EMPTY_SET);
    }

    public SetOfChoice getActivatedChoices() 
	throws ProofInputException{
	return SetAsListOfChoice.EMPTY_SET;
    }
    
    /** reads the include section and returns an Includes object.  
     */
    public Includes readIncludes() throws ProofInputException {
	RuleSource standard = RuleSource.initRuleFile("standardRules.key");
	Includes includes = new Includes();
	includes.put("standardRules", standard);
	return includes;
    }

    public void readSpecs(){
	// nothing here
    }

    public Term getPOTerm() {
	return proofObl;
    }
    
    
    public Set getModifiesElements() {
	return modifiesElements;
    }

    public Object getObjectOfContract() {
	return methRepr;
    }

    public boolean fitsToContract(Contract ct) {
	return false; //%%
    }

    public void startProtocol() {
	// do nothing
    }

    
}
