// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.proof.mgt;

import java.util.*;
import de.uka.ilkd.key.proof.init.InitConfig;
import de.uka.ilkd.key.parser.jml.*;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.sort.*;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.java.declaration.*;
import de.uka.ilkd.key.jml.*;
import de.uka.ilkd.key.rule.*;

public class JMLMethodLemmaFactory extends AbstractMethodLemmaFactory {

    private InitConfig initConfig;
    // a set of JMLMethodSpecs
    private JMLMethodSpec spec = null;
    private JMLLemmaMethodSpec lemmaspec = null;
    private String suffix = null;
    private ProgramMethod pm;
    private static int mm_count = 0;
    private SchemaVariable substForMM;
    private Modality modality;

    public JMLMethodLemmaFactory(InitConfig initConfig, JMLMethodSpec spec, 
				 String suffix, Modality modality) {
	super(initConfig.getServices());
	this.initConfig = initConfig;	
	this.spec = spec;
	this.suffix = suffix;
	this.modality = modality;
	if(spec instanceof JMLLemmaMethodSpec){
	    lemmaspec = (JMLLemmaMethodSpec) spec;
	}
    }

    protected Modality getModality() {
	return modality;
    }

    public Taclet getTaclet() {
	if(lemmaspec != null && lemmaspec.terminates()){
	    return super.getTaclet();
	}else{
	    return null;
	}
    }

    protected RuleSet getRuleSet() {
	return (RuleSet)getInitConfig().ruleSetNS().lookup
	    (new Name("use_method_specs"));
    }

    public InitConfig getInitConfig(){
	return initConfig;
    }

    /** precondition which is called "PRE" by Sebastian
     * Katz's Studienarbeit */
    protected Term getPreCond(){
	return lemmaspec.getCompletePre(true, JMLMethodContract.allInvariants);
    } 
    
    /** extra quantifiable variables created by the transformation necessary
     * after the Studienarbeit of Sebastian Katz */
    protected ListOfQuantifierPrefixEntry getExtraVars(){
	return lemmaspec.getOldLVs();
    } 

    protected ListOfQuantifierPrefixEntry getQuantifierPrefix () {
        return SLListOfQuantifierPrefixEntry.EMPTY_LIST;
    }

    /** extra precondtion created by the transformation necessary
     * after the Studienarbeit of Sebastian Katz 
     * there this is called "pi1" */
    protected Term getExtraPre(){
	return lemmaspec.getPi1();
    }
    
    /** post condition of the method spec
     * this is called "pi2" in Sebastian Katz's Studienarbeit */
    protected Term getPostCond(){
	return lemmaspec.getCompletePost(true,JMLMethodContract.allInvariants);
    } 

    /** modifies elements as a set of terms */
    protected SetOfTerm getModifies(){
	return lemmaspec.replaceModelFieldsInAssignable();	
    }

    /** statement before the transformation so we can get the real variables (no schema variables) */
    protected ProgramElement getProofStatement(){
	return lemmaspec.getProofStatement();
    }
    
    /** string which is printed out a couple of times and given in the
     * constructor of the classes that extend this abstract class */
    protected String getSuffix(){
	return suffix;
    }

}
