// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.proof.mgt;

import java.util.*;
import de.uka.ilkd.key.proof.ProofList;

/** The repository where all specification (contracts) of a proof
 * environment are contained. An instance must belong to only one
 * {@link ProofEnvironment}.
 */
public class SpecificationRepository {
    
    private Map contracts = new HashMap(); 
    private ProofEnvironment env;
        
    /** returns an iterator of specifications ({@link Contract})
     * contained in this repository. Modifications to the underlying
     * set are not allowed.
     */
    public Iterator getSpecs() {
	return contracts.values().iterator();
    }

    /** returns a set of contracts with the argument as identifying
     * contracted object.
     */
    public ContractSet getContract(Object objOfContract) {
	return (ContractSet) contracts.get(objOfContract);
    }

    /** adds a contract to this specification repository.
     */
    public void add(Contract ct) {
	Object o = ct.getObjectOfContract();
	ContractSet cs =(ContractSet) contracts.get(o);
	if (cs==null) {
	    cs = new ContractSet();
	    contracts.put(o, cs);
	}	
	cs.add(ct);
    }

    public void removeProofList(ProofList pl) {
	Iterator it = getSpecs();
	while (it.hasNext()) {
	    Object o = it.next();
	    ((ContractSet) o).removeProofList(pl);
	}
    }

}

