// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.rule.updatesimplifier;

import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.op.ArrayOp;
import de.uka.ilkd.key.logic.op.ShadowedOperator;
import de.uka.ilkd.key.rule.UpdateSimplifier;

/**
 * Applies an update on an array using rule ...
 */
public class ApplyOnArray extends ApplyOnArrayAccess {
       
    /**
     * creates an instance of this rule used by the given update
     * simplifier
     */
    public ApplyOnArray(UpdateSimplifier us) {
	super(us);
    }

    /**
     * determines whether this rule is applicable for the 
     * update x target (the term on which the update will be
     * applied) pair 
     * @param update the Update to be applied on target
     * @param target the Term on which the update is applied
     * @return true if the top level operator of target is an
     * attribute operator
     */
    public boolean isApplicable(Update update, Term target) {
	return target.op() instanceof ArrayOp && 
		!(target.op() instanceof ShadowedOperator);
    }

    /**
     * (non-Javadoc)
     * @see de.uka.ilkd.key.rule.updatesimplifier.ApplyOnAttributeAccess#createAliasResolutionTerm(de.uka.ilkd.key.logic.Term[], de.uka.ilkd.key.rule.updatesimplifier.ArrayOfAssignmentPair, de.uka.ilkd.key.logic.Term)
     */
    protected Term createAliasResolutionTerm(Term[] locSubs, 
            ArrayOfAssignmentPair mayBeAliases, Term simplifiedLocation) {
        Term result = simplifiedLocation;        
        if (mayBeAliases.size() > 0) {
            result = UpdateSimplifierTermFactory.DEFAULT.
            createConjunctionConditionalStarTerm(locSubs, 
                    mayBeAliases, result);
        } 
        return result;
    }
}
