// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.rule.updatesimplifier;

import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.op.AttributeOp;
import de.uka.ilkd.key.logic.op.ShadowedOperator;
import de.uka.ilkd.key.rule.UpdateSimplifier;

/**
 * Rule used by the update simplifier as application of an update on
 * a local variable. 
 * Implements rule (S3) described in ....
 */
public class ApplyOnInstanceAttribute extends ApplyOnAttributeAccess {
       
    /**
     * creates an instance of this rule used by the given update
     * simplifier
     */
    public ApplyOnInstanceAttribute(UpdateSimplifier us) {
	super(us);
    }

    /**
     * determines whether this rule is applicable for the 
     * update x target (the term on which the update will be
     * applied) pair 
     * @param update the Update to be applied on target
     * @param target the Term on which the update is applied
     * @return true if the top level operator of target is an
     * attribute operator
     */
    public boolean isApplicable(Update update, Term target) {
	return target.op() instanceof AttributeOp &&
	    !(target.op() instanceof ShadowedOperator);
    }    

    /**
     *  creates the result term of the simplification 
     *  
     * @param locSubs an array of Terms which contains the simplified sub terms 
     * of the attribute on which the update is applied
     * @param mayBeAliases an ArrayOfAssignmentPair of the candidates which may 
     * be aliased to the original location 
     * 
     * @return the term describing the simplified value after the application of
     *  the update on the original attribute containing  
     */
    protected Term createAliasResolutionTerm(Term[] locSubs, 
                                             ArrayOfAssignmentPair mayBeAliases,
                                             Term simplifiedLocation) {
        if (mayBeAliases.size() == 0) {
            return simplifiedLocation;
        }
        return UpdateSimplifierTermFactory.DEFAULT.
           createConditionalStarTermAndSimplify(locSubs[0], mayBeAliases,
                                                simplifiedLocation);
    }
}
