// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
/*
 * Created on 18.12.2004
 */
package de.uka.ilkd.key.rule.updatesimplifier;

import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.TermFactory;
import de.uka.ilkd.key.logic.op.ShadowArrayOp;
import de.uka.ilkd.key.logic.op.ShadowedOperator;
import de.uka.ilkd.key.rule.UpdateSimplifier;

/**
 * Update simplification rule for update application on a shadowed array 
 * access operator
 * 
 * @author bubel
 */
public class ApplyOnShadowedArray extends ApplyOnArrayAccess {

    /**
     * @param updateSimplifier
     */
    public ApplyOnShadowedArray(UpdateSimplifier updateSimplifier) {
        super(updateSimplifier);
    }

    /**
     *  (non-Javadoc)
     * @see de.uka.ilkd.key.rule.updatesimplifier.ApplyOnAccessTerm#createAliasResolutionTerm(de.uka.ilkd.key.logic.Term[], de.uka.ilkd.key.rule.updatesimplifier.ArrayOfAssignmentPair, de.uka.ilkd.key.logic.Term)
     */
    protected Term createAliasResolutionTerm(Term[] locSubs,
            ArrayOfAssignmentPair mayBeAliases, Term simplifiedLocation) {
        if (mayBeAliases.size() == 0) {
            return simplifiedLocation;
        }

        Term result = simplifiedLocation;
        final TermFactory tf = TermFactory.DEFAULT;
        
        final Term[] unshadowedArrayLocs = new Term[2]; 
      
        for (int j = 0; j<unshadowedArrayLocs.length; j++) {
            unshadowedArrayLocs[j] = locSubs[j];
        }
        for (int i = 0; i<mayBeAliases.size(); i++) {
            AssignmentPair mayBeAlias = mayBeAliases.getAssignmentPair(i);                        
            if (mayBeAlias.location() instanceof ShadowedOperator) { 
                final Term[] mayBeAliasSubs = mayBeAlias.locationSubs();
                final Term[] unshadowedAliasLocs = new Term[2];
                for (int j = 0; j<unshadowedAliasLocs.length; j++) {                  
                    unshadowedAliasLocs[j] = mayBeAliasSubs[j];
                }
                // the transaction numbers have to be checked as well  
                result = tf.createConjConditionalTermAndSimplify
                (unshadowedArrayLocs, unshadowedAliasLocs, 
                        tf.createIfElseTermAndSimplify
                        (new Term[]{locSubs[2], mayBeAliasSubs[2], 
                                mayBeAlias.value(), result}), 
                                result);
            } else {
                result = tf.createConjConditionalTermAndSimplify(
                        unshadowedArrayLocs, 
                        mayBeAlias.locationSubs(),
                        mayBeAlias.value(), result);
            }
        } 
        return result;
    }

    /**
     * this rule is applicable if the target represents a schadowed array access
     */
    public boolean isApplicable(Update update, Term target) {       
        return target.op() instanceof ShadowArrayOp;
    }

}
