// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
/*
 * Created on 18.12.2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package de.uka.ilkd.key.rule.updatesimplifier;

import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.TermFactory;
import de.uka.ilkd.key.logic.op.AttributeOp;
import de.uka.ilkd.key.logic.op.ShadowedOperator;
import de.uka.ilkd.key.rule.UpdateSimplifier;

/**
 * This rule applies an update on a shadowed attribute. 
 * @author bubel
 */
public class ApplyOnShadowedInstanceAttribute 
    extends ApplyOnAttributeAccess {

    /**
     * creates an instance of this rule used by the given update simplifier
     */
    public ApplyOnShadowedInstanceAttribute(UpdateSimplifier updateSimplifier) {
        super(updateSimplifier);
    }

    /**
     * determines whether this rule is applicable     
     * @param update
     *            the Update to be applied on target
     * @param target
     *            the Term on which the update is applied
     * @return true if the top level operator of target is an attribute operator
     */
    public boolean isApplicable(Update update, Term target) {
        return target.op() instanceof AttributeOp
                && target.op() instanceof ShadowedOperator;
    }

    /**
     * creates the result term of the simplification
     * 
     * @param locSubs
     *            an array of Terms which contains the simplified sub terms of
     *            the attribute on which the update is applied
     * @param mayBeAliases
     *            an ArrayOfAssignmentPair of the candidates which may be
     *            aliased to the original location
     * 
     * @return the term describing the simplified value after the application of
     *         the update on the original attribute containing
     */
    protected Term createAliasResolutionTerm(Term[] locSubs,
            ArrayOfAssignmentPair mayBeAliases, Term simplifiedLocation) {
               
        if (mayBeAliases.size() == 0) {
            return simplifiedLocation;
        }

        Term result = simplifiedLocation;
                
        for (int i = 0; i<mayBeAliases.size(); i++) {
            AssignmentPair mayBeAlias = mayBeAliases.getAssignmentPair(i);
            
            if (mayBeAlias.location() instanceof ShadowedOperator) { 
                // in this case a conjunction of conditions has to be used
                // as the transaction number needs to be checked as well  
                result = TermFactory.DEFAULT.
                    createConjConditionalTermAndSimplify
                      (locSubs, mayBeAlias.locationSubs(), 
                       mayBeAlias.value(), result);
            } else {
                result = TermFactory.DEFAULT.
                    createIfElseTermAndSimplify(new Term[] 
                           {locSubs[0], 
                            mayBeAlias.locationSubs()[0],
                            mayBeAlias.value(), result});
            }
        }
        
        return result;
    }
}
