// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.rule.updatesimplifier;

import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.op.*;

/**
 * Models an assignment pair <code> l_i := t_i </code>  of an update.
 * This data structure is only used for update simplification.
 */
public interface AssignmentPair {
    

    /**
     * @author bubel
     *
     * TODO To change the template for this generated type comment go to
     * Window - Preferences - Java - Code Style - Code Templates
     */
    public static class LocationAsKey {
        final AssignmentPair pair;
        
        public LocationAsKey(AssignmentPair pair) {        
            this.pair = pair;
        }
        
        public int hashCode() {
            return pair.locationHashCode();
        }
        
        public boolean equals(Object o) {     
            if (!(o instanceof LocationAsKey)) return false;
            return pair.equalLocations(((LocationAsKey)o).pair);
        }
        
        public AssignmentPair getAssignmentPair() {
            return pair;
        }          
    }
    
    /**
     * returns the location
     * @return the left side of the assignment
     */
    Term locationAsTerm();

    /**
     * returns the location operator
     * @return the location specifying operator
     */
    Location location();

    /**
     * returns the locations sub terms
     * @return the left side of the assignment
     */
    Term[] locationSubs();

    /** 
     * returns the value that is assigned to the location
     * @return the right side of the assignment      
     */
    Term value();

    /**
     * compares the location of the given assignment pair with the
     * current location and returns true if they are equal
     * @param p the AssignmentPair whose location is compared
     * @return true if the location that is updated is equal to the location
     * of the given assignment pair
     */
    boolean equalLocations(AssignmentPair p);

    /**
     * returns an int fulfilling the usual hashcode properties but
     * without consideration of the value part of the assignment pair
     * @return an int as location hashcode 
     */
    int locationHashCode();

}
