// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.strategy.feature;

import de.uka.ilkd.key.logic.PosInOccurrence;
import de.uka.ilkd.key.proof.Goal;
import de.uka.ilkd.key.rule.IteratorOfIfFormulaInstantiation;
import de.uka.ilkd.key.rule.IteratorOfRuleApp;
import de.uka.ilkd.key.rule.ListOfRuleApp;
import de.uka.ilkd.key.rule.PosTacletApp;
import de.uka.ilkd.key.rule.RuleApp;
import de.uka.ilkd.key.rule.TacletApp;

/**
 * Binary feature that returns zero iff a certain Taclet app has not already
 * been performed
 */
public class NonDuplicateAppFeature extends BinaryTacletAppFeature {

    public static final Feature INSTANCE = new NonDuplicateAppFeature ();

    private static boolean actuallyContains (ListOfRuleApp list,
                                             TacletApp rapp,
                                             PosInOccurrence pio) {

        final IteratorOfRuleApp it = list.iterator ();
        historyLoop: while (it.hasNext ()) {
            final RuleApp ruleCmp = it.next ();

            // compare the rules
            if (rapp.rule () != ruleCmp.rule ()) continue;

            final TacletApp cmp = (TacletApp) ruleCmp;

            // compare the position of application
            if (pio != null) {
                if (!(cmp instanceof PosTacletApp)) continue;
                final PosTacletApp cmp2 = (PosTacletApp) cmp;
                if (!cmp2.posInOccurrence ().equals ( pio )) continue;
            }

            // compare the if-sequent instantiations
            if (rapp.ifFormulaInstantiations () == null
                    && cmp.ifFormulaInstantiations () == null) return true;

            if (rapp.ifFormulaInstantiations () != null
                    && cmp.ifFormulaInstantiations () != null) {
                final IteratorOfIfFormulaInstantiation it0 = rapp
                        .ifFormulaInstantiations ().iterator ();
                final IteratorOfIfFormulaInstantiation it1 = cmp
                        .ifFormulaInstantiations ().iterator ();

                while ( it0.hasNext () ) {
                    // this test should be improved
                    if ( it0.next ().getConstrainedFormula () !=
                         it1.next ().getConstrainedFormula () )
                        continue historyLoop;
                }

                return true;
            }
        }

        return false;
    }

    public boolean filter ( TacletApp app, PosInOccurrence pos, Goal goal ) {
	if ( !app.ifInstsComplete () )
	    return true;
        return !actuallyContains ( goal.appliedRuleApps (), app, pos );
    }
}
